####################################################################################################
#
# Now available: developmental python x3d.py package on PyPI for import.
#   This approach simplifies Python X3D deployment and use.
#   https://pypi.org/project/x3d
#
# Installation:
#       pip install x3d
# or
#       python -m pip install x3d
#
# Developer options for loading x3d package:
#
#    from x3d import *  # preferred approach, terser source that avoids x3d.* class prefixes
#
# or
#    import x3d         # traditional way to subclass x3d package, all classes require x3d.* prefix,
#                       # but python source is very verbose, for example x3d.Material x3d.Shape etc.
#                       # X3dToPython.xslt stylesheet insertPackagePrefix=true supports this option.
#
####################################################################################################

from x3d import *

newModel=X3D(profile='Immersive',version='3.3',
  head=head(
    children=[
    meta(content='PositionInterpolatorPrototype.x3d',name='title'),
    meta(content='This is a prototype for a CircleFish, which is also used by CircleFishLod. Default fish size: length is 1 m (Y-axis), max Height is 1 m (Z-axis), max Width is 10 cm (X-axis).',name='description'),
    meta(content='Ildeniz Duman',name='creator'),
    meta(content='Scott Tufts',name='translator'),
    meta(content='1 May 1998',name='created'),
    meta(content='11 December 2001',name='translated'),
    meta(content='20 October 2019',name='modified'),
    meta(content='https://X3dGraphics.com/examples/X3dForWebAuthors/KelpForestExhibit/CircleFishPrototype.x3d',name='reference'),
    meta(content='https://X3dGraphics.com/examples/X3dForWebAuthors/Chapter07EventAnimationInterpolation/PositionInterpolatorPrototype.x3d',name='identifier'),
    meta(content='X3D-Edit 3.3, https://savage.nps.edu/X3D-Edit',name='generator'),
    meta(content='../license.html',name='license')]),
  Scene=Scene(
    children=[
    WorldInfo(title='PositionInterpolatorPrototype.x3d'),
    ProtoDeclare(name='CircleFish',
      ProtoInterface=ProtoInterface(
        field=[
        field(accessType='inputOutput',name='skinTextureUrl',type='MFString'),
        field(accessType='inputOutput',name='fishScale',type='SFVec3f',value=(1,1,1)),
        field(accessType='inputOutput',name='fishColor',type='SFColor',value=(0.5,0.5,0.5)),
        field(accessType='inputOutput',name='finColor',type='SFColor',value=(0.5,0.5,0.5)),
        field(accessType='inputOutput',name='positionKey',type='MFFloat',value=[0.0,0.125,0.25,0.375,0.5,0.625,0.750,0.8751]),
        field(accessType='inputOutput',name='positionKeyValue',type='MFVec3f',value=[(1.5,2,0),(1.5,0,-1.0),(0,-1,-1.0),(-1.5,0,-1.0),(-1.5,2,0),(-1.5,0,1.0),(0,-2,1.0),(1.5,0,1.0),(1.5,2,0)]),
        field(accessType='inputOutput',name='orientationKey',type='MFFloat',value=[0.0,0.25,0.5,0.750,1]),
        field(accessType='inputOutput',name='orientationKeyValue',type='MFRotation',value=[(0,1,0,0),(0,1,0,-1.57),(0,1,0,-3.14),(0,1,0,-4.71),(0,1,0,-6.28)]),
        field(accessType='inputOutput',name='orbitTime',type='SFTime',value=90)]),
      ProtoBody=ProtoBody(
        children=[
        Transform(DEF='Way',
          children=[
          Transform(rotation=(1,0,0,1.395),
            children=[
            Transform(DEF='FishTransform',
              IS=IS(
                connect=[
                connect(nodeField='scale',protoField='fishScale')]),
              children=[
              Shape(
                appearance=Appearance(
                  material=Material(DEF='FishColor',
                    IS=IS(
                      connect=[
                      connect(nodeField='diffuseColor',protoField='fishColor')])),
                  texture=ImageTexture(DEF='FishTexture',
                    IS=IS(
                      connect=[
                      connect(nodeField='url',protoField='skinTextureUrl')]))),
                geometry=Extrusion(DEF='FishBodyExtrusion',creaseAngle=0.57,crossSection=[(-0.1,1.0),(0.1,1),(0.5,0.5),(0.5,-0.5),(0.1,-1),(-0.1,-1),(-0.5,-0.5),(-0.5,0.5),(-0.1,1)],scale=[(0.03,0.03),(0.05,0.3),(0.05,0.45),(0.1,0.5),(0.05,0.45),(0.05,0.2),(0.03,0.4)],spine=[(0,1,0),(0,0.8,0),(0,0.6,0),(0,0.5,0),(0,0.3,0),(0,0.2,0),(0,0,0)])),
              Shape(
                appearance=Appearance(DEF='FinSkin',
                  material=Material(DEF='FinColor',transparency=0.2,
                    IS=IS(
                      connect=[
                      connect(nodeField='diffuseColor',protoField='finColor')])))),
              Transform(translation=(0.015,0.8,-0.2),
                children=[
                Shape(DEF='Eye',
                  appearance=Appearance(
                    material=Material(diffuseColor=(0.5,0.5,0.5),shininess=1,specularColor=(0.3,0.3,0.3))),
                  geometry=Sphere(radius=0.03))]),
              Transform(translation=(-0.015,0.8,-0.2),
                children=[
                Shape(USE='Eye')]),
              Transform(scale=(1,1,0.7),translation=(0,0.98,0),
                children=[
                Shape(USE='Eye')]),
              Transform(DEF='LeftPectoral',translation=(0.04,0.4,0.3),
                children=[
                Shape(
                  appearance=Appearance(USE='FinSkin'),
                  geometry=IndexedFaceSet(coordIndex=[0,1,2,-1],solid=False,
                    coord=Coordinate(point=[(0,0,0),(0,0.2,0),(0.2,0.1,0)])))]),
              Transform(DEF='RightPectoral',translation=(-0.04,0.4,0.3),
                children=[
                Shape(
                  appearance=Appearance(USE='FinSkin'),
                  geometry=IndexedFaceSet(coordIndex=[0,1,2,-1],solid=False,
                    coord=Coordinate(point=[(0,0,0),(0,0.2,0),(-0.2,0.1,0)])))])])]),
          Group(DEF='AnimationGroup',
            children=[
            TimeSensor(DEF='OrbitClock',loop=True,
              IS=IS(
                connect=[
                connect(nodeField='cycleInterval',protoField='orbitTime')])),
            PositionInterpolator(DEF='CirclefishPath',
              IS=IS(
                connect=[
                connect(nodeField='key',protoField='positionKey'),
                connect(nodeField='keyValue',protoField='positionKeyValue')])),
            OrientationInterpolator(DEF='CirclefishRotations',
              IS=IS(
                connect=[
                connect(nodeField='key',protoField='orientationKey'),
                connect(nodeField='keyValue',protoField='orientationKeyValue')])),
            ROUTE(fromField='fraction_changed',fromNode='OrbitClock',toField='set_fraction',toNode='CirclefishPath'),
            ROUTE(fromField='value_changed',fromNode='CirclefishPath',toField='set_translation',toNode='Way'),
            ROUTE(fromField='fraction_changed',fromNode='OrbitClock',toField='set_fraction',toNode='CirclefishRotations'),
            ROUTE(fromField='value_changed',fromNode='CirclefishRotations',toField='set_rotation',toNode='Way')])])])),
    #  ==================== 
    ProtoInstance(name='CircleFish')])
) # X3D model complete

####################################################################################################
# Self-test diagnostics
####################################################################################################

print('Self-test diagnostics for PositionInterpolatorPrototype.py:')
if        metaDiagnostics(newModel): # built-in utility method in X3D class
    print(metaDiagnostics(newModel)) # display meta info, hint, warning, error, TODO values in this model
# print('check newModel.XML() serialization...')
newModelXML= newModel.XML() # test export method XML() for exceptions during export
newModel.XMLvalidate()
# print(newModelXML) # diagnostic

try:
#   print('check newModel.VRML() serialization...')
    newModelVRML=newModel.VRML() # test export method VRML() for exceptions during export
    # print(prependLineNumbers(newModelVRML)) # debug
    print("Python-to-VRML export of VRML output successful", flush=True)
except Exception as err: # usually BaseException
    # https://stackoverflow.com/questions/18176602/how-to-get-the-name-of-an-exception-that-was-caught-in-python
    print("*** Python-to-VRML export of VRML output failed:", type(err).__name__, err)
    if newModelVRML: # may have failed to generate
        print(prependLineNumbers(newModelVRML, err.lineno))

try:
#   print('check newModel.JSON() serialization...')
    newModelJSON=newModel.JSON() # test export method JSON() for exceptions during export
#   print(prependLineNumbers(newModelJSON)) # debug
    print("Python-to-JSON export of JSON output successful (under development)")
except Exception as err: # usually SyntaxError
    print("*** Python-to-JSON export of JSON output failed:", type(err).__name__, err)
    if newModelJSON: # may have failed to generate
        print(prependLineNumbers(newModelJSON,err.lineno))

print("python PositionInterpolatorPrototype.py load and self-test diagnostics complete.")
