####################################################################################################
#
# Now available: developmental python x3d.py package on PyPI for import.
#   This approach simplifies Python X3D deployment and use.
#   https://pypi.org/project/x3d
#
# Installation:
#       pip install x3d
# or
#       python -m pip install x3d
#
# Developer options for loading x3d package:
#
#    from x3d import *  # preferred approach, terser source that avoids x3d.* class prefixes
#
# or
#    import x3d         # traditional way to subclass x3d package, all classes require x3d.* prefix,
#                       # but python source is very verbose, for example x3d.Material x3d.Shape etc.
#                       # X3dToPython.xslt stylesheet insertPackagePrefix=true supports this option.
#
####################################################################################################

from x3d import *

newModel=X3D(profile='Immersive',version='3.3',
  head=head(
    children=[
    meta(content='SharkLefty.x3d',name='title'),
    meta(content='Lefty Shark, who has the red/gold NPS logo because he is proud of being a Marine animal.',name='description'),
    meta(content='Tim McLean',name='creator'),
    meta(content='Donald Julian',name='translator'),
    meta(content='Claude Hutton',name='contributor'),
    meta(content='1 June 1998',name='created'),
    meta(content='20 October 2019',name='modified'),
    meta(content='http://faculty.nps.edu/brutzman/kelp',name='reference'),
    meta(content='SharkDarkEyeNPS.jpg',name='Image'),
    meta(content='SharkDarkEye.jpg',name='Image'),
    meta(content='shark X3D',name='subject'),
    meta(content='https://X3dGraphics.com/examples/X3dForWebAuthors/KelpForestExhibit/SharkLefty.x3d',name='identifier'),
    meta(content='Vrml97ToX3dNist, http://ovrt.nist.gov/v2_x3d.html',name='generator'),
    meta(content='X3D-Edit 3.3, https://savage.nps.edu/X3D-Edit',name='generator'),
    meta(content='(originally) Published by Cosmo Worlds V2.0',name='generator'),
    meta(content='../license.html',name='license')]),
  Scene=Scene(
    children=[
    Viewpoint(DEF='behindShark',description='Behind Lefty Shark',orientation=(0.00390132,-0.999986,0.0034463,1.44711),position=(-1.5071,0.358747,0.17578)),
    Viewpoint(DEF='frontQuarterShark',description='In front of Lefty Shark',orientation=(0.120567,0.991301,-0.0527853,0.831805),position=(1.59628,-0.250648,0.925251)),
    LOD(range=[60,100],
      children=[
      Transform(center=(-0.00303999,0.898843,-0.091817),rotation=(0,1,0,1.57079),translation=(0.00303999,-0.898843,0.049247),
        children=[
        Shape(
          appearance=Appearance(
            material=Material(DEF='SharkMaterial',ambientIntensity=0.0534211,diffuseColor=(0.756863,0.729412,0.537255),emissiveColor=(0.18,0.17,0.13),shininess=0.19,specularColor=(0.38,0.38,0.27)),
            texture=ImageTexture(url=["SharkDarkEyeNPS.jpg","https://X3dGraphics.com/examples/X3dForWebAuthors/KelpForestExhibit/SharkDarkEyeNPS.jpg"])),
          geometry=IndexedFaceSet(DEF='_0',colorIndex=[0,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127,128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143,144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175,176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191,192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207,208,209,210,211,212,213,214,215,216,217,218,219,220,221,222,223,2,1,3,224,225],colorPerVertex=False,convex=False,coordIndex=[0,1,2,-1,12,13,14,-1,15,16,17,-1,18,19,20,-1,21,22,23,-1,24,25,26,-1,27,28,29,-1,30,31,32,-1,33,34,35,-1,36,37,38,-1,39,40,41,-1,42,43,44,-1,45,46,47,-1,48,49,50,-1,51,52,53,-1,54,55,56,-1,57,58,59,-1,60,61,62,-1,63,64,65,-1,66,67,68,-1,69,70,71,-1,72,73,74,-1,75,76,77,-1,78,79,80,-1,81,82,83,-1,84,85,86,-1,87,88,89,-1,90,91,92,-1,93,94,95,-1,96,97,98,-1,99,100,101,-1,102,103,104,-1,105,106,107,-1,108,109,110,-1,111,112,113,-1,114,115,116,-1,117,118,119,-1,120,121,122,-1,123,124,125,-1,126,127,128,-1,129,130,131,-1,132,133,134,-1,135,136,137,-1,138,139,140,-1,141,142,143,-1,144,145,146,-1,147,148,149,-1,150,151,152,-1,153,154,155,-1,156,157,158,-1,159,160,161,-1,162,163,164,-1,165,166,167,-1,168,169,170,-1,171,172,173,-1,174,175,176,-1,177,178,179,-1,180,181,182,-1,183,184,185,-1,186,187,188,-1,189,190,191,-1,192,193,194,-1,195,196,197,-1,198,199,200,-1,201,202,203,-1,204,205,206,-1,207,208,209,-1,210,211,212,-1,213,214,215,-1,216,217,218,-1,219,220,221,-1,222,223,224,-1,225,226,227,-1,228,229,230,-1,231,232,233,-1,234,235,236,-1,237,238,239,-1,240,241,242,-1,243,244,245,-1,246,247,248,-1,249,250,251,-1,252,253,254,-1,255,256,257,-1,258,259,260,-1,261,262,263,-1,264,265,266,-1,267,268,269,-1,270,271,272,-1,273,274,275,-1,276,277,278,-1,279,280,281,-1,282,283,284,-1,285,286,287,-1,288,289,290,-1,291,292,293,-1,294,295,296,-1,297,298,299,-1,300,301,302,-1,303,304,305,-1,306,307,308,-1,309,310,311,-1,312,313,314,-1,315,316,317,-1,318,319,320,-1,321,322,323,-1,324,325,326,-1,327,328,329,-1,330,331,332,-1,333,334,335,-1,336,337,338,-1,339,340,341,-1,342,343,344,-1,345,346,347,-1,348,349,350,-1,351,352,353,-1,354,355,356,-1,357,358,359,-1,360,361,362,-1,363,364,365,-1,366,367,368,-1,369,370,371,-1,372,373,374,-1,375,376,377,-1,378,379,380,-1,381,382,383,-1,384,385,386,-1,387,388,389,-1,390,391,392,-1,393,394,395,-1,396,397,398,-1,399,400,401,-1,402,403,404,-1,405,406,407,-1,408,409,410,-1,411,412,413,-1,414,415,416,-1,417,418,419,-1,420,421,422,-1,423,424,425,-1,426,427,428,-1,429,430,431,-1,432,433,434,-1,435,436,437,-1,438,439,440,-1,441,442,443,-1,444,445,446,-1,447,448,449,-1,450,451,452,-1,453,454,455,-1,456,457,458,-1,459,460,461,-1,462,463,464,-1,465,466,467,-1,468,469,470,-1,471,472,473,-1,474,475,476,-1,477,478,479,-1,480,481,482,-1,483,484,485,-1,486,487,488,-1,489,490,491,-1,492,493,494,-1,495,496,497,-1,498,499,500,-1,501,502,503,-1,504,505,506,-1,507,508,509,-1,510,511,512,-1,513,514,515,-1,516,517,518,-1,519,520,521,-1,522,523,524,-1,525,526,527,-1,528,529,530,-1,531,532,533,-1,534,535,536,-1,537,538,539,-1,540,541,542,-1,543,544,545,-1,546,547,548,-1,549,550,551,-1,552,553,554,-1,555,556,557,-1,558,559,560,-1,561,562,563,-1,564,565,566,-1,567,568,569,-1,570,571,572,-1,573,574,575,-1,576,577,578,-1,579,580,581,-1,582,583,584,-1,585,586,587,-1,588,589,590,-1,591,592,593,-1,594,595,596,-1,597,598,599,-1,600,601,602,-1,603,604,605,-1,606,607,608,-1,609,610,611,-1,612,613,614,-1,615,616,617,-1,618,619,620,-1,621,622,623,-1,624,625,626,-1,627,628,629,-1,630,631,632,-1,633,634,635,-1,636,637,638,-1,639,640,641,-1,642,643,644,-1,645,646,647,-1,648,649,650,-1,651,652,653,-1,654,655,656,-1,657,658,659,-1,660,661,662,-1,663,664,665,-1,666,667,668,-1,669,670,671,-1,6,7,8,-1,3,4,5,-1,9,10,11,-1,672,673,674,-1,675,676,677,-1],creaseAngle=3.14159,normalIndex=[0,1,2,-1,10,10,10,-1,11,11,11,-1,12,12,12,-1,13,13,13,-1,14,14,14,-1,15,16,17,-1,18,19,20,-1,21,22,23,-1,24,25,26,-1,27,27,27,-1,28,28,28,-1,29,29,29,-1,30,31,32,-1,33,34,35,-1,36,37,38,-1,39,40,41,-1,42,43,44,-1,45,46,47,-1,48,49,50,-1,51,51,51,-1,52,53,54,-1,55,56,57,-1,58,59,60,-1,61,62,63,-1,64,65,66,-1,67,67,67,-1,68,69,70,-1,71,72,73,-1,74,74,74,-1,75,75,75,-1,76,76,76,-1,77,78,79,-1,80,80,80,-1,81,82,83,-1,84,84,84,-1,85,85,85,-1,86,86,86,-1,87,87,87,-1,88,88,88,-1,89,89,89,-1,90,91,92,-1,93,94,95,-1,96,96,96,-1,97,98,99,-1,100,101,102,-1,103,104,105,-1,106,107,108,-1,109,110,111,-1,112,113,114,-1,115,115,115,-1,116,117,118,-1,119,119,119,-1,120,121,122,-1,123,124,125,-1,126,127,128,-1,129,129,129,-1,130,131,132,-1,133,134,135,-1,136,136,136,-1,137,138,139,-1,140,141,142,-1,143,143,143,-1,144,145,146,-1,147,148,149,-1,150,151,152,-1,153,154,155,-1,156,156,156,-1,157,157,157,-1,158,159,160,-1,161,162,163,-1,164,164,164,-1,165,165,165,-1,166,167,168,-1,169,170,171,-1,172,173,174,-1,175,176,177,-1,178,178,178,-1,179,179,179,-1,180,181,182,-1,183,184,185,-1,186,187,188,-1,189,189,189,-1,190,190,190,-1,191,192,193,-1,194,195,196,-1,197,197,197,-1,198,198,198,-1,199,199,199,-1,200,201,202,-1,203,204,205,-1,206,206,206,-1,207,208,209,-1,210,210,210,-1,211,211,211,-1,212,212,212,-1,213,213,213,-1,214,215,216,-1,217,218,219,-1,220,220,220,-1,221,221,221,-1,222,222,222,-1,223,224,225,-1,226,227,228,-1,229,229,229,-1,230,231,232,-1,233,234,235,-1,236,237,238,-1,239,240,241,-1,242,243,244,-1,245,246,247,-1,248,248,248,-1,249,249,249,-1,250,251,252,-1,253,254,255,-1,256,257,258,-1,259,260,261,-1,262,263,264,-1,265,265,265,-1,266,266,266,-1,267,267,267,-1,268,269,270,-1,271,272,273,-1,274,275,276,-1,277,278,279,-1,280,281,282,-1,283,284,285,-1,286,287,288,-1,289,289,289,-1,290,291,292,-1,293,294,295,-1,296,297,298,-1,299,300,301,-1,302,303,304,-1,305,305,305,-1,306,307,308,-1,309,310,311,-1,312,312,312,-1,313,313,313,-1,314,314,314,-1,315,316,317,-1,318,318,318,-1,319,320,321,-1,322,322,322,-1,323,323,323,-1,324,324,324,-1,325,325,325,-1,326,326,326,-1,327,327,327,-1,328,329,330,-1,331,332,333,-1,334,334,334,-1,335,336,337,-1,338,339,340,-1,341,342,343,-1,344,345,346,-1,347,348,349,-1,350,351,352,-1,353,353,353,-1,354,355,356,-1,357,357,357,-1,358,359,360,-1,361,362,363,-1,364,365,366,-1,367,367,367,-1,368,369,370,-1,371,372,373,-1,374,374,374,-1,375,376,377,-1,378,379,380,-1,381,381,381,-1,382,383,384,-1,385,386,387,-1,388,389,390,-1,391,392,393,-1,394,394,394,-1,395,395,395,-1,396,397,398,-1,399,400,401,-1,402,402,402,-1,403,403,403,-1,404,405,406,-1,407,408,409,-1,410,411,412,-1,413,414,415,-1,416,416,416,-1,417,417,417,-1,418,419,420,-1,421,422,423,-1,424,425,426,-1,427,427,427,-1,428,428,428,-1,429,430,431,-1,432,433,434,-1,435,435,435,-1,436,436,436,-1,437,437,437,-1,438,439,440,-1,441,442,443,-1,444,444,444,-1,445,446,447,-1,448,448,448,-1,449,449,449,-1,450,450,450,-1,451,451,451,-1,452,453,454,-1,455,456,457,-1,458,458,458,-1,459,459,459,-1,460,460,460,-1,461,462,463,-1,464,465,466,-1,467,467,467,-1,468,469,470,-1,471,472,473,-1,474,475,476,-1,477,478,479,-1,480,481,482,-1,483,484,485,-1,486,486,486,-1,487,487,487,-1,4,5,6,-1,3,3,3,-1,7,8,9,-1,488,488,488,-1,489,490,491,-1],solid=False,
            coord=Coordinate(DEF='_1',point=[(-0.0902426,0.707736,-0.361894),(-0.126242,0.7163,-0.510588),(-0.0821966,0.717949,-0.490936),(1.05985e-006,0.934617,0.0504887),(-0.00664806,0.898872,-0.0277972),(0,0.901021,0.122661),(1.05985e-006,0.934617,0.0504887),(0,0.901021,0.122661),(-2.03005e-006,0.90317,0.277591),(1.06986e-006,1.0793,3.47475e-007),(1.05985e-006,0.934617,0.0504887),(-2.03005e-006,0.90317,0.277591),(-0.114758,0.901562,-0.463227),(-0.118379,0.85895,-0.478181),(-0.098076,0.874364,-0.394307),(-0.118483,0.948969,-0.478612),(-0.114758,0.901562,-0.463227),(-0.098076,0.874364,-0.394307),(-0.148198,0.693025,-0.60134),(-0.153978,0.652374,-0.625216),(-0.140082,0.684348,-0.567816),(-0.126187,0.716322,-0.510419),(-0.148198,0.693025,-0.60134),(-0.140082,0.684348,-0.567816),(-0.151929,0.723428,-0.61675),(-0.148198,0.693025,-0.60134),(-0.126187,0.716322,-0.510419),(-0.123346,0.739931,-0.0712886),(-0.147796,0.734624,-0.210758),(-0.142964,0.693107,-0.139234),(-0.147796,0.734624,-0.210758),(-0.154642,0.690461,-0.206715),(-0.142964,0.693107,-0.139234),(-0.142964,0.693107,-0.139234),(-0.154642,0.690461,-0.206715),(-0.170992,0.648327,-0.242015),(-0.118464,0.756954,0.588829),(-0.134253,0.756954,0.279345),(-0.302092,0.706351,0.270908),(-0.171744,0.724864,-0.680752),(-0.227054,0.740499,-0.761237),(-0.221648,0.65237,-0.934256),(-0.227054,0.740499,-0.761237),(-0.171744,0.724864,-0.680752),(-0.156404,0.717949,-0.47297),(-0.227054,0.740499,-0.761237),(-0.156404,0.717949,-0.47297),(-0.193516,0.738534,-0.463984),(-0.193516,0.738534,-0.463984),(-0.156404,0.717949,-0.47297),(-0.10699,0.717949,-0.106569),(-0.156404,0.717949,-0.47297),(-0.0902276,0.707736,-0.361897),(-0.0580738,0.70326,-0.0822888),(-0.0902276,0.707736,-0.361897),(-0.00919576,0.699209,-0.0271805),(-0.0580738,0.70326,-0.0822888),(-0.156404,0.717949,-0.47297),(-0.0580737,0.70326,-0.0822888),(-0.10699,0.717949,-0.106569),(-0.10699,0.717949,-0.106569),(-0.0580738,0.70326,-0.0822888),(-0.0766211,0.707736,0.27759),(-0.113414,0.724864,0.27759),(-0.10699,0.717949,-0.106569),(-0.0766211,0.707736,0.27759),(-0.0580738,0.70326,-0.0822888),(-0.00919576,0.699209,-0.0271805),(-0.0352099,0.699209,0.27759),(-0.00919576,0.699209,-0.0271805),(-0.0030355,0.697365,0.277591),(-0.0352099,0.699209,0.27759),(-0.0580738,0.70326,-0.0822888),(-0.0352099,0.699209,0.27759),(-0.0766211,0.707736,0.27759),(-0.00303552,0.697365,0.277591),(-0.00303664,0.699209,0.4528),(-0.0352099,0.699209,0.27759),(-0.131733,0.744141,0.27759),(-0.113414,0.724864,0.27759),(-0.123031,0.740499,0.495402),(-0.123031,0.740499,0.495402),(-0.113414,0.724864,0.27759),(-0.091476,0.717949,0.518391),(-0.113414,0.724864,0.27759),(-0.0766211,0.707736,0.27759),(-0.091476,0.717949,0.518391),(-0.00303936,0.724864,0.878668),(-0.047258,0.717949,0.759194),(-0.00303808,0.707736,0.678309),(-0.0766211,0.707736,0.27759),(-0.0430349,0.70326,0.495403),(-0.091476,0.717949,0.518391),(-0.0352099,0.699209,0.27759),(-0.0430349,0.70326,0.495403),(-0.0766211,0.707736,0.27759),(-0.0430349,0.70326,0.495403),(-0.047258,0.717949,0.759194),(-0.091476,0.717949,0.518391),(-0.00303808,0.707736,0.678309),(-0.047258,0.717949,0.759194),(-0.0430349,0.70326,0.495403),(-0.00303664,0.699209,0.4528),(-0.00303808,0.707736,0.678309),(-0.0430349,0.70326,0.495403),(-0.0352099,0.699209,0.27759),(-0.00303664,0.699209,0.4528),(-0.0430349,0.70326,0.495403),(-0.00303936,0.724864,0.878668),(-0.0430376,0.740499,0.931031),(-0.047258,0.717949,0.759194),(-0.0352143,0.759118,0.978427),(-0.076625,0.759118,0.878668),(-0.0430376,0.740499,0.931031),(-0.0914776,0.738534,0.759194),(-0.113416,0.759118,0.678308),(-0.123031,0.740499,0.495402),(-0.0914776,0.738534,0.759194),(-0.123031,0.740499,0.495402),(-0.091476,0.717949,0.518391),(-0.076625,0.759118,0.878668),(-0.113416,0.759118,0.678308),(-0.0914776,0.738534,0.759194),(-0.0430376,0.740499,0.931031),(-0.076625,0.759118,0.878668),(-0.0914776,0.738534,0.759194),(-0.0430376,0.740499,0.931031),(-0.0914776,0.738534,0.759194),(-0.047258,0.717949,0.759194),(-0.047258,0.717949,0.759194),(-0.0914776,0.738534,0.759194),(-0.091476,0.717949,0.518391),(-0.113416,0.759118,0.678308),(-0.131734,0.759118,0.452799),(-0.123031,0.740499,0.495402),(-0.131734,0.759118,0.452799),(-0.131733,0.744141,0.27759),(-0.123031,0.740499,0.495402),(-0.135694,0.759118,0.27759),(-0.131733,0.744141,0.27759),(-0.131734,0.759118,0.452799),(-0.125212,0.740499,-0.0660358),(-0.193516,0.738534,-0.463984),(-0.159365,0.739516,-0.265011),(-0.182866,0.759118,-0.339472),(-0.193516,0.738534,-0.463984),(-0.125212,0.740499,-0.0660358),(-0.182866,0.759118,-0.339472),(-0.125212,0.740499,-0.0660358),(-0.135326,0.759118,-0.00108072),(-0.125212,0.740499,-0.0660358),(-0.10699,0.717949,-0.106569),(-0.113414,0.724864,0.27759),(-0.135326,0.759118,-0.00108072),(-0.125212,0.740499,-0.0660358),(-0.131733,0.744141,0.27759),(-0.131733,0.744141,0.27759),(-0.125212,0.740499,-0.0660358),(-0.113414,0.724864,0.27759),(-0.135694,0.759118,0.27759),(-0.135326,0.759118,-0.00108072),(-0.131733,0.744141,0.27759),(-0.182866,0.759118,-0.339472),(-0.241643,0.759118,-0.666992),(-0.193516,0.738534,-0.463984),(-0.241643,0.759118,-0.666992),(-0.227054,0.740499,-0.761237),(-0.193516,0.738534,-0.463984),(-0.235296,0.759118,-0.842319),(-0.227054,0.740499,-0.761237),(-0.241643,0.759118,-0.666992),(-0.235296,0.759118,-0.842319),(-0.221648,0.65237,-0.934256),(-0.227054,0.740499,-0.761237),(-0.235296,0.759118,-0.842319),(-0.211867,0.759118,-0.884567),(-0.221648,0.65237,-0.934256),(-0.00304013,0.759118,0.999996),(-0.0352143,0.759118,0.978427),(-0.00304,0.744141,0.978427),(-0.00304,0.744141,0.978427),(-0.0430376,0.740499,0.931031),(-0.00303936,0.724864,0.878668),(-0.00304,0.744141,0.978427),(-0.0352143,0.759118,0.978427),(-0.0430376,0.740499,0.931031),(-0.131733,0.794038,0.27759),(-0.135326,0.759118,-0.00108072),(-0.135694,0.759118,0.27759),(-0.125212,0.80253,-0.0660358),(-0.135326,0.759118,-0.00108072),(-0.131733,0.794038,0.27759),(-0.125212,0.80253,-0.0660358),(-0.182866,0.759118,-0.339472),(-0.135326,0.759118,-0.00108072),(-0.193516,0.807112,-0.463984),(-0.182866,0.759118,-0.339472),(-0.125212,0.80253,-0.0660358),(-0.193516,0.807112,-0.463984),(-0.241643,0.759118,-0.666992),(-0.182866,0.759118,-0.339472),(-0.193516,0.807112,-0.463984),(-0.227054,0.80253,-0.761237),(-0.241643,0.759118,-0.666992),(-0.246206,1.14936,-1.059),(-0.227054,0.80253,-0.761237),(-0.171744,0.838984,-0.680752),(-0.227054,0.80253,-0.761237),(-0.235296,0.759118,-0.842323),(-0.241643,0.759118,-0.666992),(-0.246206,1.14936,-1.059),(-0.235296,0.759118,-0.842323),(-0.227054,0.80253,-0.761237),(-0.246206,1.14936,-1.059),(-0.211867,0.759118,-0.884567),(-0.235296,0.759118,-0.842323),(-0.113414,0.838984,0.27759),(-0.125212,0.80253,-0.0660358),(-0.131733,0.794038,0.27759),(-0.10699,0.855105,-0.106569),(-0.125212,0.80253,-0.0660358),(-0.113414,0.838984,0.27759),(-0.10699,0.855105,-0.106569),(-0.193516,0.807112,-0.463984),(-0.125212,0.80253,-0.0660358),(-0.156404,0.855105,-0.47297),(-0.193516,0.807112,-0.463984),(-0.10699,0.855105,-0.106569),(-0.156404,0.855105,-0.47297),(-0.227054,0.80253,-0.761237),(-0.193516,0.807112,-0.463984),(-0.156404,0.855105,-0.47297),(-0.171744,0.838984,-0.680752),(-0.227054,0.80253,-0.761237),(-0.076621,0.878917,0.27759),(-0.10699,0.855105,-0.106569),(-0.113414,0.838984,0.27759),(-0.0580738,0.889352,-0.0822888),(-0.10699,0.855105,-0.106569),(-0.076621,0.878917,0.27759),(-0.0580738,0.889352,-0.0822888),(-0.156404,0.855105,-0.47297),(-0.10699,0.855105,-0.106569),(-0.0580738,0.889352,-0.0822888),(-0.0902276,0.878917,-0.361897),(-0.156404,0.855105,-0.47297),(-0.00919576,0.898799,-0.0271805),(-0.0902276,0.878917,-0.361897),(-0.0580738,0.889352,-0.0822888),(-0.0352099,0.898799,0.27759),(-0.0580738,0.889352,-0.0822888),(-0.076621,0.878917,0.27759),(-0.00919576,0.898799,-0.0271805),(-0.0580738,0.889352,-0.0822888),(-0.0352099,0.898799,0.27759),(-0.00919576,0.898799,-0.0271805),(-0.0352099,0.898799,0.27759),(-0.0030355,0.903097,0.277591),(-0.00303999,0.794038,0.978427),(-0.0352143,0.759118,0.978427),(-0.00304013,0.759118,0.999996),(-0.00303999,0.794038,0.978427),(-0.0430376,0.80253,0.931031),(-0.0352143,0.759118,0.978427),(-0.0352143,0.759118,0.978427),(-0.0430376,0.80253,0.931031),(-0.076625,0.759118,0.878668),(-0.0430376,0.80253,0.931031),(-0.0914776,0.807112,0.759194),(-0.076625,0.759118,0.878668),(-0.076625,0.759118,0.878668),(-0.0914776,0.807112,0.759194),(-0.113416,0.759118,0.678308),(-0.0914776,0.807112,0.759194),(-0.123031,0.80253,0.495402),(-0.113416,0.759118,0.678308),(-0.123031,0.80253,0.495402),(-0.131734,0.759118,0.452799),(-0.113416,0.759118,0.678308),(-0.123031,0.80253,0.495402),(-0.131733,0.794038,0.27759),(-0.131734,0.759118,0.452799),(-0.131733,0.794038,0.27759),(-0.135694,0.759118,0.27759),(-0.131734,0.759118,0.452799),(-0.00303999,0.794038,0.978427),(-0.00303936,0.838984,0.878668),(-0.0430376,0.80253,0.931031),(-0.00303936,0.838984,0.878668),(-0.047258,0.855105,0.759194),(-0.0430376,0.80253,0.931031),(-0.0430376,0.80253,0.931031),(-0.047258,0.855105,0.759194),(-0.0914776,0.807112,0.759194),(-0.047258,0.855105,0.759194),(-0.091476,0.855105,0.518391),(-0.0914776,0.807112,0.759194),(-0.091476,0.855105,0.518391),(-0.123031,0.80253,0.495402),(-0.0914776,0.807112,0.759194),(-0.091476,0.855105,0.518391),(-0.113414,0.838984,0.27759),(-0.123031,0.80253,0.495402),(-0.113414,0.838984,0.27759),(-0.131733,0.794038,0.27759),(-0.123031,0.80253,0.495402),(-0.00303808,0.878917,0.678309),(-0.047258,0.855105,0.759194),(-0.00303936,0.838984,0.878668),(-0.00303808,0.878917,0.678309),(-0.0430349,0.889352,0.495403),(-0.047258,0.855105,0.759194),(-0.0430349,0.889352,0.495403),(-0.091476,0.855105,0.518391),(-0.047258,0.855105,0.759194),(-0.0430349,0.889352,0.495403),(-0.076621,0.878917,0.27759),(-0.091476,0.855105,0.518391),(-0.076621,0.878917,0.27759),(-0.113414,0.838984,0.27759),(-0.091476,0.855105,0.518391),(-0.00303808,0.878917,0.678309),(-0.00303664,0.898799,0.4528),(-0.0430349,0.889352,0.495403),(-0.00303664,0.898799,0.4528),(-0.0352099,0.898799,0.27759),(-0.0430349,0.889352,0.495403),(-0.0352099,0.898799,0.27759),(-0.076621,0.878917,0.27759),(-0.0430349,0.889352,0.495403),(-0.00303551,0.903097,0.277591),(-0.0352099,0.898799,0.27759),(-0.00303664,0.898799,0.4528),(-0.126225,0.85895,-0.510592),(-0.171744,0.838984,-0.680752),(-0.156404,0.855105,-0.47297),(-0.0902276,0.878917,-0.361897),(-0.126225,0.85895,-0.510592),(-0.156404,0.855105,-0.47297),(-0.159365,0.739516,-0.265011),(-0.193516,0.738534,-0.463984),(-0.10699,0.717949,-0.106569),(-0.125212,0.740499,-0.0660358),(-0.159365,0.739516,-0.265011),(-0.10699,0.717949,-0.106569),(-0.171744,0.724864,-0.680752),(-0.126225,0.7163,-0.510592),(-0.156404,0.717949,-0.47297),(-0.156404,0.717949,-0.47297),(-0.126225,0.7163,-0.510592),(-0.0902276,0.707736,-0.361897),(0.0584968,0.693107,-0.188007),(0.0300801,0.734624,-0.253822),(0.0721283,0.739931,-0.118612),(0.0584968,0.693107,-0.188007),(0.0380172,0.690461,-0.253357),(0.0300801,0.734624,-0.253822),(0.036411,0.648327,-0.292226),(0.0380172,0.690461,-0.253357),(0.0584968,0.693107,-0.188007),(0.296012,0.706351,0.270908),(0.128173,0.756954,0.279345),(0.112384,0.756954,0.588829),(-0.221672,0.65237,-0.934251),(-0.151089,0.740499,-0.776192),(-0.171766,0.724864,-0.680748),(-0.0821966,0.717949,-0.490936),(-0.171766,0.724864,-0.680748),(-0.151089,0.740499,-0.776192),(-0.0450844,0.738534,-0.49992),(-0.0821966,0.717949,-0.490936),(-0.151089,0.740499,-0.776192),(0.0414473,0.717949,-0.142505),(-0.0821966,0.717949,-0.490936),(-0.0450844,0.738534,-0.49992),(0.00905313,0.70326,-0.0985401),(-0.0902426,0.707736,-0.361894),(-0.0821966,0.717949,-0.490936),(0.00905313,0.70326,-0.0985401),(-0.00920661,0.699209,-0.0271778),(-0.0902426,0.707736,-0.361894),(0.0414473,0.717949,-0.142505),(0.00905313,0.70326,-0.0985401),(-0.0821966,0.717949,-0.490936),(0.0705411,0.707736,0.27759),(0.00905313,0.70326,-0.0985401),(0.0414473,0.717949,-0.142505),(0.0705411,0.707736,0.27759),(0.0414473,0.717949,-0.142505),(0.107334,0.724864,0.27759),(0.0291299,0.699209,0.27759),(-0.00920661,0.699209,-0.0271778),(0.00905313,0.70326,-0.0985401),(0.0291299,0.699209,0.27759),(-0.00304449,0.697365,0.277591),(-0.00920661,0.699209,-0.0271778),(0.0705411,0.707736,0.27759),(0.0291299,0.699209,0.27759),(0.00905313,0.70326,-0.0985401),(0.0291299,0.699209,0.27759),(-0.00304335,0.699209,0.4528),(-0.00304447,0.697365,0.277591),(0.116951,0.740499,0.495402),(0.107334,0.724864,0.27759),(0.125653,0.744141,0.27759),(0.085396,0.717949,0.518391),(0.107334,0.724864,0.27759),(0.116951,0.740499,0.495402),(0.085396,0.717949,0.518391),(0.0705411,0.707736,0.27759),(0.107334,0.724864,0.27759),(-0.00304191,0.707736,0.678309),(0.041178,0.717949,0.759194),(-0.00304063,0.724864,0.878668),(0.085396,0.717949,0.518391),(0.0369549,0.70326,0.495403),(0.0705411,0.707736,0.27759),(0.0705411,0.707736,0.27759),(0.0369549,0.70326,0.495403),(0.0291299,0.699209,0.27759),(0.085396,0.717949,0.518391),(0.041178,0.717949,0.759194),(0.0369549,0.70326,0.495403),(0.0369549,0.70326,0.495403),(0.041178,0.717949,0.759194),(-0.00304191,0.707736,0.678309),(0.0369549,0.70326,0.495403),(-0.00304191,0.707736,0.678309),(-0.00304335,0.699209,0.4528),(0.0369549,0.70326,0.495403),(-0.00304335,0.699209,0.4528),(0.0291299,0.699209,0.27759),(0.041178,0.717949,0.759194),(0.0369577,0.740499,0.931031),(-0.00304063,0.724864,0.878668),(0.0369577,0.740499,0.931031),(0.070545,0.759118,0.878668),(0.0291344,0.759118,0.978427),(0.116951,0.740499,0.495402),(0.107336,0.759118,0.678308),(0.0853976,0.738534,0.759194),(0.085396,0.717949,0.518391),(0.116951,0.740499,0.495402),(0.0853976,0.738534,0.759194),(0.0853976,0.738534,0.759194),(0.107336,0.759118,0.678308),(0.070545,0.759118,0.878668),(0.0853976,0.738534,0.759194),(0.070545,0.759118,0.878668),(0.0369577,0.740499,0.931031),(0.041178,0.717949,0.759194),(0.0853976,0.738534,0.759194),(0.0369577,0.740499,0.931031),(0.085396,0.717949,0.518391),(0.0853976,0.738534,0.759194),(0.041178,0.717949,0.759194),(0.116951,0.740499,0.495402),(0.125654,0.759118,0.452799),(0.107336,0.759118,0.678308),(0.116951,0.740499,0.495402),(0.125653,0.744141,0.27759),(0.125654,0.759118,0.452799),(0.125654,0.759118,0.452799),(0.125653,0.744141,0.27759),(0.129614,0.759118,0.27759),(0.0155534,0.739516,-0.307358),(-0.0450844,0.738534,-0.49992),(0.0761915,0.740499,-0.114795),(0.0761915,0.740499,-0.114795),(-0.0450844,0.738534,-0.49992),(0.00239428,0.759118,-0.384322),(0.0988058,0.759118,-0.053328),(0.0761915,0.740499,-0.114795),(0.00239428,0.759118,-0.384322),(0.107334,0.724864,0.27759),(0.0414473,0.717949,-0.142505),(0.0761915,0.740499,-0.114795),(0.125653,0.744141,0.27759),(0.0761915,0.740499,-0.114795),(0.0988058,0.759118,-0.053328),(0.107334,0.724864,0.27759),(0.0761915,0.740499,-0.114795),(0.125653,0.744141,0.27759),(0.125653,0.744141,0.27759),(0.0988058,0.759118,-0.053328),(0.129614,0.759118,0.27759),(-0.0450844,0.738534,-0.49992),(-0.101867,0.759118,-0.694509),(0.00239428,0.759118,-0.384322),(-0.0450844,0.738534,-0.49992),(-0.151089,0.740499,-0.776192),(-0.101867,0.759118,-0.694509),(-0.101867,0.759118,-0.694509),(-0.151089,0.740499,-0.776192),(-0.174195,0.759118,-0.854348),(-0.151089,0.740499,-0.776192),(-0.221672,0.65237,-0.934251),(-0.174195,0.759118,-0.854348),(-0.221672,0.65237,-0.934251),(-0.21189,0.759118,-0.884562),(-0.174195,0.759118,-0.854348),(-0.00303999,0.744141,0.978427),(0.0291344,0.759118,0.978427),(-0.00303985,0.759118,0.999996),(-0.00304063,0.724864,0.878668),(0.0369577,0.740499,0.931031),(-0.00303999,0.744141,0.978427),(0.0369577,0.740499,0.931031),(0.0291344,0.759118,0.978427),(-0.00303999,0.744141,0.978427),(0.129614,0.759118,0.27759),(0.0988058,0.759118,-0.053328),(0.125653,0.794038,0.27759),(0.125653,0.794038,0.27759),(0.0988058,0.759118,-0.053328),(0.0761915,0.80253,-0.114795),(0.0988058,0.759118,-0.053328),(0.00239428,0.759118,-0.384322),(0.0761915,0.80253,-0.114795),(0.0761915,0.80253,-0.114795),(0.00239428,0.759118,-0.384322),(-0.0450844,0.807112,-0.49992),(0.00239428,0.759118,-0.384322),(-0.101867,0.759118,-0.694509),(-0.0450844,0.807112,-0.49992),(-0.101867,0.759118,-0.694509),(-0.151089,0.80253,-0.776192),(-0.0450844,0.807112,-0.49992),(-0.171766,0.838984,-0.680748),(-0.151089,0.80253,-0.776192),(-0.246233,1.14936,-1.05901),(-0.101867,0.759118,-0.694509),(-0.174196,0.759118,-0.854351),(-0.151089,0.80253,-0.776192),(-0.151089,0.80253,-0.776192),(-0.174196,0.759118,-0.854351),(-0.246233,1.14936,-1.05901),(-0.174196,0.759118,-0.854351),(-0.21189,0.759118,-0.884562),(-0.246233,1.14936,-1.05901),(0.125653,0.794038,0.27759),(0.0761915,0.80253,-0.114795),(0.107334,0.838984,0.27759),(0.107334,0.838984,0.27759),(0.0761915,0.80253,-0.114795),(0.0414473,0.855105,-0.142505),(0.0761915,0.80253,-0.114795),(-0.0450844,0.807112,-0.49992),(0.0414473,0.855105,-0.142505),(0.0414473,0.855105,-0.142505),(-0.0450844,0.807112,-0.49992),(-0.0821966,0.855105,-0.490936),(-0.0450844,0.807112,-0.49992),(-0.151089,0.80253,-0.776192),(-0.0821966,0.855105,-0.490936),(-0.151089,0.80253,-0.776192),(-0.171766,0.838984,-0.680748),(-0.0821966,0.855105,-0.490936),(0.107334,0.838984,0.27759),(0.0414473,0.855105,-0.142505),(0.070541,0.878917,0.27759),(0.070541,0.878917,0.27759),(0.0414473,0.855105,-0.142505),(0.00905313,0.889352,-0.0985401),(0.0414473,0.855105,-0.142505),(-0.0821966,0.855105,-0.490936),(0.00905313,0.889352,-0.0985401),(-0.0821966,0.855105,-0.490936),(-0.0902426,0.878917,-0.361894),(0.00905313,0.889352,-0.0985401),(0.00905313,0.889352,-0.0985401),(-0.0902426,0.878917,-0.361894),(-0.00920661,0.898799,-0.0271778),(0.070541,0.878917,0.27759),(0.00905313,0.889352,-0.0985401),(0.0291299,0.898799,0.27759),(0.0291299,0.898799,0.27759),(0.00905313,0.889352,-0.0985401),(-0.00920661,0.898799,-0.0271778),(-0.00304449,0.903097,0.277591),(0.0291299,0.898799,0.27759),(-0.00920661,0.898799,-0.0271778),(-0.00303986,0.759118,0.999996),(0.0291344,0.759118,0.978427),(-0.00303999,0.794038,0.978427),(0.0291344,0.759118,0.978427),(0.0369577,0.80253,0.931031),(-0.00303999,0.794038,0.978427),(0.070545,0.759118,0.878668),(0.0369577,0.80253,0.931031),(0.0291344,0.759118,0.978427),(0.070545,0.759118,0.878668),(0.0853976,0.807112,0.759194),(0.0369577,0.80253,0.931031),(0.107336,0.759118,0.678308),(0.0853976,0.807112,0.759194),(0.070545,0.759118,0.878668),(0.107336,0.759118,0.678308),(0.116951,0.80253,0.495402),(0.0853976,0.807112,0.759194),(0.107336,0.759118,0.678308),(0.125654,0.759118,0.452799),(0.116951,0.80253,0.495402),(0.125654,0.759118,0.452799),(0.125653,0.794038,0.27759),(0.116951,0.80253,0.495402),(0.125654,0.759118,0.452799),(0.129614,0.759118,0.27759),(0.125653,0.794038,0.27759),(0.0369577,0.80253,0.931031),(-0.00304063,0.838984,0.878668),(-0.00303999,0.794038,0.978427),(0.0369577,0.80253,0.931031),(0.041178,0.855105,0.759194),(-0.00304063,0.838984,0.878668),(0.0853976,0.807112,0.759194),(0.041178,0.855105,0.759194),(0.0369577,0.80253,0.931031),(0.0853976,0.807112,0.759194),(0.085396,0.855105,0.518391),(0.041178,0.855105,0.759194),(0.0853976,0.807112,0.759194),(0.116951,0.80253,0.495402),(0.085396,0.855105,0.518391),(0.116951,0.80253,0.495402),(0.107334,0.838984,0.27759),(0.085396,0.855105,0.518391),(0.116951,0.80253,0.495402),(0.125653,0.794038,0.27759),(0.107334,0.838984,0.27759),(-0.00304063,0.838984,0.878668),(0.041178,0.855105,0.759194),(-0.00304191,0.878917,0.678309),(0.041178,0.855105,0.759194),(0.0369549,0.889352,0.495403),(-0.00304191,0.878917,0.678309),(0.041178,0.855105,0.759194),(0.085396,0.855105,0.518391),(0.0369549,0.889352,0.495403),(0.085396,0.855105,0.518391),(0.070541,0.878917,0.27759),(0.0369549,0.889352,0.495403),(0.085396,0.855105,0.518391),(0.107334,0.838984,0.27759),(0.070541,0.878917,0.27759),(0.0369549,0.889352,0.495403),(-0.00304335,0.898799,0.4528),(-0.00304191,0.878917,0.678309),(0.0369549,0.889352,0.495403),(0.0291299,0.898799,0.27759),(-0.00304335,0.898799,0.4528),(0.0369549,0.889352,0.495403),(0.070541,0.878917,0.27759),(0.0291299,0.898799,0.27759),(-0.00304335,0.898799,0.4528),(0.0291299,0.898799,0.27759),(-0.00304448,0.903097,0.277591),(-0.0821966,0.855105,-0.490936),(-0.171766,0.838984,-0.680748),(-0.126242,0.85895,-0.510588),(-0.0821966,0.855105,-0.490936),(-0.126242,0.85895,-0.510588),(-0.0902426,0.878917,-0.361894),(0.0414473,0.717949,-0.142505),(-0.0450844,0.738534,-0.49992),(0.0155534,0.739516,-0.307358),(0.0414473,0.717949,-0.142505),(0.0155534,0.739516,-0.307358),(0.0761915,0.740499,-0.114795),(-0.0821966,0.717949,-0.490936),(-0.126242,0.7163,-0.510588),(-0.171766,0.724864,-0.680748),(-0.00303241,0.934544,0.0504887),(-0.0026253,0.898799,-0.0279467),(-0.00303347,0.900948,0.122661),(-0.00303241,0.934544,0.0504887),(-0.00303347,0.900948,0.122661),(-0.0030355,0.903097,0.277591)]),
            color=Color(color=[(0.756863,0.729412,0.537255),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.936863,0.899412,0.667255),(0.697647,0.487647,0.377059),(0.697647,0.487647,0.377059)]),
            normal=Normal(vector=[(0.168232,-0.985487,-0.0226496),(0.68203,-0.729642,-0.0495783),(0.68203,-0.729642,-0.0495783),(-0.971819,0.234757,-0.0213944),(-0.971819,0.234757,-0.0213944),(-0.971819,0.234757,-0.0213944),(-0.922596,0.385721,0.00593723),(-0.971819,0.234757,-0.0213944),(-0.971819,0.234757,-0.0213944),(-0.922596,0.385721,0.00593723),(-0.998226,0.0567675,-0.0179637),(-0.998226,0.0567675,-0.0179637),(-0.998226,0.0567675,-0.0179637),(-0.998226,0.0567675,-0.0179637),(-0.998226,0.0567675,-0.0179637),(-0.971819,0.234757,-0.0213944),(-0.998226,0.0567675,-0.0179637),(-0.971819,0.234757,-0.0213944),(-0.998226,0.0567675,-0.0179637),(-0.998226,0.0567675,-0.0179637),(-0.971819,0.234757,-0.0213944),(-0.971819,0.234757,-0.0213944),(-0.998226,0.0567675,-0.0179637),(-0.998226,0.0567675,-0.0179637),(-0.922596,0.385721,0.00593723),(-0.971819,0.234757,-0.0213944),(-0.971819,0.234757,-0.0213944),(-0.68203,-0.729642,-0.0495783),(-0.68203,-0.729642,-0.0495783),(-0.68203,-0.729642,-0.0495783),(-0.68203,-0.729642,-0.0495783),(-0.68203,-0.729642,-0.0495783),(-0.934381,0.347146,-0.0801311),(-0.68203,-0.729642,-0.0495783),(-0.168232,-0.985487,-0.0226496),(-0.168232,-0.985487,-0.0226496),(-0.168232,-0.985487,-0.0226496),(-0.0667016,-0.99777,-0.00224277),(-0.168232,-0.985487,-0.0226496),(-0.68203,-0.729642,-0.0495783),(-0.168232,-0.985487,-0.0226496),(-0.168232,-0.985487,-0.0226496),(-0.168232,-0.985487,-0.0226496),(-0.168232,-0.985487,-0.0226496),(-0.996534,0.0829071,0.00677949),(-0.996534,0.0829071,0.00677949),(-0.934381,0.347146,-0.0801311),(-0.996534,0.0829071,0.00677949),(-0.168232,-0.985487,-0.0226496),(-0.0667016,-0.99777,-0.00224277),(-0.0667016,-0.99777,-0.00224277),(-0.0667016,-0.99777,-0.00224277),(-0.168232,-0.985487,-0.0226496),(-0.996534,0.0829071,0.00677949),(-0.996534,0.0829071,0.00677949),(-0.0667016,-0.99777,-0.00224277),(-0.478343,-0.871368,0.109115),(-0.0667016,-0.99777,-0.00224277),(-0.996534,0.0829071,0.00677949),(-0.996534,0.0829071,0.00677949),(-0.478343,-0.871368,0.109115),(-0.478343,-0.871368,0.109115),(-0.996534,0.0829071,0.00677949),(-0.478343,-0.871368,0.109115),(-0.996534,0.0829071,0.00677949),(-0.996534,0.0829071,0.00677949),(-0.478343,-0.871368,0.109115),(-0.478343,-0.871368,0.109115),(-0.996534,0.0829071,0.00677949),(-0.478343,-0.871368,0.109115),(-0.478343,-0.871368,0.109115),(-0.996534,0.0829071,0.00677949),(-0.478343,-0.871368,0.109115),(-0.996534,0.0829071,0.00677949),(-0.478343,-0.871368,0.109115),(-0.478343,-0.871368,0.109115),(-0.478343,-0.871368,0.109115),(-0.0667016,-0.99777,-0.00224277),(-0.478343,-0.871368,0.109115),(-0.478343,-0.871368,0.109115),(-0.478343,-0.871368,0.109115),(-0.659723,-0.285817,0.695036),(-0.478343,-0.871368,0.109115),(-0.478343,-0.871368,0.109115),(-0.478343,-0.871368,0.109115),(-0.478343,-0.871368,0.109115),(-0.478343,-0.871368,0.109115),(-0.478343,-0.871368,0.109115),(-0.478343,-0.871368,0.109115),(-0.478343,-0.871368,0.109115),(-0.478343,-0.871368,0.109115),(-0.996534,0.0829071,0.00677949),(-0.478343,-0.871368,0.109115),(-0.996534,0.0829071,0.00677949),(-0.996534,0.0829071,0.00677949),(-0.478343,-0.871368,0.109115),(-0.996534,0.0829071,0.00677949),(-1.45578e-008,-2.94332e-008,1),(-0.68203,-0.729642,-0.0495783),(-1.45578e-008,-2.94332e-008,1),(-0.934381,0.347146,-0.0801311),(-0.68203,-0.729642,-0.0495783),(-0.934381,0.347146,-0.0801311),(-0.934381,0.347146,-0.0801311),(-0.934381,0.347146,-0.0801311),(-0.996534,0.0829071,0.00677949),(-0.934381,0.347146,-0.0801311),(-0.934381,0.347146,-0.0801311),(-0.996534,0.0829071,0.00677949),(-0.996534,0.0829071,0.00677949),(-0.934381,0.347146,-0.0801311),(-0.996534,0.0829071,0.00677949),(-0.996534,0.0829071,0.00677949),(-0.934381,0.347146,-0.0801311),(-0.996534,0.0829071,0.00677949),(-0.996534,0.0829071,0.00677949),(-0.934381,0.347146,-0.0801311),(-0.68203,-0.729642,-0.0495783),(-0.68203,-0.729642,-0.0495783),(-0.68203,-0.729642,-0.0495783),(-0.950622,0.147919,-0.272834),(-0.68203,-0.729642,-0.0495783),(-0.68203,-0.729642,-0.0495783),(-0.950622,0.147919,-0.272834),(-0.68203,-0.729642,-0.0495783),(-0.68203,-0.729642,-0.0495783),(-0.950622,0.147919,-0.272834),(-0.950622,0.147919,-0.272834),(-0.68203,-0.729642,-0.0495783),(-0.659723,-0.285817,0.695036),(-0.659723,-0.285817,0.695036),(-0.478343,-0.871368,0.109115),(-0.478343,-0.871368,0.109115),(-0.659723,-0.285817,0.695036),(-0.659723,-0.285817,0.695036),(-0.478343,-0.871368,0.109115),(-0.996534,0.0829071,0.00677949),(-0.934381,0.347146,-0.0801311),(-0.996534,0.0829071,0.00677949),(-0.996534,0.0829071,0.00677949),(-0.934381,0.347146,-0.0801311),(-0.934381,0.347146,-0.0801311),(-0.996534,0.0829071,0.00677949),(-0.934381,0.347146,-0.0801311),(-0.934381,0.347146,-0.0801311),(-0.68203,-0.729642,-0.0495783),(-0.934381,0.347146,-0.0801311),(-0.934381,0.347146,-0.0801311),(-0.950622,0.147919,-0.272834),(-0.68203,-0.729642,-0.0495783),(-0.950622,0.147919,-0.272834),(-0.950622,0.147919,-0.272834),(-0.68203,-0.729642,-0.0495783),(-0.950622,0.147919,-0.272834),(-0.950622,0.147919,-0.272834),(-0.68203,-0.729642,-0.0495783),(-0.950622,0.147919,-0.272834),(-0.950622,0.147919,-0.272834),(-0.996534,0.0829071,0.00677949),(-0.934381,0.347146,-0.0801311),(-0.996534,0.0829071,0.00677949),(-0.934381,0.347146,-0.0801311),(-0.934381,0.347146,-0.0801311),(-0.996534,0.0829071,0.00677949),(-0.934381,0.347146,-0.0801311),(-0.934381,0.347146,-0.0801311),(-0.934381,0.347146,-0.0801311),(-0.950622,0.147919,-0.272834),(-0.934381,0.347146,-0.0801311),(-0.934381,0.347146,-0.0801311),(-0.239083,0.963111,-0.123518),(-0.950622,0.147919,-0.272834),(-0.996534,0.0829071,0.00677949),(-0.934381,0.347146,-0.0801311),(-0.996534,0.0829071,0.00677949),(-0.934381,0.347146,-0.0801311),(-0.934381,0.347146,-0.0801311),(-0.996534,0.0829071,0.00677949),(-0.934381,0.347146,-0.0801311),(-0.934381,0.347146,-0.0801311),(-0.996534,0.0829071,0.00677949),(-0.934381,0.347146,-0.0801311),(-0.934381,0.347146,-0.0801311),(-0.996534,0.0829071,0.00677949),(-0.934381,0.347146,-0.0801311),(-0.996534,0.0829071,0.00677949),(-0.996534,0.0829071,0.00677949),(-0.934381,0.347146,-0.0801311),(-0.996534,0.0829071,0.00677949),(-0.996534,0.0829071,0.00677949),(-0.659723,-0.285817,0.695036),(-0.659723,-0.285817,0.695036),(-0.687801,0.70468,0.174231),(-0.659723,-0.285817,0.695036),(-0.659723,-0.285817,0.695036),(-0.687801,0.70468,0.174231),(-0.687801,0.70468,0.174231),(-0.687801,0.70468,0.174231),(-0.687801,0.70468,0.174231),(-0.687801,0.70468,0.174231),(-0.687801,0.70468,0.174231),(-0.996534,0.0829071,0.00677949),(-0.687801,0.70468,0.174231),(-0.687801,0.70468,0.174231),(-0.996534,0.0829071,0.00677949),(-0.996534,0.0829071,0.00677949),(-0.996534,0.0829071,0.00677949),(-0.281353,0.874907,0.394181),(-0.687801,0.70468,0.174231),(-0.687801,0.70468,0.174231),(-0.687801,0.70468,0.174231),(-0.687801,0.70468,0.174231),(-0.687801,0.70468,0.174231),(-0.687801,0.70468,0.174231),(-0.687801,0.70468,0.174231),(-0.996534,0.0829071,0.00677949),(-0.687801,0.70468,0.174231),(-0.996534,0.0829071,0.00677949),(-0.996534,0.0829071,0.00677949),(-0.687801,0.70468,0.174231),(-0.687801,0.70468,0.174231),(-0.687801,0.70468,0.174231),(-0.687801,0.70468,0.174231),(-0.687801,0.70468,0.174231),(-0.996534,0.0829071,0.00677949),(-0.687801,0.70468,0.174231),(-0.996534,0.0829071,0.00677949),(-0.996534,0.0829071,0.00677949),(-0.687801,0.70468,0.174231),(-0.687801,0.70468,0.174231),(-0.687801,0.70468,0.174231),(-0.996534,0.0829071,0.00677949),(-0.687801,0.70468,0.174231),(-0.996534,0.0829071,0.00677949),(-0.996534,0.0829071,0.00677949),(-0.687801,0.70468,0.174231),(-0.996534,0.0829071,0.00677949),(-0.996534,0.0829071,0.00677949),(-0.687801,0.70468,0.174231),(-0.239083,0.963111,-0.123518),(-0.239083,0.963111,-0.123518),(-0.934381,0.347146,-0.0801311),(-0.934381,0.347146,-0.0801311),(-0.239083,0.963111,-0.123518),(-0.934381,0.347146,-0.0801311),(-0.934381,0.347146,-0.0801311),(-0.68203,-0.729642,-0.0495783),(-0.934381,0.347146,-0.0801311),(-0.934381,0.347146,-0.0801311),(-0.68203,-0.729642,-0.0495783),(-0.68203,-0.729642,-0.0495783),(-0.68203,-0.729642,-0.0495783),(-0.168232,-0.985487,-0.0226496),(0.971819,0.234757,-0.0213944),(0.998226,0.0567675,-0.0179637),(0.971819,0.234757,-0.0213944),(0.971819,0.234757,-0.0213944),(0.998226,0.0567675,-0.0179637),(0.998226,0.0567675,-0.0179637),(0.998226,0.0567675,-0.0179637),(0.998226,0.0567675,-0.0179637),(0.971819,0.234757,-0.0213944),(0.971819,0.234757,-0.0213944),(0.971819,0.234757,-0.0213944),(0.922596,0.385721,0.00593723),(0.68203,-0.729642,-0.0495783),(0.68203,-0.729642,-0.0495783),(0.68203,-0.729642,-0.0495783),(0.934382,0.347146,-0.0801311),(0.68203,-0.729642,-0.0495783),(0.68203,-0.729642,-0.0495783),(0.168232,-0.985487,-0.0226496),(0.168232,-0.985487,-0.0226496),(0.68203,-0.729642,-0.0495783),(0.168232,-0.985487,-0.0226496),(0.0667016,-0.99777,-0.00224277),(0.168232,-0.985487,-0.0226496),(0.168232,-0.985487,-0.0226496),(0.168232,-0.985487,-0.0226496),(0.68203,-0.729642,-0.0495783),(0.996534,0.0829071,0.00677949),(0.168232,-0.985487,-0.0226496),(0.168232,-0.985487,-0.0226496),(0.996534,0.0829071,0.00677949),(0.934382,0.347146,-0.0801311),(0.996534,0.0829071,0.00677949),(0.0667016,-0.99777,-0.00224277),(0.0667016,-0.99777,-0.00224277),(0.168232,-0.985487,-0.0226496),(0.0667016,-0.99777,-0.00224277),(0.996534,0.0829071,0.00677949),(0.996534,0.0829071,0.00677949),(0.168232,-0.985487,-0.0226496),(0.0667016,-0.99777,-0.00224277),(0.478343,-0.871368,0.109115),(0.0667016,-0.99777,-0.00224277),(0.478343,-0.871368,0.109115),(0.996534,0.0829071,0.00677949),(0.996534,0.0829071,0.00677949),(0.478343,-0.871368,0.109115),(0.996534,0.0829071,0.00677949),(0.478343,-0.871368,0.109115),(0.478343,-0.871368,0.109115),(0.996534,0.0829071,0.00677949),(0.996534,0.0829071,0.00677949),(0.478343,-0.871368,0.109115),(0.478343,-0.871368,0.109115),(0.478343,-0.871368,0.109115),(0.996534,0.0829071,0.00677949),(0.996534,0.0829071,0.00677949),(0.478343,-0.871368,0.109115),(0.996534,0.0829071,0.00677949),(0.478343,-0.871368,0.109115),(0.478343,-0.871368,0.109115),(0.478343,-0.871368,0.109115),(0.478343,-0.871368,0.109115),(0.478343,-0.871368,0.109115),(0.0667016,-0.99777,-0.00224277),(0.478343,-0.871368,0.109115),(0.478343,-0.871368,0.109115),(0.478343,-0.871368,0.109115),(0.659723,-0.285817,0.695036),(0.478343,-0.871368,0.109115),(0.478343,-0.871368,0.109115),(0.478343,-0.871368,0.109115),(0.478343,-0.871368,0.109115),(0.478343,-0.871368,0.109115),(0.478343,-0.871368,0.109115),(0.478343,-0.871368,0.109115),(0.996534,0.0829071,0.00677949),(0.478343,-0.871368,0.109115),(0.478343,-0.871368,0.109115),(0.996534,0.0829071,0.00677949),(0.996534,0.0829071,0.00677949),(0.996534,0.0829071,0.00677949),(1.45578e-008,-2.94332e-008,1),(0.68203,-0.729642,-0.0495783),(1.45578e-008,-2.94332e-008,1),(0.934382,0.347146,-0.0801311),(0.68203,-0.729642,-0.0495783),(0.934382,0.347146,-0.0801311),(0.996534,0.0829071,0.00677949),(0.934382,0.347146,-0.0801311),(0.934382,0.347146,-0.0801311),(0.996534,0.0829071,0.00677949),(0.934382,0.347146,-0.0801311),(0.934382,0.347146,-0.0801311),(0.996534,0.0829071,0.00677949),(0.934382,0.347146,-0.0801311),(0.996534,0.0829071,0.00677949),(0.996534,0.0829071,0.00677949),(0.934382,0.347146,-0.0801311),(0.996534,0.0829071,0.00677949),(0.996534,0.0829071,0.00677949),(0.68203,-0.729642,-0.0495783),(0.68203,-0.729642,-0.0495783),(0.934382,0.347146,-0.0801311),(0.68203,-0.729642,-0.0495783),(0.68203,-0.729642,-0.0495783),(0.68203,-0.729642,-0.0495783),(0.950622,0.147919,-0.272834),(0.68203,-0.729642,-0.0495783),(0.68203,-0.729642,-0.0495783),(0.950622,0.147919,-0.272834),(0.68203,-0.729642,-0.0495783),(0.950622,0.147919,-0.272834),(0.950622,0.147919,-0.272834),(0.659723,-0.285817,0.695036),(0.478343,-0.871368,0.109115),(0.478343,-0.871368,0.109115),(0.659723,-0.285817,0.695036),(0.478343,-0.871368,0.109115),(0.659723,-0.285817,0.695036),(0.659723,-0.285817,0.695036),(0.996534,0.0829071,0.00677949),(0.996534,0.0829071,0.00677949),(0.996534,0.0829071,0.00677949),(0.934382,0.347146,-0.0801311),(0.996534,0.0829071,0.00677949),(0.934382,0.347146,-0.0801311),(0.934382,0.347146,-0.0801311),(0.934382,0.347146,-0.0801311),(0.934382,0.347146,-0.0801311),(0.68203,-0.729642,-0.0495783),(0.934382,0.347146,-0.0801311),(0.68203,-0.729642,-0.0495783),(0.950622,0.147919,-0.272834),(0.934382,0.347146,-0.0801311),(0.68203,-0.729642,-0.0495783),(0.950622,0.147919,-0.272834),(0.950622,0.147919,-0.272834),(0.68203,-0.729642,-0.0495783),(0.950622,0.147919,-0.272834),(0.950622,0.147919,-0.272834),(0.950622,0.147919,-0.272834),(0.950622,0.147919,-0.272834),(0.996534,0.0829071,0.00677949),(0.934382,0.347146,-0.0801311),(0.996534,0.0829071,0.00677949),(0.996534,0.0829071,0.00677949),(0.934382,0.347146,-0.0801311),(0.934382,0.347146,-0.0801311),(0.934382,0.347146,-0.0801311),(0.934382,0.347146,-0.0801311),(0.934382,0.347146,-0.0801311),(0.950622,0.147919,-0.272834),(0.934382,0.347146,-0.0801311),(0.950622,0.147919,-0.272834),(0.239083,0.963111,-0.123518),(0.934382,0.347146,-0.0801311),(0.996534,0.0829071,0.00677949),(0.934382,0.347146,-0.0801311),(0.996534,0.0829071,0.00677949),(0.996534,0.0829071,0.00677949),(0.934382,0.347146,-0.0801311),(0.934382,0.347146,-0.0801311),(0.934382,0.347146,-0.0801311),(0.934382,0.347146,-0.0801311),(0.934382,0.347146,-0.0801311),(0.934382,0.347146,-0.0801311),(0.996534,0.0829071,0.00677949),(0.996534,0.0829071,0.00677949),(0.934382,0.347146,-0.0801311),(0.996534,0.0829071,0.00677949),(0.996534,0.0829071,0.00677949),(0.934382,0.347146,-0.0801311),(0.996534,0.0829071,0.00677949),(0.996534,0.0829071,0.00677949),(0.659723,-0.285817,0.695036),(0.659723,-0.285817,0.695036),(0.687801,0.70468,0.174231),(0.659723,-0.285817,0.695036),(0.687801,0.70468,0.174231),(0.687801,0.70468,0.174231),(0.659723,-0.285817,0.695036),(0.687801,0.70468,0.174231),(0.687801,0.70468,0.174231),(0.687801,0.70468,0.174231),(0.687801,0.70468,0.174231),(0.996534,0.0829071,0.00677949),(0.687801,0.70468,0.174231),(0.996534,0.0829071,0.00677949),(0.996534,0.0829071,0.00677949),(0.687801,0.70468,0.174231),(0.996534,0.0829071,0.00677949),(0.687801,0.70468,0.174231),(0.687801,0.70468,0.174231),(0.281353,0.874907,0.394181),(0.687801,0.70468,0.174231),(0.687801,0.70468,0.174231),(0.687801,0.70468,0.174231),(0.687801,0.70468,0.174231),(0.687801,0.70468,0.174231),(0.996534,0.0829071,0.00677949),(0.687801,0.70468,0.174231),(0.687801,0.70468,0.174231),(0.996534,0.0829071,0.00677949),(0.996534,0.0829071,0.00677949),(0.687801,0.70468,0.174231),(0.687801,0.70468,0.174231),(0.687801,0.70468,0.174231),(0.687801,0.70468,0.174231),(0.996534,0.0829071,0.00677949),(0.687801,0.70468,0.174231),(0.687801,0.70468,0.174231),(0.996534,0.0829071,0.00677949),(0.996534,0.0829071,0.00677949),(0.687801,0.70468,0.174231),(0.687801,0.70468,0.174231),(0.996534,0.0829071,0.00677949),(0.687801,0.70468,0.174231),(0.687801,0.70468,0.174231),(0.996534,0.0829071,0.00677949),(0.996534,0.0829071,0.00677949),(0.687801,0.70468,0.174231),(0.996534,0.0829071,0.00677949),(0.996534,0.0829071,0.00677949),(0.934382,0.347146,-0.0801311),(0.239083,0.963111,-0.123518),(0.239083,0.963111,-0.123518),(0.934382,0.347146,-0.0801311),(0.239083,0.963111,-0.123518),(0.934382,0.347146,-0.0801311),(0.934382,0.347146,-0.0801311),(0.68203,-0.729642,-0.0495783),(0.934382,0.347146,-0.0801311),(0.934382,0.347146,-0.0801311),(0.68203,-0.729642,-0.0495783),(-0.971819,0.234757,-0.0213944),(-0.971819,0.234757,-0.0213944),(-0.971819,0.234757,-0.0213944),(-0.922596,0.385721,0.00593723)]),
            texCoord=TextureCoordinate(point=[(0.643083,0.251274),(0.722004,0.293065),(0.706565,0.29656),(0.188802,0.479106),(0.214171,0.305666),(0.16506,0.304996),(0.149232,0.399965),(0.157146,0.210027),(0.0819621,0.304996),(0.172974,0.431621),(0.192759,0.241683),(0.0977903,0.241683),(0.715545,0.368403),(0.736222,0.317839),(0.637686,0.325383),(0.696307,0.529894),(0.715545,0.368403),(0.637686,0.325383),(0.638636,0.527273),(0.706818,0.345455),(0.634091,0.309091),(0.601507,0.526195),(0.634883,0.247764),(0.590593,0.316632),(0.597141,0.526195),(0.634883,0.247764),(0.581861,0.377755),(0.0931818,0.309091),(0.179545,0.327273),(0.265909,0.509091),(0.696553,0.266068),(0.660988,0.375635),(0.642022,0.469944),(0.642022,0.469944),(0.660988,0.375635),(0.711659,0.530056),(0.0931818,0.290909),(0.220455,0.218182),(0.202273,0.436364),(0.809198,0.337614),(0.859209,0.34989),(0.881891,0.20004),(0.859209,0.34989),(0.809198,0.337614),(0.706565,0.29656),(0.859209,0.34989),(0.706565,0.29656),(0.706565,0.340177),(0.716421,0.296349),(0.715943,0.279149),(0.524105,0.206332),(0.715943,0.279149),(0.650009,0.239016),(0.505709,0.175207),(0.650009,0.239016),(0.471616,0.166622),(0.505709,0.175207),(0.715943,0.279149),(0.505709,0.175207),(0.524105,0.206332),(0.524105,0.206332),(0.505709,0.175207),(0.331407,0.184691),(0.331407,0.220983),(0.524105,0.206332),(0.331407,0.184691),(0.505709,0.175207),(0.471616,0.166622),(0.331407,0.166622),(0.471616,0.166622),(0.331407,0.162714),(0.331407,0.166622),(0.505709,0.175207),(0.331407,0.166622),(0.331407,0.184691),(0.331407,0.162714),(0.252127,0.166622),(0.331407,0.166622),(0.331408,0.26183),(0.331407,0.220983),(0.23285,0.254113),(0.23285,0.254113),(0.331407,0.220983),(0.222448,0.206332),(0.331407,0.220983),(0.331407,0.184691),(0.222448,0.206332),(0.069955,0.172684),(0.113487,0.206332),(0.151595,0.138755),(0.331407,0.184691),(0.23285,0.175207),(0.222448,0.206332),(0.331407,0.166622),(0.23285,0.175207),(0.331407,0.184691),(0.23285,0.175207),(0.113487,0.206332),(0.222448,0.206332),(0.151595,0.138755),(0.113487,0.206332),(0.23285,0.175207),(0.252127,0.166622),(0.151595,0.138755),(0.23285,0.175207),(0.331407,0.166622),(0.252127,0.166622),(0.23285,0.175207),(0.069955,0.172684),(0.0357329,0.254113),(0.113487,0.206332),(0.0142868,0.293567),(0.0594265,0.293566),(0.0357329,0.254113),(0.113487,0.249949),(0.150087,0.293566),(0.23285,0.254113),(0.113487,0.249949),(0.23285,0.254113),(0.222448,0.206332),(0.0594265,0.293566),(0.150087,0.293566),(0.113487,0.249949),(0.0357329,0.254113),(0.0594265,0.293566),(0.113487,0.249949),(0.0357329,0.254113),(0.113487,0.249949),(0.113487,0.206332),(0.113487,0.206332),(0.113487,0.249949),(0.222448,0.206332),(0.150087,0.293566),(0.252127,0.293566),(0.23285,0.254113),(0.252127,0.293566),(0.331408,0.26183),(0.23285,0.254113),(0.331408,0.293566),(0.331408,0.26183),(0.252127,0.293566),(0.505709,0.254113),(0.716421,0.296349),(0.611256,0.252031),(0.652076,0.293566),(0.716421,0.296349),(0.505709,0.254113),(0.652076,0.293566),(0.505709,0.254113),(0.471616,0.293566),(0.505709,0.254113),(0.524105,0.206332),(0.331407,0.220983),(0.471616,0.293566),(0.505709,0.254113),(0.331408,0.26183),(0.331408,0.26183),(0.505709,0.254113),(0.331407,0.220983),(0.331408,0.293566),(0.471616,0.293566),(0.331408,0.26183),(0.652076,0.293566),(0.807915,0.347877),(0.706565,0.340177),(0.807915,0.347877),(0.859209,0.34989),(0.706565,0.340177),(0.865929,0.374448),(0.859209,0.34989),(0.807915,0.347877),(0.865929,0.374448),(0.881891,0.20004),(0.859209,0.34989),(0.865929,0.374448),(0.874093,0.360137),(0.881891,0.20004),(0.00859314,0.322684),(0.0142868,0.293567),(0.0186266,0.274906),(0.0186266,0.274906),(0.0357329,0.254113),(0.069955,0.172684),(0.0186266,0.274906),(0.0142868,0.293567),(0.0357329,0.254113),(0.331408,0.36756),(0.471616,0.293566),(0.331408,0.293566),(0.505709,0.385552),(0.471616,0.293566),(0.331408,0.36756),(0.505709,0.385552),(0.652076,0.293566),(0.471616,0.293566),(0.716803,0.395261),(0.652076,0.293566),(0.505709,0.385552),(0.716803,0.395261),(0.807915,0.347877),(0.652076,0.293566),(0.716803,0.395261),(0.854312,0.385552),(0.807915,0.347877),(0.990372,0.954326),(0.854312,0.385552),(0.812409,0.462796),(0.854312,0.385552),(0.86593,0.374448),(0.807915,0.347877),(0.990372,0.954326),(0.86593,0.374448),(0.854312,0.385552),(0.990372,0.954326),(0.874093,0.360137),(0.86593,0.374448),(0.331407,0.462796),(0.505709,0.385552),(0.331408,0.36756),(0.524105,0.496955),(0.505709,0.385552),(0.331407,0.462796),(0.524105,0.496955),(0.716803,0.395261),(0.505709,0.385552),(0.716803,0.496955),(0.716803,0.395261),(0.524105,0.496955),(0.716803,0.496955),(0.854312,0.385552),(0.716803,0.395261),(0.716803,0.496955),(0.812409,0.462796),(0.854312,0.385552),(0.331407,0.547411),(0.524105,0.496955),(0.331407,0.462796),(0.505709,0.569523),(0.524105,0.496955),(0.331407,0.547411),(0.505709,0.569523),(0.716803,0.496955),(0.524105,0.496955),(0.505709,0.569523),(0.652075,0.547411),(0.716803,0.496955),(0.471616,0.58954),(0.652075,0.547411),(0.505709,0.569523),(0.331407,0.58954),(0.505709,0.569523),(0.331407,0.547411),(0.471616,0.58954),(0.505709,0.569523),(0.331407,0.58954),(0.471616,0.58954),(0.331407,0.58954),(0.331407,0.598648),(0.0181488,0.355173),(0.0142868,0.293567),(0.00859314,0.322684),(0.0181488,0.355173),(0.0357329,0.385552),(0.0142868,0.293567),(0.0142868,0.293567),(0.0357329,0.385552),(0.0594265,0.293566),(0.0357329,0.385552),(0.113487,0.395261),(0.0594265,0.293566),(0.0594265,0.293566),(0.113487,0.395261),(0.150087,0.293566),(0.113487,0.395261),(0.23285,0.385552),(0.150087,0.293566),(0.23285,0.385552),(0.252127,0.293566),(0.150087,0.293566),(0.23285,0.385552),(0.331408,0.36756),(0.252127,0.293566),(0.331408,0.36756),(0.331408,0.293566),(0.252127,0.293566),(0.0181488,0.355173),(0.0627818,0.403453),(0.0357329,0.385552),(0.0617955,0.405174),(0.120734,0.471213),(0.0357329,0.385552),(0.0357329,0.385552),(0.120734,0.471213),(0.113487,0.395261),(0.120734,0.471213),(0.194189,0.510512),(0.113487,0.395261),(0.194189,0.510512),(0.23285,0.385552),(0.113487,0.395261),(0.194189,0.510512),(0.331407,0.462796),(0.23285,0.385552),(0.331407,0.462796),(0.331408,0.36756),(0.23285,0.385552),(0.208039,0.542168),(0.120734,0.471213),(0.0617955,0.405174),(0.208039,0.542168),(0.212985,0.546125),(0.120734,0.471213),(0.214054,0.545781),(0.194189,0.510512),(0.120734,0.471213),(0.214054,0.545781),(0.331407,0.547411),(0.194189,0.510512),(0.331407,0.547411),(0.331407,0.462796),(0.194189,0.510512),(0.208039,0.542168),(0.253462,0.577642),(0.214054,0.545781),(0.253462,0.577642),(0.331407,0.58954),(0.214054,0.545781),(0.331407,0.58954),(0.331407,0.547411),(0.214054,0.545781),(0.331407,0.598648),(0.331407,0.58954),(0.253462,0.577642),(0.732242,0.505104),(0.812409,0.462796),(0.716803,0.496955),(0.652075,0.547411),(0.732242,0.505104),(0.716803,0.496955),(0.611256,0.252031),(0.716421,0.296349),(0.524105,0.206332),(0.505709,0.254113),(0.611256,0.252031),(0.524105,0.206332),(0.809198,0.337614),(0.722004,0.293065),(0.706565,0.29656),(0.706565,0.29656),(0.722004,0.293065),(0.643083,0.251274),(0.270455,0.472727),(0.306818,0.436364),(0.134091,0.345455),(0.642022,0.469944),(0.660988,0.375635),(0.696553,0.266068),(0.711659,0.530056),(0.660988,0.375635),(0.642022,0.469944),(0.656081,0.508732),(0.660447,0.316632),(0.577495,0.351559),(0.881891,0.20004),(0.859209,0.34989),(0.809198,0.337614),(0.706565,0.29656),(0.809198,0.337614),(0.859209,0.34989),(0.706565,0.340177),(0.706565,0.29656),(0.859209,0.34989),(0.524105,0.206332),(0.715943,0.279149),(0.716421,0.296349),(0.505709,0.175207),(0.650009,0.239016),(0.715943,0.279149),(0.505709,0.175207),(0.471616,0.166622),(0.650009,0.239016),(0.524105,0.206332),(0.505709,0.175207),(0.715943,0.279149),(0.331407,0.184691),(0.505709,0.175207),(0.524105,0.206332),(0.331407,0.184691),(0.524105,0.206332),(0.331407,0.220983),(0.331407,0.166622),(0.471616,0.166622),(0.505709,0.175207),(0.331407,0.166622),(0.331407,0.162714),(0.471616,0.166622),(0.331407,0.184691),(0.331407,0.166622),(0.505709,0.175207),(0.331407,0.166622),(0.252127,0.166622),(0.331407,0.162714),(0.23285,0.254113),(0.331407,0.220983),(0.331408,0.26183),(0.222448,0.206332),(0.331407,0.220983),(0.23285,0.254113),(0.222448,0.206332),(0.331407,0.184691),(0.331407,0.220983),(0.151595,0.138755),(0.113487,0.206332),(0.069955,0.172684),(0.222448,0.206332),(0.23285,0.175207),(0.331407,0.184691),(0.331407,0.184691),(0.23285,0.175207),(0.331407,0.166622),(0.222448,0.206332),(0.113487,0.206332),(0.23285,0.175207),(0.23285,0.175207),(0.113487,0.206332),(0.151595,0.138755),(0.23285,0.175207),(0.151595,0.138755),(0.252127,0.166622),(0.23285,0.175207),(0.252127,0.166622),(0.331407,0.166622),(0.113487,0.206332),(0.0357329,0.254113),(0.069955,0.172684),(0.0357329,0.254113),(0.0594265,0.293566),(0.0142868,0.293567),(0.23285,0.254113),(0.150087,0.293566),(0.113487,0.249949),(0.222448,0.206332),(0.23285,0.254113),(0.113487,0.249949),(0.113487,0.249949),(0.150087,0.293566),(0.0594265,0.293566),(0.113487,0.249949),(0.0594265,0.293566),(0.0357329,0.254113),(0.113487,0.206332),(0.113487,0.249949),(0.0357329,0.254113),(0.222448,0.206332),(0.113487,0.249949),(0.113487,0.206332),(0.23285,0.254113),(0.252127,0.293566),(0.150087,0.293566),(0.23285,0.254113),(0.331408,0.26183),(0.252127,0.293566),(0.252127,0.293566),(0.331408,0.26183),(0.331408,0.293566),(0.611256,0.252031),(0.716421,0.296349),(0.505709,0.254113),(0.505709,0.254113),(0.716421,0.296349),(0.652076,0.293566),(0.471616,0.293566),(0.505709,0.254113),(0.652076,0.293566),(0.331407,0.220983),(0.524105,0.206332),(0.505709,0.254113),(0.331408,0.26183),(0.505709,0.254113),(0.471616,0.293566),(0.331407,0.220983),(0.505709,0.254113),(0.331408,0.26183),(0.331408,0.26183),(0.471616,0.293566),(0.331408,0.293566),(0.706565,0.340177),(0.807915,0.347877),(0.652076,0.293566),(0.706565,0.340177),(0.859209,0.34989),(0.807915,0.347877),(0.807915,0.347877),(0.859209,0.34989),(0.865929,0.374448),(0.859209,0.34989),(0.881891,0.20004),(0.865929,0.374448),(0.881891,0.20004),(0.874093,0.360137),(0.865929,0.374448),(0.0186266,0.274906),(0.0142868,0.293567),(0.00859314,0.322684),(0.069955,0.172684),(0.0357329,0.254113),(0.0186266,0.274906),(0.0357329,0.254113),(0.0142868,0.293567),(0.0186266,0.274906),(0.331408,0.293566),(0.471616,0.293566),(0.331408,0.36756),(0.331408,0.36756),(0.471616,0.293566),(0.505709,0.385552),(0.471616,0.293566),(0.652076,0.293566),(0.505709,0.385552),(0.505709,0.385552),(0.652076,0.293566),(0.716803,0.395261),(0.652076,0.293566),(0.807915,0.347877),(0.716803,0.395261),(0.807915,0.347877),(0.854312,0.385552),(0.716803,0.395261),(0.812409,0.462796),(0.854312,0.385552),(0.990372,0.954326),(0.807915,0.347877),(0.86593,0.374448),(0.854312,0.385552),(0.854312,0.385552),(0.86593,0.374448),(0.990372,0.954326),(0.86593,0.374448),(0.874093,0.360137),(0.990372,0.954326),(0.331408,0.36756),(0.505709,0.385552),(0.331407,0.462796),(0.331407,0.462796),(0.505709,0.385552),(0.524105,0.496955),(0.505709,0.385552),(0.716803,0.395261),(0.524105,0.496955),(0.524105,0.496955),(0.716803,0.395261),(0.716803,0.496955),(0.716803,0.395261),(0.854312,0.385552),(0.716803,0.496955),(0.854312,0.385552),(0.812409,0.462796),(0.716803,0.496955),(0.331407,0.462796),(0.524105,0.496955),(0.331407,0.547411),(0.331407,0.547411),(0.524105,0.496955),(0.505709,0.569523),(0.524105,0.496955),(0.716803,0.496955),(0.505709,0.569523),(0.716803,0.496955),(0.652075,0.547411),(0.505709,0.569523),(0.505709,0.569523),(0.652075,0.547411),(0.471616,0.58954),(0.331407,0.547411),(0.505709,0.569523),(0.331407,0.58954),(0.331407,0.58954),(0.505709,0.569523),(0.471616,0.58954),(0.331407,0.598648),(0.331407,0.58954),(0.471616,0.58954),(0.00859314,0.322684),(0.0142868,0.293567),(0.0181488,0.355173),(0.0142868,0.293567),(0.0357329,0.385552),(0.0181488,0.355173),(0.0594265,0.293566),(0.0357329,0.385552),(0.0142868,0.293567),(0.0594265,0.293566),(0.113487,0.395261),(0.0357329,0.385552),(0.150087,0.293566),(0.113487,0.395261),(0.0594265,0.293566),(0.150087,0.293566),(0.23285,0.385552),(0.113487,0.395261),(0.150087,0.293566),(0.252127,0.293566),(0.23285,0.385552),(0.252127,0.293566),(0.331408,0.36756),(0.23285,0.385552),(0.252127,0.293566),(0.331408,0.293566),(0.331408,0.36756),(0.0357329,0.385552),(0.0617955,0.405174),(0.0181488,0.355173),(0.0357329,0.385552),(0.120734,0.471213),(0.0617955,0.405174),(0.113487,0.395261),(0.120734,0.471213),(0.0357329,0.385552),(0.113487,0.395261),(0.194189,0.510512),(0.120734,0.471213),(0.113487,0.395261),(0.23285,0.385552),(0.194189,0.510512),(0.23285,0.385552),(0.331407,0.462796),(0.194189,0.510512),(0.23285,0.385552),(0.331408,0.36756),(0.331407,0.462796),(0.0617955,0.405174),(0.120734,0.471213),(0.208039,0.542168),(0.120734,0.471213),(0.214054,0.545781),(0.208039,0.542168),(0.120734,0.471213),(0.194189,0.510512),(0.214054,0.545781),(0.194189,0.510512),(0.331407,0.547411),(0.214054,0.545781),(0.194189,0.510512),(0.331407,0.462796),(0.331407,0.547411),(0.214054,0.545781),(0.253462,0.577642),(0.208039,0.542168),(0.214054,0.545781),(0.331407,0.58954),(0.253462,0.577642),(0.214054,0.545781),(0.331407,0.547411),(0.331407,0.58954),(0.253462,0.577642),(0.331407,0.58954),(0.331407,0.598648),(0.716803,0.496955),(0.812409,0.462796),(0.732242,0.505104),(0.716803,0.496955),(0.732242,0.505104),(0.652075,0.547411),(0.524105,0.206332),(0.716421,0.296349),(0.611256,0.252031),(0.524105,0.206332),(0.611256,0.252031),(0.505709,0.254113),(0.706565,0.29656),(0.722004,0.293065),(0.809198,0.337614),(0.188802,0.479106),(0.214171,0.305666),(0.16506,0.304996),(0.149232,0.399965),(0.157146,0.210027),(0.0819621,0.304996)])))]),
      Transform(center=(-0.00303999,0.898843,-0.091817),rotation=(0,1,0,1.57079),translation=(0.00303999,-0.898843,0.049247),
        children=[
        Shape(
          appearance=Appearance(
            material=Material(USE='SharkMaterial')),
          geometry=IndexedFaceSet(USE='_0'))]),
      WorldInfo(info=["null node"])]),
    Group(DEF='AnimationGroup',
      children=[
      CoordinateInterpolator(DEF='MeshInterpolator',key=[0,0.2,0.3,0.5,0.7,0.8,1],keyValue=[(-0.00264051,0.707736,-0.373331),(-0.00264151,0.7163,-0.526765),(0.035654,0.717949,-0.497215),(1.05985E-6,0.934617,0.0504887),(1.80025E-6,0.898872,-0.0279467),(0.0,0.901021,0.122661),(1.05985E-6,0.934617,0.0504887),(0.0,0.901021,0.122661),(-2.03005E-6,0.90317,0.277591),(1.06986E-6,1.0793,3.47475E-7),(1.05985E-6,0.934617,0.0504887),(-2.03005E-6,0.90317,0.277591),(-0.00262399,0.901562,-0.477891),(-0.00262401,0.85895,-0.493321),(-0.00262701,0.874364,-0.406775),(-0.00262401,0.948969,-0.493766),(-0.00262399,0.901562,-0.477891),(-0.00262701,0.874364,-0.406775),(-0.002627,0.693025,-0.620407),(-0.0026265,0.652374,-0.645043),(-0.002627,0.684348,-0.585814),(-0.00262751,0.716322,-0.526587),(-0.002627,0.693025,-0.620407),(-0.002627,0.684348,-0.585814),(-0.00262699,0.723428,-0.636308),(-0.002627,0.693025,-0.620407),(-0.00262751,0.716322,-0.526587),(-0.103485,0.739931,-0.0978782),(-0.0944054,0.734624,-0.239595),(-0.106573,0.693107,-0.168737),(-0.0944054,0.734624,-0.239595),(-0.102032,0.690461,-0.237269),(-0.106573,0.693107,-0.168737),(-0.106573,0.693107,-0.168737),(-0.102032,0.690461,-0.237269),(-0.109639,0.648327,-0.275536),(-0.118464,0.756954,0.588829),(-0.134253,0.756954,0.279345),(-0.302092,0.706351,0.270908),(-0.00783302,0.724864,-0.700279),(-0.0478305,0.740499,-0.792884),(-0.00783151,0.65237,-0.967155),(-0.0478305,0.740499,-0.792884),(-0.00783302,0.724864,-0.700279),(-0.0409195,0.717949,-0.497215),(-0.0478305,0.740499,-0.792884),(-0.0409195,0.717949,-0.497215),(-0.0792145,0.738534,-0.497214),(-0.0792145,0.738534,-0.497214),(-0.0409195,0.717949,-0.497215),(-0.079217,0.717949,-0.128407),(-0.0409195,0.717949,-0.497215),(-0.00262501,0.707736,-0.373331),(-0.037266,0.70326,-0.0931975),(-0.00262501,0.707736,-0.373331),(-0.0026271,0.699209,-0.0279467),(-0.037266,0.70326,-0.0931975),(-0.0409195,0.717949,-0.497215),(-0.0372659,0.70326,-0.0931975),(-0.079217,0.717949,-0.128407),(-0.079217,0.717949,-0.128407),(-0.037266,0.70326,-0.0931975),(-0.0766211,0.707736,0.27759),(-0.113414,0.724864,0.27759),(-0.079217,0.717949,-0.128407),(-0.0766211,0.707736,0.27759),(-0.037266,0.70326,-0.0931975),(-0.0026271,0.699209,-0.0279467),(-0.0352099,0.699209,0.27759),(-0.0026271,0.699209,-0.0279467),(-0.0030355,0.697365,0.277591),(-0.0352099,0.699209,0.27759),(-0.037266,0.70326,-0.0931975),(-0.0352099,0.699209,0.27759),(-0.0766211,0.707736,0.27759),(-0.00303552,0.697365,0.277591),(-0.00303664,0.699209,0.4528),(-0.0352099,0.699209,0.27759),(-0.131733,0.744141,0.27759),(-0.113414,0.724864,0.27759),(-0.123031,0.740499,0.495402),(-0.123031,0.740499,0.495402),(-0.113414,0.724864,0.27759),(-0.091476,0.717949,0.518391),(-0.113414,0.724864,0.27759),(-0.0766211,0.707736,0.27759),(-0.091476,0.717949,0.518391),(-0.00303936,0.724864,0.878668),(-0.047258,0.717949,0.759194),(-0.00303808,0.707736,0.678309),(-0.0766211,0.707736,0.27759),(-0.0430349,0.70326,0.495403),(-0.091476,0.717949,0.518391),(-0.0352099,0.699209,0.27759),(-0.0430349,0.70326,0.495403),(-0.0766211,0.707736,0.27759),(-0.0430349,0.70326,0.495403),(-0.047258,0.717949,0.759194),(-0.091476,0.717949,0.518391),(-0.00303808,0.707736,0.678309),(-0.047258,0.717949,0.759194),(-0.0430349,0.70326,0.495403),(-0.00303664,0.699209,0.4528),(-0.00303808,0.707736,0.678309),(-0.0430349,0.70326,0.495403),(-0.0352099,0.699209,0.27759),(-0.00303664,0.699209,0.4528),(-0.0430349,0.70326,0.495403),(-0.00303936,0.724864,0.878668),(-0.0430376,0.740499,0.931031),(-0.047258,0.717949,0.759194),(-0.0352143,0.759118,0.978427),(-0.076625,0.759118,0.878668),(-0.0430376,0.740499,0.931031),(-0.0914776,0.738534,0.759194),(-0.113416,0.759118,0.678308),(-0.123031,0.740499,0.495402),(-0.0914776,0.738534,0.759194),(-0.123031,0.740499,0.495402),(-0.091476,0.717949,0.518391),(-0.076625,0.759118,0.878668),(-0.113416,0.759118,0.678308),(-0.0914776,0.738534,0.759194),(-0.0430376,0.740499,0.931031),(-0.076625,0.759118,0.878668),(-0.0914776,0.738534,0.759194),(-0.0430376,0.740499,0.931031),(-0.0914776,0.738534,0.759194),(-0.047258,0.717949,0.759194),(-0.047258,0.717949,0.759194),(-0.0914776,0.738534,0.759194),(-0.091476,0.717949,0.518391),(-0.113416,0.759118,0.678308),(-0.131734,0.759118,0.452799),(-0.123031,0.740499,0.495402),(-0.131734,0.759118,0.452799),(-0.131733,0.744141,0.27759),(-0.123031,0.740499,0.495402),(-0.135694,0.759118,0.27759),(-0.131733,0.744141,0.27759),(-0.131734,0.759118,0.452799),(-0.106544,0.740499,-0.0931984),(-0.0792145,0.738534,-0.497214),(-0.0928796,0.739516,-0.295207),(-0.098215,0.759118,-0.373333),(-0.0792145,0.738534,-0.497214),(-0.106544,0.740499,-0.0931984),(-0.098215,0.759118,-0.373333),(-0.106544,0.740499,-0.0931984),(-0.131731,0.759118,-0.0322708),(-0.106544,0.740499,-0.0931984),(-0.079217,0.717949,-0.128407),(-0.113414,0.724864,0.27759),(-0.131731,0.759118,-0.0322708),(-0.106544,0.740499,-0.0931984),(-0.131733,0.744141,0.27759),(-0.131733,0.744141,0.27759),(-0.106544,0.740499,-0.0931984),(-0.113414,0.724864,0.27759),(-0.135694,0.759118,0.27759),(-0.131731,0.759118,-0.0322708),(-0.131733,0.744141,0.27759),(-0.098215,0.759118,-0.373333),(-0.081419,0.759118,-0.700279),(-0.0792145,0.738534,-0.497214),(-0.081419,0.759118,-0.700279),(-0.0478305,0.740499,-0.792884),(-0.0792145,0.738534,-0.497214),(-0.040006,0.759118,-0.876702),(-0.0478305,0.740499,-0.792884),(-0.081419,0.759118,-0.700279),(-0.040006,0.759118,-0.876702),(-0.00783151,0.65237,-0.967155),(-0.0478305,0.740499,-0.792884),(-0.040006,0.759118,-0.876702),(-0.007832,0.759118,-0.914845),(-0.00783151,0.65237,-0.967155),(-0.00304013,0.759118,0.999996),(-0.0352143,0.759118,0.978427),(-0.00304,0.744141,0.978427),(-0.00304,0.744141,0.978427),(-0.0430376,0.740499,0.931031),(-0.00303936,0.724864,0.878668),(-0.00304,0.744141,0.978427),(-0.0352143,0.759118,0.978427),(-0.0430376,0.740499,0.931031),(-0.131733,0.794038,0.27759),(-0.131731,0.759118,-0.0322708),(-0.135694,0.759118,0.27759),(-0.106544,0.80253,-0.0931984),(-0.131731,0.759118,-0.0322708),(-0.131733,0.794038,0.27759),(-0.106544,0.80253,-0.0931984),(-0.098215,0.759118,-0.373333),(-0.131731,0.759118,-0.0322708),(-0.0792145,0.807112,-0.497214),(-0.098215,0.759118,-0.373333),(-0.106544,0.80253,-0.0931984),(-0.0792145,0.807112,-0.497214),(-0.081419,0.759118,-0.700279),(-0.098215,0.759118,-0.373333),(-0.0792145,0.807112,-0.497214),(-0.0478305,0.80253,-0.792884),(-0.081419,0.759118,-0.700279),(-0.00783101,1.14936,-1.09848),(-0.0478305,0.80253,-0.792884),(-0.00783302,0.838984,-0.700279),(-0.0478305,0.80253,-0.792884),(-0.040006,0.759118,-0.876706),(-0.081419,0.759118,-0.700279),(-0.00783101,1.14936,-1.09848),(-0.040006,0.759118,-0.876706),(-0.0478305,0.80253,-0.792884),(-0.00783101,1.14936,-1.09848),(-0.007832,0.759118,-0.914845),(-0.040006,0.759118,-0.876706),(-0.113414,0.838984,0.27759),(-0.106544,0.80253,-0.0931984),(-0.131733,0.794038,0.27759),(-0.079217,0.855105,-0.128407),(-0.106544,0.80253,-0.0931984),(-0.113414,0.838984,0.27759),(-0.079217,0.855105,-0.128407),(-0.0792145,0.807112,-0.497214),(-0.106544,0.80253,-0.0931984),(-0.0409195,0.855105,-0.497215),(-0.0792145,0.807112,-0.497214),(-0.079217,0.855105,-0.128407),(-0.0409195,0.855105,-0.497215),(-0.0478305,0.80253,-0.792884),(-0.0792145,0.807112,-0.497214),(-0.0409195,0.855105,-0.497215),(-0.00783302,0.838984,-0.700279),(-0.0478305,0.80253,-0.792884),(-0.076621,0.878917,0.27759),(-0.079217,0.855105,-0.128407),(-0.113414,0.838984,0.27759),(-0.037266,0.889352,-0.0931975),(-0.079217,0.855105,-0.128407),(-0.076621,0.878917,0.27759),(-0.037266,0.889352,-0.0931975),(-0.0409195,0.855105,-0.497215),(-0.079217,0.855105,-0.128407),(-0.037266,0.889352,-0.0931975),(-0.00262501,0.878917,-0.373331),(-0.0409195,0.855105,-0.497215),(-0.0026271,0.898799,-0.0279467),(-0.00262501,0.878917,-0.373331),(-0.037266,0.889352,-0.0931975),(-0.0352099,0.898799,0.27759),(-0.037266,0.889352,-0.0931975),(-0.076621,0.878917,0.27759),(-0.0026271,0.898799,-0.0279467),(-0.037266,0.889352,-0.0931975),(-0.0352099,0.898799,0.27759),(-0.0026271,0.898799,-0.0279467),(-0.0352099,0.898799,0.27759),(-0.0030355,0.903097,0.277591),(-0.00303999,0.794038,0.978427),(-0.0352143,0.759118,0.978427),(-0.00304013,0.759118,0.999996),(-0.00303999,0.794038,0.978427),(-0.0430376,0.80253,0.931031),(-0.0352143,0.759118,0.978427),(-0.0352143,0.759118,0.978427),(-0.0430376,0.80253,0.931031),(-0.076625,0.759118,0.878668),(-0.0430376,0.80253,0.931031),(-0.0914776,0.807112,0.759194),(-0.076625,0.759118,0.878668),(-0.076625,0.759118,0.878668),(-0.0914776,0.807112,0.759194),(-0.113416,0.759118,0.678308),(-0.0914776,0.807112,0.759194),(-0.123031,0.80253,0.495402),(-0.113416,0.759118,0.678308),(-0.123031,0.80253,0.495402),(-0.131734,0.759118,0.452799),(-0.113416,0.759118,0.678308),(-0.123031,0.80253,0.495402),(-0.131733,0.794038,0.27759),(-0.131734,0.759118,0.452799),(-0.131733,0.794038,0.27759),(-0.135694,0.759118,0.27759),(-0.131734,0.759118,0.452799),(-0.00303999,0.794038,0.978427),(-0.00303936,0.838984,0.878668),(-0.0430376,0.80253,0.931031),(-0.00303936,0.838984,0.878668),(-0.047258,0.855105,0.759194),(-0.0430376,0.80253,0.931031),(-0.0430376,0.80253,0.931031),(-0.047258,0.855105,0.759194),(-0.0914776,0.807112,0.759194),(-0.047258,0.855105,0.759194),(-0.091476,0.855105,0.518391),(-0.0914776,0.807112,0.759194),(-0.091476,0.855105,0.518391),(-0.123031,0.80253,0.495402),(-0.0914776,0.807112,0.759194),(-0.091476,0.855105,0.518391),(-0.113414,0.838984,0.27759),(-0.123031,0.80253,0.495402),(-0.113414,0.838984,0.27759),(-0.131733,0.794038,0.27759),(-0.123031,0.80253,0.495402),(-0.00303808,0.878917,0.678309),(-0.047258,0.855105,0.759194),(-0.00303936,0.838984,0.878668),(-0.00303808,0.878917,0.678309),(-0.0430349,0.889352,0.495403),(-0.047258,0.855105,0.759194),(-0.0430349,0.889352,0.495403),(-0.091476,0.855105,0.518391),(-0.047258,0.855105,0.759194),(-0.0430349,0.889352,0.495403),(-0.076621,0.878917,0.27759),(-0.091476,0.855105,0.518391),(-0.076621,0.878917,0.27759),(-0.113414,0.838984,0.27759),(-0.091476,0.855105,0.518391),(-0.00303808,0.878917,0.678309),(-0.00303664,0.898799,0.4528),(-0.0430349,0.889352,0.495403),(-0.00303664,0.898799,0.4528),(-0.0352099,0.898799,0.27759),(-0.0430349,0.889352,0.495403),(-0.0352099,0.898799,0.27759),(-0.076621,0.878917,0.27759),(-0.0430349,0.889352,0.495403),(-0.00303551,0.903097,0.277591),(-0.0352099,0.898799,0.27759),(-0.00303664,0.898799,0.4528),(-0.00262401,0.85895,-0.526765),(-0.00783302,0.838984,-0.700279),(-0.0409195,0.855105,-0.497215),(-0.00262501,0.878917,-0.373331),(-0.00262401,0.85895,-0.526765),(-0.0409195,0.855105,-0.497215),(-0.0928796,0.739516,-0.295207),(-0.0792145,0.738534,-0.497214),(-0.079217,0.717949,-0.128407),(-0.106544,0.740499,-0.0931984),(-0.0928796,0.739516,-0.295207),(-0.079217,0.717949,-0.128407),(-0.00783302,0.724864,-0.700279),(-0.00262401,0.7163,-0.526765),(-0.0409195,0.717949,-0.497215),(-0.0409195,0.717949,-0.497215),(-0.00262401,0.7163,-0.526765),(-0.00262501,0.707736,-0.373331),(0.101308,0.693107,-0.168737),(0.0891401,0.734624,-0.239595),(0.0982192,0.739931,-0.0978782),(0.101308,0.693107,-0.168737),(0.0967671,0.690461,-0.237269),(0.0891401,0.734624,-0.239595),(0.104374,0.648327,-0.275536),(0.0967671,0.690461,-0.237269),(0.101308,0.693107,-0.168737),(0.296012,0.706351,0.270908),(0.128173,0.756954,0.279345),(0.112384,0.756954,0.588829),(-0.00785749,0.65237,-0.967155),(0.032142,0.740499,-0.792884),(-0.00785601,0.724864,-0.700279),(0.035654,0.717949,-0.497215),(-0.00785601,0.724864,-0.700279),(0.032142,0.740499,-0.792884),(0.073949,0.738534,-0.497215),(0.035654,0.717949,-0.497215),(0.032142,0.740499,-0.792884),(0.0739513,0.717949,-0.128407),(0.035654,0.717949,-0.497215),(0.073949,0.738534,-0.497215),(0.0320005,0.70326,-0.0931975),(-0.00264051,0.707736,-0.373331),(0.035654,0.717949,-0.497215),(0.0320005,0.70326,-0.0931975),(-0.0026383,0.699209,-0.0279467),(-0.00264051,0.707736,-0.373331),(0.0739513,0.717949,-0.128407),(0.0320005,0.70326,-0.0931975),(0.035654,0.717949,-0.497215),(0.0705411,0.707736,0.27759),(0.0320005,0.70326,-0.0931975),(0.0739513,0.717949,-0.128407),(0.0705411,0.707736,0.27759),(0.0739513,0.717949,-0.128407),(0.107334,0.724864,0.27759),(0.0291299,0.699209,0.27759),(-0.0026383,0.699209,-0.0279467),(0.0320005,0.70326,-0.0931975),(0.0291299,0.699209,0.27759),(-0.00304449,0.697365,0.277591),(-0.0026383,0.699209,-0.0279467),(0.0705411,0.707736,0.27759),(0.0291299,0.699209,0.27759),(0.0320005,0.70326,-0.0931975),(0.0291299,0.699209,0.27759),(-0.00304335,0.699209,0.4528),(-0.00304447,0.697365,0.277591),(0.116951,0.740499,0.495402),(0.107334,0.724864,0.27759),(0.125653,0.744141,0.27759),(0.085396,0.717949,0.518391),(0.107334,0.724864,0.27759),(0.116951,0.740499,0.495402),(0.085396,0.717949,0.518391),(0.0705411,0.707736,0.27759),(0.107334,0.724864,0.27759),(-0.00304191,0.707736,0.678309),(0.041178,0.717949,0.759194),(-0.00304063,0.724864,0.878668),(0.085396,0.717949,0.518391),(0.0369549,0.70326,0.495403),(0.0705411,0.707736,0.27759),(0.0705411,0.707736,0.27759),(0.0369549,0.70326,0.495403),(0.0291299,0.699209,0.27759),(0.085396,0.717949,0.518391),(0.041178,0.717949,0.759194),(0.0369549,0.70326,0.495403),(0.0369549,0.70326,0.495403),(0.041178,0.717949,0.759194),(-0.00304191,0.707736,0.678309),(0.0369549,0.70326,0.495403),(-0.00304191,0.707736,0.678309),(-0.00304335,0.699209,0.4528),(0.0369549,0.70326,0.495403),(-0.00304335,0.699209,0.4528),(0.0291299,0.699209,0.27759),(0.041178,0.717949,0.759194),(0.0369577,0.740499,0.931031),(-0.00304063,0.724864,0.878668),(0.0369577,0.740499,0.931031),(0.070545,0.759118,0.878668),(0.0291344,0.759118,0.978427),(0.116951,0.740499,0.495402),(0.107336,0.759118,0.678308),(0.0853976,0.738534,0.759194),(0.085396,0.717949,0.518391),(0.116951,0.740499,0.495402),(0.0853976,0.738534,0.759194),(0.0853976,0.738534,0.759194),(0.107336,0.759118,0.678308),(0.070545,0.759118,0.878668),(0.0853976,0.738534,0.759194),(0.070545,0.759118,0.878668),(0.0369577,0.740499,0.931031),(0.041178,0.717949,0.759194),(0.0853976,0.738534,0.759194),(0.0369577,0.740499,0.931031),(0.085396,0.717949,0.518391),(0.0853976,0.738534,0.759194),(0.041178,0.717949,0.759194),(0.116951,0.740499,0.495402),(0.125654,0.759118,0.452799),(0.107336,0.759118,0.678308),(0.116951,0.740499,0.495402),(0.125653,0.744141,0.27759),(0.125654,0.759118,0.452799),(0.125654,0.759118,0.452799),(0.125653,0.744141,0.27759),(0.129614,0.759118,0.27759),(0.0876139,0.739516,-0.295207),(0.073949,0.738534,-0.497215),(0.101279,0.740499,-0.0931984),(0.101279,0.740499,-0.0931984),(0.073949,0.738534,-0.497215),(0.0929495,0.759118,-0.373333),(0.108817,0.759118,-0.0279473),(0.101279,0.740499,-0.0931984),(0.0929495,0.759118,-0.373333),(0.107334,0.724864,0.27759),(0.0739513,0.717949,-0.128407),(0.101279,0.740499,-0.0931984),(0.125653,0.744141,0.27759),(0.101279,0.740499,-0.0931984),(0.108817,0.759118,-0.0279473),(0.107334,0.724864,0.27759),(0.101279,0.740499,-0.0931984),(0.125653,0.744141,0.27759),(0.125653,0.744141,0.27759),(0.108817,0.759118,-0.0279473),(0.129614,0.759118,0.27759),(0.073949,0.738534,-0.497215),(0.06573,0.759118,-0.70028),(0.0929495,0.759118,-0.373333),(0.073949,0.738534,-0.497215),(0.032142,0.740499,-0.792884),(0.06573,0.759118,-0.70028),(0.06573,0.759118,-0.70028),(0.032142,0.740499,-0.792884),(0.024318,0.759118,-0.876703),(0.032142,0.740499,-0.792884),(-0.00785749,0.65237,-0.967155),(0.024318,0.759118,-0.876703),(-0.00785749,0.65237,-0.967155),(-0.00785699,0.759118,-0.914845),(0.024318,0.759118,-0.876703),(-0.00303999,0.744141,0.978427),(0.0291344,0.759118,0.978427),(-0.00303985,0.759118,0.999996),(-0.00304063,0.724864,0.878668),(0.0369577,0.740499,0.931031),(-0.00303999,0.744141,0.978427),(0.0369577,0.740499,0.931031),(0.0291344,0.759118,0.978427),(-0.00303999,0.744141,0.978427),(0.129614,0.759118,0.27759),(0.108817,0.759118,-0.0279473),(0.125653,0.794038,0.27759),(0.125653,0.794038,0.27759),(0.108817,0.759118,-0.0279473),(0.101279,0.80253,-0.0931984),(0.108817,0.759118,-0.0279473),(0.0929495,0.759118,-0.373333),(0.101279,0.80253,-0.0931984),(0.101279,0.80253,-0.0931984),(0.0929495,0.759118,-0.373333),(0.073949,0.807112,-0.497215),(0.0929495,0.759118,-0.373333),(0.06573,0.759118,-0.70028),(0.073949,0.807112,-0.497215),(0.06573,0.759118,-0.70028),(0.032142,0.80253,-0.792884),(0.073949,0.807112,-0.497215),(-0.00785601,0.838984,-0.700279),(0.032142,0.80253,-0.792884),(-0.00785802,1.14936,-1.09849),(0.06573,0.759118,-0.70028),(0.0243175,0.759118,-0.876706),(0.032142,0.80253,-0.792884),(0.032142,0.80253,-0.792884),(0.0243175,0.759118,-0.876706),(-0.00785802,1.14936,-1.09849),(0.0243175,0.759118,-0.876706),(-0.00785699,0.759118,-0.914845),(-0.00785802,1.14936,-1.09849),(0.125653,0.794038,0.27759),(0.101279,0.80253,-0.0931984),(0.107334,0.838984,0.27759),(0.107334,0.838984,0.27759),(0.101279,0.80253,-0.0931984),(0.0739513,0.855105,-0.128407),(0.101279,0.80253,-0.0931984),(0.073949,0.807112,-0.497215),(0.0739513,0.855105,-0.128407),(0.0739513,0.855105,-0.128407),(0.073949,0.807112,-0.497215),(0.035654,0.855105,-0.497215),(0.073949,0.807112,-0.497215),(0.032142,0.80253,-0.792884),(0.035654,0.855105,-0.497215),(0.032142,0.80253,-0.792884),(-0.00785601,0.838984,-0.700279),(0.035654,0.855105,-0.497215),(0.107334,0.838984,0.27759),(0.0739513,0.855105,-0.128407),(0.070541,0.878917,0.27759),(0.070541,0.878917,0.27759),(0.0739513,0.855105,-0.128407),(0.0320005,0.889352,-0.0931975),(0.0739513,0.855105,-0.128407),(0.035654,0.855105,-0.497215),(0.0320005,0.889352,-0.0931975),(0.035654,0.855105,-0.497215),(-0.00264051,0.878917,-0.373331),(0.0320005,0.889352,-0.0931975),(0.0320005,0.889352,-0.0931975),(-0.00264051,0.878917,-0.373331),(-0.0026383,0.898799,-0.0279467),(0.070541,0.878917,0.27759),(0.0320005,0.889352,-0.0931975),(0.0291299,0.898799,0.27759),(0.0291299,0.898799,0.27759),(0.0320005,0.889352,-0.0931975),(-0.0026383,0.898799,-0.0279467),(-0.00304449,0.903097,0.277591),(0.0291299,0.898799,0.27759),(-0.0026383,0.898799,-0.0279467),(-0.00303986,0.759118,0.999996),(0.0291344,0.759118,0.978427),(-0.00303999,0.794038,0.978427),(0.0291344,0.759118,0.978427),(0.0369577,0.80253,0.931031),(-0.00303999,0.794038,0.978427),(0.070545,0.759118,0.878668),(0.0369577,0.80253,0.931031),(0.0291344,0.759118,0.978427),(0.070545,0.759118,0.878668),(0.0853976,0.807112,0.759194),(0.0369577,0.80253,0.931031),(0.107336,0.759118,0.678308),(0.0853976,0.807112,0.759194),(0.070545,0.759118,0.878668),(0.107336,0.759118,0.678308),(0.116951,0.80253,0.495402),(0.0853976,0.807112,0.759194),(0.107336,0.759118,0.678308),(0.125654,0.759118,0.452799),(0.116951,0.80253,0.495402),(0.125654,0.759118,0.452799),(0.125653,0.794038,0.27759),(0.116951,0.80253,0.495402),(0.125654,0.759118,0.452799),(0.129614,0.759118,0.27759),(0.125653,0.794038,0.27759),(0.0369577,0.80253,0.931031),(-0.00304063,0.838984,0.878668),(-0.00303999,0.794038,0.978427),(0.0369577,0.80253,0.931031),(0.041178,0.855105,0.759194),(-0.00304063,0.838984,0.878668),(0.0853976,0.807112,0.759194),(0.041178,0.855105,0.759194),(0.0369577,0.80253,0.931031),(0.0853976,0.807112,0.759194),(0.085396,0.855105,0.518391),(0.041178,0.855105,0.759194),(0.0853976,0.807112,0.759194),(0.116951,0.80253,0.495402),(0.085396,0.855105,0.518391),(0.116951,0.80253,0.495402),(0.107334,0.838984,0.27759),(0.085396,0.855105,0.518391),(0.116951,0.80253,0.495402),(0.125653,0.794038,0.27759),(0.107334,0.838984,0.27759),(-0.00304063,0.838984,0.878668),(0.041178,0.855105,0.759194),(-0.00304191,0.878917,0.678309),(0.041178,0.855105,0.759194),(0.0369549,0.889352,0.495403),(-0.00304191,0.878917,0.678309),(0.041178,0.855105,0.759194),(0.085396,0.855105,0.518391),(0.0369549,0.889352,0.495403),(0.085396,0.855105,0.518391),(0.070541,0.878917,0.27759),(0.0369549,0.889352,0.495403),(0.085396,0.855105,0.518391),(0.107334,0.838984,0.27759),(0.070541,0.878917,0.27759),(0.0369549,0.889352,0.495403),(-0.00304335,0.898799,0.4528),(-0.00304191,0.878917,0.678309),(0.0369549,0.889352,0.495403),(0.0291299,0.898799,0.27759),(-0.00304335,0.898799,0.4528),(0.0369549,0.889352,0.495403),(0.070541,0.878917,0.27759),(0.0291299,0.898799,0.27759),(-0.00304335,0.898799,0.4528),(0.0291299,0.898799,0.27759),(-0.00304448,0.903097,0.277591),(0.035654,0.855105,-0.497215),(-0.00785601,0.838984,-0.700279),(-0.00264151,0.85895,-0.526765),(0.035654,0.855105,-0.497215),(-0.00264151,0.85895,-0.526765),(-0.00264051,0.878917,-0.373331),(0.0739513,0.717949,-0.128407),(0.073949,0.738534,-0.497215),(0.0876139,0.739516,-0.295207),(0.0739513,0.717949,-0.128407),(0.0876139,0.739516,-0.295207),(0.101279,0.740499,-0.0931984),(0.035654,0.717949,-0.497215),(-0.00264151,0.7163,-0.526765),(-0.00785601,0.724864,-0.700279),(-0.00303241,0.934544,0.0504887),(-0.0026253,0.898799,-0.0279467),(-0.00303347,0.900948,0.122661),(-0.00303241,0.934544,0.0504887),(-0.00303347,0.900948,0.122661),(-0.0030355,0.903097,0.277591),(0.09397,0.707736,-0.360507),(0.133681,0.7163,-0.508713),(0.163023,0.717949,-0.470258),(1.05985E-6,0.934617,0.0504887),(0.00713026,0.898872,-0.0262073),(0.0,0.901021,0.122661),(1.05985E-6,0.934617,0.0504887),(0.0,0.901021,0.122661),(-2.03005E-6,0.90317,0.277591),(1.06986E-6,1.0793,3.47475E-7),(1.05985E-6,0.934617,0.0504887),(-2.03005E-6,0.90317,0.277591),(0.121048,0.901562,-0.4615),(0.125042,0.85895,-0.476404),(0.102639,0.874364,-0.392808),(0.125157,0.948969,-0.476834),(0.121048,0.901562,-0.4615),(0.102639,0.874364,-0.392808),(0.157931,0.693025,-0.59916),(0.164308,0.652374,-0.622957),(0.148978,0.684348,-0.565746),(0.133648,0.716322,-0.508537),(0.157931,0.693025,-0.59916),(0.148978,0.684348,-0.565746),(0.162047,0.723428,-0.61452),(0.157931,0.693025,-0.59916),(0.133648,0.716322,-0.508537),(-0.0747307,0.739931,-0.12054),(-0.0292815,0.734624,-0.255078),(-0.0593739,0.693107,-0.189784),(-0.0292815,0.734624,-0.255078),(-0.0372502,0.690461,-0.254805),(-0.0593739,0.693107,-0.189784),(-0.0593739,0.693107,-0.189784),(-0.0372502,0.690461,-0.254805),(-0.0346938,0.648327,-0.293737),(-0.118464,0.756954,0.588829),(-0.134253,0.756954,0.279345),(-0.302092,0.706351,0.270908),(0.167667,0.724864,-0.672708),(0.12767,0.740499,-0.765313),(0.167669,0.65237,-0.939584),(0.12767,0.740499,-0.765313),(0.167667,0.724864,-0.672708),(0.0890589,0.717949,-0.490077),(0.12767,0.740499,-0.765313),(0.0890589,0.717949,-0.490077),(0.0520685,0.738534,-0.499987),(0.0520685,0.738534,-0.499987),(0.0890589,0.717949,-0.490077),(-0.0433882,0.717949,-0.143748),(0.0890589,0.717949,-0.490077),(0.093985,0.707736,-0.360503),(-0.0119795,0.70326,-0.0988804),(0.093985,0.707736,-0.360503),(0.00459094,0.699209,-0.0268877),(-0.0119795,0.70326,-0.0988804),(0.0890589,0.717949,-0.490077),(-0.0119794,0.70326,-0.0988803),(-0.0433882,0.717949,-0.143748),(-0.0433882,0.717949,-0.143748),(-0.0119795,0.70326,-0.0988804),(-0.0766211,0.707736,0.27759),(-0.113414,0.724864,0.27759),(-0.0433882,0.717949,-0.143748),(-0.0766211,0.707736,0.27759),(-0.0119795,0.70326,-0.0988804),(0.00459094,0.699209,-0.0268877),(-0.0352099,0.699209,0.27759),(0.00459094,0.699209,-0.0268877),(-0.0030355,0.697365,0.277591),(-0.0352099,0.699209,0.27759),(-0.0119795,0.70326,-0.0988804),(-0.0352099,0.699209,0.27759),(-0.0766211,0.707736,0.27759),(-0.00303552,0.697365,0.277591),(-0.00303664,0.699209,0.4528),(-0.0352099,0.699209,0.27759),(-0.131733,0.744141,0.27759),(-0.113414,0.724864,0.27759),(-0.123031,0.740499,0.495402),(-0.123031,0.740499,0.495402),(-0.113414,0.724864,0.27759),(-0.091476,0.717949,0.518391),(-0.113414,0.724864,0.27759),(-0.0766211,0.707736,0.27759),(-0.091476,0.717949,0.518391),(-0.00303936,0.724864,0.878668),(-0.047258,0.717949,0.759194),(-0.00303808,0.707736,0.678309),(-0.0766211,0.707736,0.27759),(-0.0430349,0.70326,0.495403),(-0.091476,0.717949,0.518391),(-0.0352099,0.699209,0.27759),(-0.0430349,0.70326,0.495403),(-0.0766211,0.707736,0.27759),(-0.0430349,0.70326,0.495403),(-0.047258,0.717949,0.759194),(-0.091476,0.717949,0.518391),(-0.00303808,0.707736,0.678309),(-0.047258,0.717949,0.759194),(-0.0430349,0.70326,0.495403),(-0.00303664,0.699209,0.4528),(-0.00303808,0.707736,0.678309),(-0.0430349,0.70326,0.495403),(-0.0352099,0.699209,0.27759),(-0.00303664,0.699209,0.4528),(-0.0430349,0.70326,0.495403),(-0.00303936,0.724864,0.878668),(-0.0430376,0.740499,0.931031),(-0.047258,0.717949,0.759194),(-0.0352143,0.759118,0.978427),(-0.076625,0.759118,0.878668),(-0.0430376,0.740499,0.931031),(-0.0914776,0.738534,0.759194),(-0.113416,0.759118,0.678308),(-0.123031,0.740499,0.495402),(-0.0914776,0.738534,0.759194),(-0.123031,0.740499,0.495402),(-0.091476,0.717949,0.518391),(-0.076625,0.759118,0.878668),(-0.113416,0.759118,0.678308),(-0.0914776,0.738534,0.759194),(-0.0430376,0.740499,0.931031),(-0.076625,0.759118,0.878668),(-0.0914776,0.738534,0.759194),(-0.0430376,0.740499,0.931031),(-0.0914776,0.738534,0.759194),(-0.047258,0.717949,0.759194),(-0.047258,0.717949,0.759194),(-0.0914776,0.738534,0.759194),(-0.091476,0.717949,0.518391),(-0.113416,0.759118,0.678308),(-0.131734,0.759118,0.452799),(-0.123031,0.740499,0.495402),(-0.131734,0.759118,0.452799),(-0.131733,0.744141,0.27759),(-0.123031,0.740499,0.495402),(-0.135694,0.759118,0.27759),(-0.131733,0.744141,0.27759),(-0.131734,0.759118,0.452799),(-0.0788967,0.740499,-0.116812),(0.0520685,0.738534,-0.499987),(-0.0134142,0.739516,-0.3084),(0.00165268,0.759118,-0.385245),(0.0520685,0.738534,-0.499987),(-0.0788967,0.740499,-0.116812),(0.00165268,0.759118,-0.385245),(-0.0788967,0.740499,-0.116812),(-0.118995,0.759118,-0.064479),(-0.0788967,0.740499,-0.116812),(-0.0433882,0.717949,-0.143748),(-0.113414,0.724864,0.27759),(-0.118995,0.759118,-0.064479),(-0.0788967,0.740499,-0.116812),(-0.131733,0.744141,0.27759),(-0.131733,0.744141,0.27759),(-0.0788967,0.740499,-0.116812),(-0.113414,0.724864,0.27759),(-0.135694,0.759118,0.27759),(-0.118995,0.759118,-0.064479),(-0.131733,0.744141,0.27759),(0.00165268,0.759118,-0.385245),(0.094081,0.759118,-0.672708),(0.0520685,0.738534,-0.499987),(0.094081,0.759118,-0.672708),(0.12767,0.740499,-0.765313),(0.0520685,0.738534,-0.499987),(0.135494,0.759118,-0.849131),(0.12767,0.740499,-0.765313),(0.094081,0.759118,-0.672708),(0.135494,0.759118,-0.849131),(0.167669,0.65237,-0.939584),(0.12767,0.740499,-0.765313),(0.135494,0.759118,-0.849131),(0.167668,0.759118,-0.887274),(0.167669,0.65237,-0.939584),(-0.00304013,0.759118,0.999996),(-0.0352143,0.759118,0.978427),(-0.00304,0.744141,0.978427),(-0.00304,0.744141,0.978427),(-0.0430376,0.740499,0.931031),(-0.00303936,0.724864,0.878668),(-0.00304,0.744141,0.978427),(-0.0352143,0.759118,0.978427),(-0.0430376,0.740499,0.931031),(-0.131733,0.794038,0.27759),(-0.118995,0.759118,-0.064479),(-0.135694,0.759118,0.27759),(-0.0788967,0.80253,-0.116812),(-0.118995,0.759118,-0.064479),(-0.131733,0.794038,0.27759),(-0.0788967,0.80253,-0.116812),(0.00165268,0.759118,-0.385245),(-0.118995,0.759118,-0.064479),(0.0520685,0.807112,-0.499987),(0.00165268,0.759118,-0.385245),(-0.0788967,0.80253,-0.116812),(0.0520685,0.807112,-0.499987),(0.094081,0.759118,-0.672708),(0.00165268,0.759118,-0.385245),(0.0520685,0.807112,-0.499987),(0.12767,0.80253,-0.765313),(0.094081,0.759118,-0.672708),(0.167669,1.14936,-1.07091),(0.12767,0.80253,-0.765313),(0.167667,0.838984,-0.672708),(0.12767,0.80253,-0.765313),(0.135494,0.759118,-0.849135),(0.094081,0.759118,-0.672708),(0.167669,1.14936,-1.07091),(0.135494,0.759118,-0.849135),(0.12767,0.80253,-0.765313),(0.167669,1.14936,-1.07091),(0.167668,0.759118,-0.887274),(0.135494,0.759118,-0.849135),(-0.113414,0.838984,0.27759),(-0.0788967,0.80253,-0.116812),(-0.131733,0.794038,0.27759),(-0.0433882,0.855105,-0.143748),(-0.0788967,0.80253,-0.116812),(-0.113414,0.838984,0.27759),(-0.0433882,0.855105,-0.143748),(0.0520685,0.807112,-0.499987),(-0.0788967,0.80253,-0.116812),(0.0890589,0.855105,-0.490077),(0.0520685,0.807112,-0.499987),(-0.0433882,0.855105,-0.143748),(0.0890589,0.855105,-0.490077),(0.12767,0.80253,-0.765313),(0.0520685,0.807112,-0.499987),(0.0890589,0.855105,-0.490077),(0.167667,0.838984,-0.672708),(0.12767,0.80253,-0.765313),(-0.076621,0.878917,0.27759),(-0.0433882,0.855105,-0.143748),(-0.113414,0.838984,0.27759),(-0.0119795,0.889352,-0.0988804),(-0.0433882,0.855105,-0.143748),(-0.076621,0.878917,0.27759),(-0.0119795,0.889352,-0.0988804),(0.0890589,0.855105,-0.490077),(-0.0433882,0.855105,-0.143748),(-0.0119795,0.889352,-0.0988804),(0.093985,0.878917,-0.360503),(0.0890589,0.855105,-0.490077),(0.00459094,0.898799,-0.0268877),(0.093985,0.878917,-0.360503),(-0.0119795,0.889352,-0.0988804),(-0.0352099,0.898799,0.27759),(-0.0119795,0.889352,-0.0988804),(-0.076621,0.878917,0.27759),(0.00459094,0.898799,-0.0268877),(-0.0119795,0.889352,-0.0988804),(-0.0352099,0.898799,0.27759),(0.00459094,0.898799,-0.0268877),(-0.0352099,0.898799,0.27759),(-0.0030355,0.903097,0.277591),(-0.00303999,0.794038,0.978427),(-0.0352143,0.759118,0.978427),(-0.00304013,0.759118,0.999996),(-0.00303999,0.794038,0.978427),(-0.0430376,0.80253,0.931031),(-0.0352143,0.759118,0.978427),(-0.0352143,0.759118,0.978427),(-0.0430376,0.80253,0.931031),(-0.076625,0.759118,0.878668),(-0.0430376,0.80253,0.931031),(-0.0914776,0.807112,0.759194),(-0.076625,0.759118,0.878668),(-0.076625,0.759118,0.878668),(-0.0914776,0.807112,0.759194),(-0.113416,0.759118,0.678308),(-0.0914776,0.807112,0.759194),(-0.123031,0.80253,0.495402),(-0.113416,0.759118,0.678308),(-0.123031,0.80253,0.495402),(-0.131734,0.759118,0.452799),(-0.113416,0.759118,0.678308),(-0.123031,0.80253,0.495402),(-0.131733,0.794038,0.27759),(-0.131734,0.759118,0.452799),(-0.131733,0.794038,0.27759),(-0.135694,0.759118,0.27759),(-0.131734,0.759118,0.452799),(-0.00303999,0.794038,0.978427),(-0.00303936,0.838984,0.878668),(-0.0430376,0.80253,0.931031),(-0.00303936,0.838984,0.878668),(-0.047258,0.855105,0.759194),(-0.0430376,0.80253,0.931031),(-0.0430376,0.80253,0.931031),(-0.047258,0.855105,0.759194),(-0.0914776,0.807112,0.759194),(-0.047258,0.855105,0.759194),(-0.091476,0.855105,0.518391),(-0.0914776,0.807112,0.759194),(-0.091476,0.855105,0.518391),(-0.123031,0.80253,0.495402),(-0.0914776,0.807112,0.759194),(-0.091476,0.855105,0.518391),(-0.113414,0.838984,0.27759),(-0.123031,0.80253,0.495402),(-0.113414,0.838984,0.27759),(-0.131733,0.794038,0.27759),(-0.123031,0.80253,0.495402),(-0.00303808,0.878917,0.678309),(-0.047258,0.855105,0.759194),(-0.00303936,0.838984,0.878668),(-0.00303808,0.878917,0.678309),(-0.0430349,0.889352,0.495403),(-0.047258,0.855105,0.759194),(-0.0430349,0.889352,0.495403),(-0.091476,0.855105,0.518391),(-0.047258,0.855105,0.759194),(-0.0430349,0.889352,0.495403),(-0.076621,0.878917,0.27759),(-0.091476,0.855105,0.518391),(-0.076621,0.878917,0.27759),(-0.113414,0.838984,0.27759),(-0.091476,0.855105,0.518391),(-0.00303808,0.878917,0.678309),(-0.00303664,0.898799,0.4528),(-0.0430349,0.889352,0.495403),(-0.00303664,0.898799,0.4528),(-0.0352099,0.898799,0.27759),(-0.0430349,0.889352,0.495403),(-0.0352099,0.898799,0.27759),(-0.076621,0.878917,0.27759),(-0.0430349,0.889352,0.495403),(-0.00303551,0.903097,0.277591),(-0.0352099,0.898799,0.27759),(-0.00303664,0.898799,0.4528),(0.133698,0.85895,-0.508708),(0.167667,0.838984,-0.672708),(0.0890589,0.855105,-0.490077),(0.093985,0.878917,-0.360503),(0.133698,0.85895,-0.508708),(0.0890589,0.855105,-0.490077),(-0.0134142,0.739516,-0.3084),(0.0520685,0.738534,-0.499987),(-0.0433882,0.717949,-0.143748),(-0.0788967,0.740499,-0.116812),(-0.0134142,0.739516,-0.3084),(-0.0433882,0.717949,-0.143748),(0.167667,0.724864,-0.672708),(0.133698,0.7163,-0.508708),(0.0890589,0.717949,-0.490077),(0.0890589,0.717949,-0.490077),(0.133698,0.7163,-0.508708),(0.093985,0.707736,-0.360503),(0.141424,0.693107,-0.13598),(0.14801,0.734624,-0.207573),(0.120101,0.739931,-0.0683354),(0.141424,0.693107,-0.13598),(0.154775,0.690461,-0.203352),(0.14801,0.734624,-0.207573),(0.172027,0.648327,-0.238347),(0.154775,0.690461,-0.203352),(0.141424,0.693107,-0.13598),(0.296012,0.706351,0.270908),(0.128173,0.756954,0.279345),(0.112384,0.756954,0.588829),(0.167643,0.65237,-0.939584),(0.207642,0.740499,-0.765313),(0.167644,0.724864,-0.672708),(0.163023,0.717949,-0.470258),(0.167644,0.724864,-0.672708),(0.207642,0.740499,-0.765313),(0.200013,0.738534,-0.460347),(0.163023,0.717949,-0.470258),(0.207642,0.740499,-0.765313),(0.104561,0.717949,-0.104105),(0.163023,0.717949,-0.470258),(0.200013,0.738534,-0.460347),(0.0549268,0.70326,-0.0809529),(0.09397,0.707736,-0.360507),(0.163023,0.717949,-0.470258),(0.0549268,0.70326,-0.0809529),(0.00458012,0.699209,-0.0268906),(0.09397,0.707736,-0.360507),(0.104561,0.717949,-0.104105),(0.0549268,0.70326,-0.0809529),(0.163023,0.717949,-0.470258),(0.0705411,0.707736,0.27759),(0.0549268,0.70326,-0.0809529),(0.104561,0.717949,-0.104105),(0.0705411,0.707736,0.27759),(0.104561,0.717949,-0.104105),(0.107334,0.724864,0.27759),(0.0291299,0.699209,0.27759),(0.00458012,0.699209,-0.0268906),(0.0549268,0.70326,-0.0809529),(0.0291299,0.699209,0.27759),(-0.00304449,0.697365,0.277591),(0.00458012,0.699209,-0.0268906),(0.0705411,0.707736,0.27759),(0.0291299,0.699209,0.27759),(0.0549268,0.70326,-0.0809529),(0.0291299,0.699209,0.27759),(-0.00304335,0.699209,0.4528),(-0.00304447,0.697365,0.277591),(0.116951,0.740499,0.495402),(0.107334,0.724864,0.27759),(0.125653,0.744141,0.27759),(0.085396,0.717949,0.518391),(0.107334,0.724864,0.27759),(0.116951,0.740499,0.495402),(0.085396,0.717949,0.518391),(0.0705411,0.707736,0.27759),(0.107334,0.724864,0.27759),(-0.00304191,0.707736,0.678309),(0.041178,0.717949,0.759194),(-0.00304063,0.724864,0.878668),(0.085396,0.717949,0.518391),(0.0369549,0.70326,0.495403),(0.0705411,0.707736,0.27759),(0.0705411,0.707736,0.27759),(0.0369549,0.70326,0.495403),(0.0291299,0.699209,0.27759),(0.085396,0.717949,0.518391),(0.041178,0.717949,0.759194),(0.0369549,0.70326,0.495403),(0.0369549,0.70326,0.495403),(0.041178,0.717949,0.759194),(-0.00304191,0.707736,0.678309),(0.0369549,0.70326,0.495403),(-0.00304191,0.707736,0.678309),(-0.00304335,0.699209,0.4528),(0.0369549,0.70326,0.495403),(-0.00304335,0.699209,0.4528),(0.0291299,0.699209,0.27759),(0.041178,0.717949,0.759194),(0.0369577,0.740499,0.931031),(-0.00304063,0.724864,0.878668),(0.0369577,0.740499,0.931031),(0.070545,0.759118,0.878668),(0.0291344,0.759118,0.978427),(0.116951,0.740499,0.495402),(0.107336,0.759118,0.678308),(0.0853976,0.738534,0.759194),(0.085396,0.717949,0.518391),(0.116951,0.740499,0.495402),(0.0853976,0.738534,0.759194),(0.0853976,0.738534,0.759194),(0.107336,0.759118,0.678308),(0.070545,0.759118,0.878668),(0.0853976,0.738534,0.759194),(0.070545,0.759118,0.878668),(0.0369577,0.740499,0.931031),(0.041178,0.717949,0.759194),(0.0853976,0.738534,0.759194),(0.0369577,0.740499,0.931031),(0.085396,0.717949,0.518391),(0.0853976,0.738534,0.759194),(0.041178,0.717949,0.759194),(0.116951,0.740499,0.495402),(0.125654,0.759118,0.452799),(0.107336,0.759118,0.678308),(0.116951,0.740499,0.495402),(0.125653,0.744141,0.27759),(0.125654,0.759118,0.452799),(0.125654,0.759118,0.452799),(0.125653,0.744141,0.27759),(0.129614,0.759118,0.27759),(0.160929,0.739516,-0.261685),(0.200013,0.738534,-0.460347),(0.121845,0.740499,-0.0630231),(0.121845,0.740499,-0.0630231),(0.200013,0.738534,-0.460347),(0.186303,0.759118,-0.335768),(0.112238,0.759118,0.00195557),(0.121845,0.740499,-0.0630231),(0.186303,0.759118,-0.335768),(0.107334,0.724864,0.27759),(0.104561,0.717949,-0.104105),(0.121845,0.740499,-0.0630231),(0.125653,0.744141,0.27759),(0.121845,0.740499,-0.0630231),(0.112238,0.759118,0.00195557),(0.107334,0.724864,0.27759),(0.121845,0.740499,-0.0630231),(0.125653,0.744141,0.27759),(0.125653,0.744141,0.27759),(0.112238,0.759118,0.00195557),(0.129614,0.759118,0.27759),(0.200013,0.738534,-0.460347),(0.24123,0.759118,-0.672709),(0.186303,0.759118,-0.335768),(0.200013,0.738534,-0.460347),(0.207642,0.740499,-0.765313),(0.24123,0.759118,-0.672709),(0.24123,0.759118,-0.672709),(0.207642,0.740499,-0.765313),(0.199818,0.759118,-0.849132),(0.207642,0.740499,-0.765313),(0.167643,0.65237,-0.939584),(0.199818,0.759118,-0.849132),(0.167643,0.65237,-0.939584),(0.167643,0.759118,-0.887274),(0.199818,0.759118,-0.849132),(-0.00303999,0.744141,0.978427),(0.0291344,0.759118,0.978427),(-0.00303985,0.759118,0.999996),(-0.00304063,0.724864,0.878668),(0.0369577,0.740499,0.931031),(-0.00303999,0.744141,0.978427),(0.0369577,0.740499,0.931031),(0.0291344,0.759118,0.978427),(-0.00303999,0.744141,0.978427),(0.129614,0.759118,0.27759),(0.112238,0.759118,0.00195557),(0.125653,0.794038,0.27759),(0.125653,0.794038,0.27759),(0.112238,0.759118,0.00195557),(0.121845,0.80253,-0.0630231),(0.112238,0.759118,0.00195557),(0.186303,0.759118,-0.335768),(0.121845,0.80253,-0.0630231),(0.121845,0.80253,-0.0630231),(0.186303,0.759118,-0.335768),(0.200013,0.807112,-0.460347),(0.186303,0.759118,-0.335768),(0.24123,0.759118,-0.672709),(0.200013,0.807112,-0.460347),(0.24123,0.759118,-0.672709),(0.207642,0.80253,-0.765313),(0.200013,0.807112,-0.460347),(0.167644,0.838984,-0.672708),(0.207642,0.80253,-0.765313),(0.167642,1.14936,-1.07092),(0.24123,0.759118,-0.672709),(0.199818,0.759118,-0.849135),(0.207642,0.80253,-0.765313),(0.207642,0.80253,-0.765313),(0.199818,0.759118,-0.849135),(0.167642,1.14936,-1.07092),(0.199818,0.759118,-0.849135),(0.167643,0.759118,-0.887274),(0.167642,1.14936,-1.07092),(0.125653,0.794038,0.27759),(0.121845,0.80253,-0.0630231),(0.107334,0.838984,0.27759),(0.107334,0.838984,0.27759),(0.121845,0.80253,-0.0630231),(0.104561,0.855105,-0.104105),(0.121845,0.80253,-0.0630231),(0.200013,0.807112,-0.460347),(0.104561,0.855105,-0.104105),(0.104561,0.855105,-0.104105),(0.200013,0.807112,-0.460347),(0.163023,0.855105,-0.470258),(0.200013,0.807112,-0.460347),(0.207642,0.80253,-0.765313),(0.163023,0.855105,-0.470258),(0.207642,0.80253,-0.765313),(0.167644,0.838984,-0.672708),(0.163023,0.855105,-0.470258),(0.107334,0.838984,0.27759),(0.104561,0.855105,-0.104105),(0.070541,0.878917,0.27759),(0.070541,0.878917,0.27759),(0.104561,0.855105,-0.104105),(0.0549268,0.889352,-0.0809529),(0.104561,0.855105,-0.104105),(0.163023,0.855105,-0.470258),(0.0549268,0.889352,-0.0809529),(0.163023,0.855105,-0.470258),(0.09397,0.878917,-0.360507),(0.0549268,0.889352,-0.0809529),(0.0549268,0.889352,-0.0809529),(0.09397,0.878917,-0.360507),(0.00458012,0.898799,-0.0268906),(0.070541,0.878917,0.27759),(0.0549268,0.889352,-0.0809529),(0.0291299,0.898799,0.27759),(0.0291299,0.898799,0.27759),(0.0549268,0.889352,-0.0809529),(0.00458012,0.898799,-0.0268906),(-0.00304449,0.903097,0.277591),(0.0291299,0.898799,0.27759),(0.00458012,0.898799,-0.0268906),(-0.00303986,0.759118,0.999996),(0.0291344,0.759118,0.978427),(-0.00303999,0.794038,0.978427),(0.0291344,0.759118,0.978427),(0.0369577,0.80253,0.931031),(-0.00303999,0.794038,0.978427),(0.070545,0.759118,0.878668),(0.0369577,0.80253,0.931031),(0.0291344,0.759118,0.978427),(0.070545,0.759118,0.878668),(0.0853976,0.807112,0.759194),(0.0369577,0.80253,0.931031),(0.107336,0.759118,0.678308),(0.0853976,0.807112,0.759194),(0.070545,0.759118,0.878668),(0.107336,0.759118,0.678308),(0.116951,0.80253,0.495402),(0.0853976,0.807112,0.759194),(0.107336,0.759118,0.678308),(0.125654,0.759118,0.452799),(0.116951,0.80253,0.495402),(0.125654,0.759118,0.452799),(0.125653,0.794038,0.27759),(0.116951,0.80253,0.495402),(0.125654,0.759118,0.452799),(0.129614,0.759118,0.27759),(0.125653,0.794038,0.27759),(0.0369577,0.80253,0.931031),(-0.00304063,0.838984,0.878668),(-0.00303999,0.794038,0.978427),(0.0369577,0.80253,0.931031),(0.041178,0.855105,0.759194),(-0.00304063,0.838984,0.878668),(0.0853976,0.807112,0.759194),(0.041178,0.855105,0.759194),(0.0369577,0.80253,0.931031),(0.0853976,0.807112,0.759194),(0.085396,0.855105,0.518391),(0.041178,0.855105,0.759194),(0.0853976,0.807112,0.759194),(0.116951,0.80253,0.495402),(0.085396,0.855105,0.518391),(0.116951,0.80253,0.495402),(0.107334,0.838984,0.27759),(0.085396,0.855105,0.518391),(0.116951,0.80253,0.495402),(0.125653,0.794038,0.27759),(0.107334,0.838984,0.27759),(-0.00304063,0.838984,0.878668),(0.041178,0.855105,0.759194),(-0.00304191,0.878917,0.678309),(0.041178,0.855105,0.759194),(0.0369549,0.889352,0.495403),(-0.00304191,0.878917,0.678309),(0.041178,0.855105,0.759194),(0.085396,0.855105,0.518391),(0.0369549,0.889352,0.495403),(0.085396,0.855105,0.518391),(0.070541,0.878917,0.27759),(0.0369549,0.889352,0.495403),(0.085396,0.855105,0.518391),(0.107334,0.838984,0.27759),(0.070541,0.878917,0.27759),(0.0369549,0.889352,0.495403),(-0.00304335,0.898799,0.4528),(-0.00304191,0.878917,0.678309),(0.0369549,0.889352,0.495403),(0.0291299,0.898799,0.27759),(-0.00304335,0.898799,0.4528),(0.0369549,0.889352,0.495403),(0.070541,0.878917,0.27759),(0.0291299,0.898799,0.27759),(-0.00304335,0.898799,0.4528),(0.0291299,0.898799,0.27759),(-0.00304448,0.903097,0.277591),(0.163023,0.855105,-0.470258),(0.167644,0.838984,-0.672708),(0.133681,0.85895,-0.508713),(0.163023,0.855105,-0.470258),(0.133681,0.85895,-0.508713),(0.09397,0.878917,-0.360507),(0.104561,0.717949,-0.104105),(0.200013,0.738534,-0.460347),(0.160929,0.739516,-0.261685),(0.104561,0.717949,-0.104105),(0.160929,0.739516,-0.261685),(0.121845,0.740499,-0.0630231),(0.163023,0.717949,-0.470258),(0.133681,0.7163,-0.508713),(0.167644,0.724864,-0.672708),(-0.00303241,0.934544,0.0504887),(0.00459268,0.898799,-0.0268872),(-0.00303347,0.900948,0.122661),(-0.00303241,0.934544,0.0504887),(-0.00303347,0.900948,0.122661),(-0.0030355,0.903097,0.277591),(0.0698174,0.707736,-0.363713),(0.0996002,0.7163,-0.513226),(0.131181,0.717949,-0.476997),(1.05985E-6,0.934617,0.0504887),(0.00534815,0.898872,-0.0266421),(0.0,0.901021,0.122661),(1.05985E-6,0.934617,0.0504887),(0.0,0.901021,0.122661),(-2.03005E-6,0.90317,0.277591),(1.06986E-6,1.0793,3.47475E-7),(1.05985E-6,0.934617,0.0504887),(-2.03005E-6,0.90317,0.277591),(0.0901301,0.901562,-0.465598),(0.0931252,0.85895,-0.480633),(0.0763225,0.874364,-0.3963),(0.0932116,0.948969,-0.481067),(0.0901301,0.901562,-0.465598),(0.0763225,0.874364,-0.3963),(0.117792,0.693025,-0.604472),(0.122574,0.652374,-0.628478),(0.111077,0.684348,-0.570763),(0.0995793,0.716322,-0.51305),(0.117792,0.693025,-0.604472),(0.111077,0.684348,-0.570763),(0.120878,0.723428,-0.619967),(0.117792,0.693025,-0.604472),(0.0995793,0.716322,-0.51305),(-0.0819193,0.739931,-0.114875),(-0.0455625,0.734624,-0.251207),(-0.0711737,0.693107,-0.184522),(-0.0455625,0.734624,-0.251207),(-0.0534457,0.690461,-0.250421),(-0.0711737,0.693107,-0.184522),(-0.0711737,0.693107,-0.184522),(-0.0534457,0.690461,-0.250421),(-0.0534301,0.648327,-0.289187),(-0.118464,0.756954,0.588829),(-0.134253,0.756954,0.279345),(-0.302092,0.706351,0.270908),(0.116892,0.724864,-0.680852),(0.134939,0.740499,-0.779441),(0.264149,0.65237,-0.90137),(0.134939,0.740499,-0.779441),(0.116892,0.724864,-0.680852),(0.0565643,0.717949,-0.491861),(0.134939,0.740499,-0.779441),(0.0565643,0.717949,-0.491861),(0.0192478,0.738534,-0.499294),(0.0192478,0.738534,-0.499294),(0.0565643,0.717949,-0.491861),(-0.0523454,0.717949,-0.139913),(0.0565643,0.717949,-0.491861),(0.0698325,0.707736,-0.36371),(-0.0183011,0.70326,-0.0974596),(0.0698325,0.707736,-0.36371),(0.00278643,0.699209,-0.0271525),(-0.0183011,0.70326,-0.0974596),(0.0565643,0.717949,-0.491861),(-0.018301,0.70326,-0.0974596),(-0.0523454,0.717949,-0.139913),(-0.0523454,0.717949,-0.139913),(-0.0183011,0.70326,-0.0974596),(-0.0766211,0.707736,0.27759),(-0.113414,0.724864,0.27759),(-0.0523454,0.717949,-0.139913),(-0.0766211,0.707736,0.27759),(-0.0183011,0.70326,-0.0974596),(0.00278643,0.699209,-0.0271525),(-0.0352099,0.699209,0.27759),(0.00278643,0.699209,-0.0271525),(-0.0030355,0.697365,0.277591),(-0.0352099,0.699209,0.27759),(-0.0183011,0.70326,-0.0974596),(-0.0352099,0.699209,0.27759),(-0.0766211,0.707736,0.27759),(-0.00303552,0.697365,0.277591),(-0.00303664,0.699209,0.4528),(-0.0352099,0.699209,0.27759),(-0.131733,0.744141,0.27759),(-0.113414,0.724864,0.27759),(-0.123031,0.740499,0.495402),(-0.123031,0.740499,0.495402),(-0.113414,0.724864,0.27759),(-0.091476,0.717949,0.518391),(-0.113414,0.724864,0.27759),(-0.0766211,0.707736,0.27759),(-0.091476,0.717949,0.518391),(-0.00303936,0.724864,0.878668),(-0.047258,0.717949,0.759194),(-0.00303808,0.707736,0.678309),(-0.0766211,0.707736,0.27759),(-0.0430349,0.70326,0.495403),(-0.091476,0.717949,0.518391),(-0.0352099,0.699209,0.27759),(-0.0430349,0.70326,0.495403),(-0.0766211,0.707736,0.27759),(-0.0430349,0.70326,0.495403),(-0.047258,0.717949,0.759194),(-0.091476,0.717949,0.518391),(-0.00303808,0.707736,0.678309),(-0.047258,0.717949,0.759194),(-0.0430349,0.70326,0.495403),(-0.00303664,0.699209,0.4528),(-0.00303808,0.707736,0.678309),(-0.0430349,0.70326,0.495403),(-0.0352099,0.699209,0.27759),(-0.00303664,0.699209,0.4528),(-0.0430349,0.70326,0.495403),(-0.00303936,0.724864,0.878668),(-0.0430376,0.740499,0.931031),(-0.047258,0.717949,0.759194),(-0.0352143,0.759118,0.978427),(-0.076625,0.759118,0.878668),(-0.0430376,0.740499,0.931031),(-0.0914776,0.738534,0.759194),(-0.113416,0.759118,0.678308),(-0.123031,0.740499,0.495402),(-0.0914776,0.738534,0.759194),(-0.123031,0.740499,0.495402),(-0.091476,0.717949,0.518391),(-0.076625,0.759118,0.878668),(-0.113416,0.759118,0.678308),(-0.0914776,0.738534,0.759194),(-0.0430376,0.740499,0.931031),(-0.076625,0.759118,0.878668),(-0.0914776,0.738534,0.759194),(-0.0430376,0.740499,0.931031),(-0.0914776,0.738534,0.759194),(-0.047258,0.717949,0.759194),(-0.047258,0.717949,0.759194),(-0.0914776,0.738534,0.759194),(-0.091476,0.717949,0.518391),(-0.113416,0.759118,0.678308),(-0.131734,0.759118,0.452799),(-0.123031,0.740499,0.495402),(-0.131734,0.759118,0.452799),(-0.131733,0.744141,0.27759),(-0.123031,0.740499,0.495402),(-0.135694,0.759118,0.27759),(-0.131733,0.744141,0.27759),(-0.131734,0.759118,0.452799),(-0.0858085,0.740499,-0.110908),(0.0192478,0.738534,-0.499294),(-0.0332806,0.739516,-0.305102),(-0.0233142,0.759118,-0.382267),(0.0192478,0.738534,-0.499294),(-0.0858085,0.740499,-0.110908),(-0.0233142,0.759118,-0.382267),(-0.0858085,0.740499,-0.110908),(-0.122179,0.759118,-0.056427),(-0.0858085,0.740499,-0.110908),(-0.0523454,0.717949,-0.139913),(-0.113414,0.724864,0.27759),(-0.122179,0.759118,-0.056427),(-0.0858085,0.740499,-0.110908),(-0.131733,0.744141,0.27759),(-0.131733,0.744141,0.27759),(-0.0858085,0.740499,-0.110908),(-0.113414,0.724864,0.27759),(-0.135694,0.759118,0.27759),(-0.122179,0.759118,-0.056427),(-0.131733,0.744141,0.27759),(-0.0233142,0.759118,-0.382267),(0.0560878,0.759118,-0.721455),(0.0192478,0.738534,-0.499294),(0.0560878,0.759118,-0.721455),(0.134939,0.740499,-0.779441),(0.0192478,0.738534,-0.499294),(0.187653,0.759118,-0.844382),(0.134939,0.740499,-0.779441),(0.0560878,0.759118,-0.721455),(0.187653,0.759118,-0.844382),(0.264149,0.65237,-0.90137),(0.134939,0.740499,-0.779441),(0.187653,0.759118,-0.844382),(0.235285,0.759118,-0.858146),(0.264149,0.65237,-0.90137),(-0.00304013,0.759118,0.999996),(-0.0352143,0.759118,0.978427),(-0.00304,0.744141,0.978427),(-0.00304,0.744141,0.978427),(-0.0430376,0.740499,0.931031),(-0.00303936,0.724864,0.878668),(-0.00304,0.744141,0.978427),(-0.0352143,0.759118,0.978427),(-0.0430376,0.740499,0.931031),(-0.131733,0.794038,0.27759),(-0.122179,0.759118,-0.056427),(-0.135694,0.759118,0.27759),(-0.0858085,0.80253,-0.110908),(-0.122179,0.759118,-0.056427),(-0.131733,0.794038,0.27759),(-0.0858085,0.80253,-0.110908),(-0.0233142,0.759118,-0.382267),(-0.122179,0.759118,-0.056427),(0.0192478,0.807112,-0.499294),(-0.0233142,0.759118,-0.382267),(-0.0858085,0.80253,-0.110908),(0.0192478,0.807112,-0.499294),(0.0560878,0.759118,-0.721455),(-0.0233142,0.759118,-0.382267),(0.0192478,0.807112,-0.499294),(0.134939,0.80253,-0.779441),(0.0560878,0.759118,-0.721455),(0.336611,1.14936,-1.00988),(0.134939,0.80253,-0.779441),(0.116892,0.838984,-0.680852),(0.134939,0.80253,-0.779441),(0.187655,0.759118,-0.844385),(0.0560878,0.759118,-0.721455),(0.336611,1.14936,-1.00988),(0.187655,0.759118,-0.844385),(0.134939,0.80253,-0.779441),(0.336611,1.14936,-1.00988),(0.235285,0.759118,-0.858146),(0.187655,0.759118,-0.844385),(-0.113414,0.838984,0.27759),(-0.0858085,0.80253,-0.110908),(-0.131733,0.794038,0.27759),(-0.0523454,0.855105,-0.139913),(-0.0858085,0.80253,-0.110908),(-0.113414,0.838984,0.27759),(-0.0523454,0.855105,-0.139913),(0.0192478,0.807112,-0.499294),(-0.0858085,0.80253,-0.110908),(0.0565643,0.855105,-0.491861),(0.0192478,0.807112,-0.499294),(-0.0523454,0.855105,-0.139913),(0.0565643,0.855105,-0.491861),(0.134939,0.80253,-0.779441),(0.0192478,0.807112,-0.499294),(0.0565643,0.855105,-0.491861),(0.116892,0.838984,-0.680852),(0.134939,0.80253,-0.779441),(-0.076621,0.878917,0.27759),(-0.0523454,0.855105,-0.139913),(-0.113414,0.838984,0.27759),(-0.0183011,0.889352,-0.0974596),(-0.0523454,0.855105,-0.139913),(-0.076621,0.878917,0.27759),(-0.0183011,0.889352,-0.0974596),(0.0565643,0.855105,-0.491861),(-0.0523454,0.855105,-0.139913),(-0.0183011,0.889352,-0.0974596),(0.0698325,0.878917,-0.36371),(0.0565643,0.855105,-0.491861),(0.00278643,0.898799,-0.0271525),(0.0698325,0.878917,-0.36371),(-0.0183011,0.889352,-0.0974596),(-0.0352099,0.898799,0.27759),(-0.0183011,0.889352,-0.0974596),(-0.076621,0.878917,0.27759),(0.00278643,0.898799,-0.0271525),(-0.0183011,0.889352,-0.0974596),(-0.0352099,0.898799,0.27759),(0.00278643,0.898799,-0.0271525),(-0.0352099,0.898799,0.27759),(-0.0030355,0.903097,0.277591),(-0.00303999,0.794038,0.978427),(-0.0352143,0.759118,0.978427),(-0.00304013,0.759118,0.999996),(-0.00303999,0.794038,0.978427),(-0.0430376,0.80253,0.931031),(-0.0352143,0.759118,0.978427),(-0.0352143,0.759118,0.978427),(-0.0430376,0.80253,0.931031),(-0.076625,0.759118,0.878668),(-0.0430376,0.80253,0.931031),(-0.0914776,0.807112,0.759194),(-0.076625,0.759118,0.878668),(-0.076625,0.759118,0.878668),(-0.0914776,0.807112,0.759194),(-0.113416,0.759118,0.678308),(-0.0914776,0.807112,0.759194),(-0.123031,0.80253,0.495402),(-0.113416,0.759118,0.678308),(-0.123031,0.80253,0.495402),(-0.131734,0.759118,0.452799),(-0.113416,0.759118,0.678308),(-0.123031,0.80253,0.495402),(-0.131733,0.794038,0.27759),(-0.131734,0.759118,0.452799),(-0.131733,0.794038,0.27759),(-0.135694,0.759118,0.27759),(-0.131734,0.759118,0.452799),(-0.00303999,0.794038,0.978427),(-0.00303936,0.838984,0.878668),(-0.0430376,0.80253,0.931031),(-0.00303936,0.838984,0.878668),(-0.047258,0.855105,0.759194),(-0.0430376,0.80253,0.931031),(-0.0430376,0.80253,0.931031),(-0.047258,0.855105,0.759194),(-0.0914776,0.807112,0.759194),(-0.047258,0.855105,0.759194),(-0.091476,0.855105,0.518391),(-0.0914776,0.807112,0.759194),(-0.091476,0.855105,0.518391),(-0.123031,0.80253,0.495402),(-0.0914776,0.807112,0.759194),(-0.091476,0.855105,0.518391),(-0.113414,0.838984,0.27759),(-0.123031,0.80253,0.495402),(-0.113414,0.838984,0.27759),(-0.131733,0.794038,0.27759),(-0.123031,0.80253,0.495402),(-0.00303808,0.878917,0.678309),(-0.047258,0.855105,0.759194),(-0.00303936,0.838984,0.878668),(-0.00303808,0.878917,0.678309),(-0.0430349,0.889352,0.495403),(-0.047258,0.855105,0.759194),(-0.0430349,0.889352,0.495403),(-0.091476,0.855105,0.518391),(-0.047258,0.855105,0.759194),(-0.0430349,0.889352,0.495403),(-0.076621,0.878917,0.27759),(-0.091476,0.855105,0.518391),(-0.076621,0.878917,0.27759),(-0.113414,0.838984,0.27759),(-0.091476,0.855105,0.518391),(-0.00303808,0.878917,0.678309),(-0.00303664,0.898799,0.4528),(-0.0430349,0.889352,0.495403),(-0.00303664,0.898799,0.4528),(-0.0352099,0.898799,0.27759),(-0.0430349,0.889352,0.495403),(-0.0352099,0.898799,0.27759),(-0.076621,0.878917,0.27759),(-0.0430349,0.889352,0.495403),(-0.00303551,0.903097,0.277591),(-0.0352099,0.898799,0.27759),(-0.00303664,0.898799,0.4528),(0.0996172,0.85895,-0.513223),(0.116892,0.838984,-0.680852),(0.0565643,0.855105,-0.491861),(0.0698325,0.878917,-0.36371),(0.0996172,0.85895,-0.513223),(0.0565643,0.855105,-0.491861),(-0.0332806,0.739516,-0.305102),(0.0192478,0.738534,-0.499294),(-0.0523454,0.717949,-0.139913),(-0.0858085,0.740499,-0.110908),(-0.0332806,0.739516,-0.305102),(-0.0523454,0.717949,-0.139913),(0.116892,0.724864,-0.680852),(0.0996172,0.7163,-0.513223),(0.0565643,0.717949,-0.491861),(0.0565643,0.717949,-0.491861),(0.0996172,0.7163,-0.513223),(0.0698325,0.707736,-0.36371),(0.131395,0.693107,-0.144169),(0.133292,0.734624,-0.215579),(0.11463,0.739931,-0.0757211),(0.131395,0.693107,-0.144169),(0.140273,0.690461,-0.211832),(0.133292,0.734624,-0.215579),(0.155114,0.648327,-0.247644),(0.140273,0.690461,-0.211832),(0.131395,0.693107,-0.144169),(0.296012,0.706351,0.270908),(0.128173,0.756954,0.279345),(0.112384,0.756954,0.588829),(0.264127,0.65237,-0.901384),(0.20102,0.740499,-0.735314),(0.116873,0.724864,-0.680865),(0.131181,0.717949,-0.476997),(0.116873,0.724864,-0.680865),(0.20102,0.740499,-0.735314),(0.168497,0.738534,-0.469564),(0.131181,0.717949,-0.476997),(0.20102,0.740499,-0.735314),(0.0969086,0.717949,-0.11018),(0.131181,0.717949,-0.476997),(0.168497,0.738534,-0.469564),(0.0491952,0.70326,-0.084014),(0.0698174,0.707736,-0.363713),(0.131181,0.717949,-0.476997),(0.0491952,0.70326,-0.084014),(0.00277551,0.699209,-0.0271546),(0.0698174,0.707736,-0.363713),(0.0969086,0.717949,-0.11018),(0.0491952,0.70326,-0.084014),(0.131181,0.717949,-0.476997),(0.0705411,0.707736,0.27759),(0.0491952,0.70326,-0.084014),(0.0969086,0.717949,-0.11018),(0.0705411,0.707736,0.27759),(0.0969086,0.717949,-0.11018),(0.107334,0.724864,0.27759),(0.0291299,0.699209,0.27759),(0.00277551,0.699209,-0.0271546),(0.0491952,0.70326,-0.084014),(0.0291299,0.699209,0.27759),(-0.00304449,0.697365,0.277591),(0.00277551,0.699209,-0.0271546),(0.0705411,0.707736,0.27759),(0.0291299,0.699209,0.27759),(0.0491952,0.70326,-0.084014),(0.0291299,0.699209,0.27759),(-0.00304335,0.699209,0.4528),(-0.00304447,0.697365,0.277591),(0.116951,0.740499,0.495402),(0.107334,0.724864,0.27759),(0.125653,0.744141,0.27759),(0.085396,0.717949,0.518391),(0.107334,0.724864,0.27759),(0.116951,0.740499,0.495402),(0.085396,0.717949,0.518391),(0.0705411,0.707736,0.27759),(0.107334,0.724864,0.27759),(-0.00304191,0.707736,0.678309),(0.041178,0.717949,0.759194),(-0.00304063,0.724864,0.878668),(0.085396,0.717949,0.518391),(0.0369549,0.70326,0.495403),(0.0705411,0.707736,0.27759),(0.0705411,0.707736,0.27759),(0.0369549,0.70326,0.495403),(0.0291299,0.699209,0.27759),(0.085396,0.717949,0.518391),(0.041178,0.717949,0.759194),(0.0369549,0.70326,0.495403),(0.0369549,0.70326,0.495403),(0.041178,0.717949,0.759194),(-0.00304191,0.707736,0.678309),(0.0369549,0.70326,0.495403),(-0.00304191,0.707736,0.678309),(-0.00304335,0.699209,0.4528),(0.0369549,0.70326,0.495403),(-0.00304335,0.699209,0.4528),(0.0291299,0.699209,0.27759),(0.041178,0.717949,0.759194),(0.0369577,0.740499,0.931031),(-0.00304063,0.724864,0.878668),(0.0369577,0.740499,0.931031),(0.070545,0.759118,0.878668),(0.0291344,0.759118,0.978427),(0.116951,0.740499,0.495402),(0.107336,0.759118,0.678308),(0.0853976,0.738534,0.759194),(0.085396,0.717949,0.518391),(0.116951,0.740499,0.495402),(0.0853976,0.738534,0.759194),(0.0853976,0.738534,0.759194),(0.107336,0.759118,0.678308),(0.070545,0.759118,0.878668),(0.0853976,0.738534,0.759194),(0.070545,0.759118,0.878668),(0.0369577,0.740499,0.931031),(0.041178,0.717949,0.759194),(0.0853976,0.738534,0.759194),(0.0369577,0.740499,0.931031),(0.085396,0.717949,0.518391),(0.0853976,0.738534,0.759194),(0.041178,0.717949,0.759194),(0.116951,0.740499,0.495402),(0.125654,0.759118,0.452799),(0.107336,0.759118,0.678308),(0.116951,0.740499,0.495402),(0.125653,0.744141,0.27759),(0.125654,0.759118,0.452799),(0.125654,0.759118,0.452799),(0.125653,0.744141,0.27759),(0.129614,0.759118,0.27759),(0.1426,0.739516,-0.270066),(0.168497,0.738534,-0.469564),(0.116703,0.740499,-0.070567),(0.116703,0.740499,-0.070567),(0.168497,0.738534,-0.469564),(0.162965,0.759118,-0.345159),(0.111383,0.759118,-0.00552014),(0.116703,0.740499,-0.070567),(0.162965,0.759118,-0.345159),(0.107334,0.724864,0.27759),(0.0969086,0.717949,-0.11018),(0.116703,0.740499,-0.070567),(0.125653,0.744141,0.27759),(0.116703,0.740499,-0.070567),(0.111383,0.759118,-0.00552014),(0.107334,0.724864,0.27759),(0.116703,0.740499,-0.070567),(0.125653,0.744141,0.27759),(0.125653,0.744141,0.27759),(0.111383,0.759118,-0.00552014),(0.129614,0.759118,0.27759),(0.168497,0.738534,-0.469564),(0.177677,0.759118,-0.640263),(0.162965,0.759118,-0.345159),(0.168497,0.738534,-0.469564),(0.20102,0.740499,-0.735314),(0.177677,0.759118,-0.640263),(0.177677,0.759118,-0.640263),(0.20102,0.740499,-0.735314),(0.240804,0.759118,-0.80889),(0.20102,0.740499,-0.735314),(0.264127,0.65237,-0.901384),(0.240804,0.759118,-0.80889),(0.264127,0.65237,-0.901384),(0.235264,0.759118,-0.85816),(0.240804,0.759118,-0.80889),(-0.00303999,0.744141,0.978427),(0.0291344,0.759118,0.978427),(-0.00303985,0.759118,0.999996),(-0.00304063,0.724864,0.878668),(0.0369577,0.740499,0.931031),(-0.00303999,0.744141,0.978427),(0.0369577,0.740499,0.931031),(0.0291344,0.759118,0.978427),(-0.00303999,0.744141,0.978427),(0.129614,0.759118,0.27759),(0.111383,0.759118,-0.00552014),(0.125653,0.794038,0.27759),(0.125653,0.794038,0.27759),(0.111383,0.759118,-0.00552014),(0.116703,0.80253,-0.070567),(0.111383,0.759118,-0.00552014),(0.162965,0.759118,-0.345159),(0.116703,0.80253,-0.070567),(0.116703,0.80253,-0.070567),(0.162965,0.759118,-0.345159),(0.168497,0.807112,-0.469564),(0.162965,0.759118,-0.345159),(0.177677,0.759118,-0.640263),(0.168497,0.807112,-0.469564),(0.177677,0.759118,-0.640263),(0.20102,0.80253,-0.735314),(0.168497,0.807112,-0.469564),(0.116873,0.838984,-0.680865),(0.20102,0.80253,-0.735314),(0.336595,1.14936,-1.00991),(0.177677,0.759118,-0.640263),(0.240806,0.759118,-0.808893),(0.20102,0.80253,-0.735314),(0.20102,0.80253,-0.735314),(0.240806,0.759118,-0.808893),(0.336595,1.14936,-1.00991),(0.240806,0.759118,-0.808893),(0.235264,0.759118,-0.85816),(0.336595,1.14936,-1.00991),(0.125653,0.794038,0.27759),(0.116703,0.80253,-0.070567),(0.107334,0.838984,0.27759),(0.107334,0.838984,0.27759),(0.116703,0.80253,-0.070567),(0.0969086,0.855105,-0.11018),(0.116703,0.80253,-0.070567),(0.168497,0.807112,-0.469564),(0.0969086,0.855105,-0.11018),(0.0969086,0.855105,-0.11018),(0.168497,0.807112,-0.469564),(0.131181,0.855105,-0.476997),(0.168497,0.807112,-0.469564),(0.20102,0.80253,-0.735314),(0.131181,0.855105,-0.476997),(0.20102,0.80253,-0.735314),(0.116873,0.838984,-0.680865),(0.131181,0.855105,-0.476997),(0.107334,0.838984,0.27759),(0.0969086,0.855105,-0.11018),(0.070541,0.878917,0.27759),(0.070541,0.878917,0.27759),(0.0969086,0.855105,-0.11018),(0.0491952,0.889352,-0.084014),(0.0969086,0.855105,-0.11018),(0.131181,0.855105,-0.476997),(0.0491952,0.889352,-0.084014),(0.131181,0.855105,-0.476997),(0.0698174,0.878917,-0.363713),(0.0491952,0.889352,-0.084014),(0.0491952,0.889352,-0.084014),(0.0698174,0.878917,-0.363713),(0.00277551,0.898799,-0.0271546),(0.070541,0.878917,0.27759),(0.0491952,0.889352,-0.084014),(0.0291299,0.898799,0.27759),(0.0291299,0.898799,0.27759),(0.0491952,0.889352,-0.084014),(0.00277551,0.898799,-0.0271546),(-0.00304449,0.903097,0.277591),(0.0291299,0.898799,0.27759),(0.00277551,0.898799,-0.0271546),(-0.00303986,0.759118,0.999996),(0.0291344,0.759118,0.978427),(-0.00303999,0.794038,0.978427),(0.0291344,0.759118,0.978427),(0.0369577,0.80253,0.931031),(-0.00303999,0.794038,0.978427),(0.070545,0.759118,0.878668),(0.0369577,0.80253,0.931031),(0.0291344,0.759118,0.978427),(0.070545,0.759118,0.878668),(0.0853976,0.807112,0.759194),(0.0369577,0.80253,0.931031),(0.107336,0.759118,0.678308),(0.0853976,0.807112,0.759194),(0.070545,0.759118,0.878668),(0.107336,0.759118,0.678308),(0.116951,0.80253,0.495402),(0.0853976,0.807112,0.759194),(0.107336,0.759118,0.678308),(0.125654,0.759118,0.452799),(0.116951,0.80253,0.495402),(0.125654,0.759118,0.452799),(0.125653,0.794038,0.27759),(0.116951,0.80253,0.495402),(0.125654,0.759118,0.452799),(0.129614,0.759118,0.27759),(0.125653,0.794038,0.27759),(0.0369577,0.80253,0.931031),(-0.00304063,0.838984,0.878668),(-0.00303999,0.794038,0.978427),(0.0369577,0.80253,0.931031),(0.041178,0.855105,0.759194),(-0.00304063,0.838984,0.878668),(0.0853976,0.807112,0.759194),(0.041178,0.855105,0.759194),(0.0369577,0.80253,0.931031),(0.0853976,0.807112,0.759194),(0.085396,0.855105,0.518391),(0.041178,0.855105,0.759194),(0.0853976,0.807112,0.759194),(0.116951,0.80253,0.495402),(0.085396,0.855105,0.518391),(0.116951,0.80253,0.495402),(0.107334,0.838984,0.27759),(0.085396,0.855105,0.518391),(0.116951,0.80253,0.495402),(0.125653,0.794038,0.27759),(0.107334,0.838984,0.27759),(-0.00304063,0.838984,0.878668),(0.041178,0.855105,0.759194),(-0.00304191,0.878917,0.678309),(0.041178,0.855105,0.759194),(0.0369549,0.889352,0.495403),(-0.00304191,0.878917,0.678309),(0.041178,0.855105,0.759194),(0.085396,0.855105,0.518391),(0.0369549,0.889352,0.495403),(0.085396,0.855105,0.518391),(0.070541,0.878917,0.27759),(0.0369549,0.889352,0.495403),(0.085396,0.855105,0.518391),(0.107334,0.838984,0.27759),(0.070541,0.878917,0.27759),(0.0369549,0.889352,0.495403),(-0.00304335,0.898799,0.4528),(-0.00304191,0.878917,0.678309),(0.0369549,0.889352,0.495403),(0.0291299,0.898799,0.27759),(-0.00304335,0.898799,0.4528),(0.0369549,0.889352,0.495403),(0.070541,0.878917,0.27759),(0.0291299,0.898799,0.27759),(-0.00304335,0.898799,0.4528),(0.0291299,0.898799,0.27759),(-0.00304448,0.903097,0.277591),(0.131181,0.855105,-0.476997),(0.116873,0.838984,-0.680865),(0.0996002,0.85895,-0.513226),(0.131181,0.855105,-0.476997),(0.0996002,0.85895,-0.513226),(0.0698174,0.878917,-0.363713),(0.0969086,0.717949,-0.11018),(0.168497,0.738534,-0.469564),(0.1426,0.739516,-0.270066),(0.0969086,0.717949,-0.11018),(0.1426,0.739516,-0.270066),(0.116703,0.740499,-0.070567),(0.131181,0.717949,-0.476997),(0.0996002,0.7163,-0.513226),(0.116873,0.724864,-0.680865),(-0.00303241,0.934544,0.0504887),(0.00278818,0.898799,-0.0271521),(-0.00303347,0.900948,0.122661),(-0.00303241,0.934544,0.0504887),(-0.00303347,0.900948,0.122661),(-0.0030355,0.903097,0.277591),(-0.00264051,0.707736,-0.373331),(-0.00264151,0.7163,-0.526765),(0.035654,0.717949,-0.497215),(1.05985E-6,0.934617,0.0504887),(1.80026E-6,0.898872,-0.0279467),(0.0,0.901021,0.122661),(1.05985E-6,0.934617,0.0504887),(0.0,0.901021,0.122661),(-2.03005E-6,0.90317,0.277591),(1.06986E-6,1.0793,3.47475E-7),(1.05985E-6,0.934617,0.0504887),(-2.03005E-6,0.90317,0.277591),(-0.00262399,0.901562,-0.477891),(-0.00262401,0.85895,-0.493321),(-0.00262701,0.874364,-0.406775),(-0.002624,0.948969,-0.493766),(-0.00262399,0.901562,-0.477891),(-0.00262701,0.874364,-0.406775),(-0.002627,0.693025,-0.620407),(-0.0026265,0.652374,-0.645043),(-0.00262701,0.684348,-0.585814),(-0.0026275,0.716322,-0.526587),(-0.002627,0.693025,-0.620407),(-0.00262701,0.684348,-0.585814),(-0.002627,0.723428,-0.636308),(-0.002627,0.693025,-0.620407),(-0.0026275,0.716322,-0.526587),(-0.103485,0.739931,-0.0978782),(-0.0944054,0.734624,-0.239595),(-0.106573,0.693107,-0.168737),(-0.0944054,0.734624,-0.239595),(-0.102032,0.690461,-0.237269),(-0.106573,0.693107,-0.168737),(-0.106573,0.693107,-0.168737),(-0.102032,0.690461,-0.237269),(-0.109639,0.648327,-0.275536),(-0.118464,0.756954,0.588829),(-0.134253,0.756954,0.279345),(-0.302092,0.706351,0.270908),(-0.00857241,0.724864,-0.701638),(-0.0232391,0.740499,-0.80144),(0.0605016,0.65237,-0.95942),(-0.0232391,0.740499,-0.80144),(-0.00857241,0.724864,-0.701638),(-0.0409195,0.717949,-0.497215),(-0.0232391,0.740499,-0.80144),(-0.0409195,0.717949,-0.497215),(-0.0792145,0.738534,-0.497214),(-0.0792145,0.738534,-0.497214),(-0.0409195,0.717949,-0.497215),(-0.079217,0.717949,-0.128407),(-0.0409195,0.717949,-0.497215),(-0.00262501,0.707736,-0.373331),(-0.037266,0.70326,-0.0931975),(-0.00262501,0.707736,-0.373331),(-0.0026271,0.699209,-0.0279467),(-0.037266,0.70326,-0.0931975),(-0.0409195,0.717949,-0.497215),(-0.0372659,0.70326,-0.0931975),(-0.079217,0.717949,-0.128407),(-0.079217,0.717949,-0.128407),(-0.037266,0.70326,-0.0931975),(-0.0766211,0.707736,0.27759),(-0.113414,0.724864,0.27759),(-0.079217,0.717949,-0.128407),(-0.0766211,0.707736,0.27759),(-0.037266,0.70326,-0.0931975),(-0.0026271,0.699209,-0.0279467),(-0.0352099,0.699209,0.27759),(-0.0026271,0.699209,-0.0279467),(-0.0030355,0.697365,0.277591),(-0.0352099,0.699209,0.27759),(-0.037266,0.70326,-0.0931975),(-0.0352099,0.699209,0.27759),(-0.0766211,0.707736,0.27759),(-0.00303552,0.697365,0.277591),(-0.00303664,0.699209,0.4528),(-0.0352099,0.699209,0.27759),(-0.131733,0.744141,0.27759),(-0.113414,0.724864,0.27759),(-0.123031,0.740499,0.495402),(-0.123031,0.740499,0.495402),(-0.113414,0.724864,0.27759),(-0.091476,0.717949,0.518391),(-0.113414,0.724864,0.27759),(-0.0766211,0.707736,0.27759),(-0.091476,0.717949,0.518391),(-0.00303936,0.724864,0.878668),(-0.047258,0.717949,0.759194),(-0.00303808,0.707736,0.678309),(-0.0766211,0.707736,0.27759),(-0.0430349,0.70326,0.495403),(-0.091476,0.717949,0.518391),(-0.0352099,0.699209,0.27759),(-0.0430349,0.70326,0.495403),(-0.0766211,0.707736,0.27759),(-0.0430349,0.70326,0.495403),(-0.047258,0.717949,0.759194),(-0.091476,0.717949,0.518391),(-0.00303808,0.707736,0.678309),(-0.047258,0.717949,0.759194),(-0.0430349,0.70326,0.495403),(-0.00303664,0.699209,0.4528),(-0.00303808,0.707736,0.678309),(-0.0430349,0.70326,0.495403),(-0.0352099,0.699209,0.27759),(-0.00303664,0.699209,0.4528),(-0.0430349,0.70326,0.495403),(-0.00303936,0.724864,0.878668),(-0.0430376,0.740499,0.931031),(-0.047258,0.717949,0.759194),(-0.0352143,0.759118,0.978427),(-0.076625,0.759118,0.878668),(-0.0430376,0.740499,0.931031),(-0.0914776,0.738534,0.759194),(-0.113416,0.759118,0.678308),(-0.123031,0.740499,0.495402),(-0.0914776,0.738534,0.759194),(-0.123031,0.740499,0.495402),(-0.091476,0.717949,0.518391),(-0.076625,0.759118,0.878668),(-0.113416,0.759118,0.678308),(-0.0914776,0.738534,0.759194),(-0.0430376,0.740499,0.931031),(-0.076625,0.759118,0.878668),(-0.0914776,0.738534,0.759194),(-0.0430376,0.740499,0.931031),(-0.0914776,0.738534,0.759194),(-0.047258,0.717949,0.759194),(-0.047258,0.717949,0.759194),(-0.0914776,0.738534,0.759194),(-0.091476,0.717949,0.518391),(-0.113416,0.759118,0.678308),(-0.131734,0.759118,0.452799),(-0.123031,0.740499,0.495402),(-0.131734,0.759118,0.452799),(-0.131733,0.744141,0.27759),(-0.123031,0.740499,0.495402),(-0.135694,0.759118,0.27759),(-0.131733,0.744141,0.27759),(-0.131734,0.759118,0.452799),(-0.106544,0.740499,-0.0931984),(-0.0792145,0.738534,-0.497214),(-0.0928796,0.739516,-0.295207),(-0.098215,0.759118,-0.373333),(-0.0792145,0.738534,-0.497214),(-0.106544,0.740499,-0.0931984),(-0.098215,0.759118,-0.373333),(-0.106544,0.740499,-0.0931984),(-0.131731,0.759118,-0.0322708),(-0.106544,0.740499,-0.0931984),(-0.079217,0.717949,-0.128407),(-0.113414,0.724864,0.27759),(-0.131731,0.759118,-0.0322708),(-0.106544,0.740499,-0.0931984),(-0.131733,0.744141,0.27759),(-0.131733,0.744141,0.27759),(-0.106544,0.740499,-0.0931984),(-0.113414,0.724864,0.27759),(-0.135694,0.759118,0.27759),(-0.131731,0.759118,-0.0322708),(-0.131733,0.744141,0.27759),(-0.098215,0.759118,-0.373333),(-0.079651,0.759118,-0.720684),(-0.0792145,0.738534,-0.497214),(-0.079651,0.759118,-0.720684),(-0.0232391,0.740499,-0.80144),(-0.0792145,0.738534,-0.497214),(0.0060125,0.759118,-0.880377),(-0.0232391,0.740499,-0.80144),(-0.079651,0.759118,-0.720684),(0.0060125,0.759118,-0.880377),(0.0605016,0.65237,-0.95942),(-0.0232391,0.740499,-0.80144),(0.0060125,0.759118,-0.880377),(0.0469623,0.759118,-0.908893),(0.0605016,0.65237,-0.95942),(-0.00304013,0.759118,0.999996),(-0.0352143,0.759118,0.978427),(-0.00304,0.744141,0.978427),(-0.00304,0.744141,0.978427),(-0.0430376,0.740499,0.931031),(-0.00303936,0.724864,0.878668),(-0.00304,0.744141,0.978427),(-0.0352143,0.759118,0.978427),(-0.0430376,0.740499,0.931031),(-0.131733,0.794038,0.27759),(-0.131731,0.759118,-0.0322708),(-0.135694,0.759118,0.27759),(-0.106544,0.80253,-0.0931984),(-0.131731,0.759118,-0.0322708),(-0.131733,0.794038,0.27759),(-0.106544,0.80253,-0.0931984),(-0.098215,0.759118,-0.373333),(-0.131731,0.759118,-0.0322708),(-0.0792145,0.807112,-0.497214),(-0.098215,0.759118,-0.373333),(-0.106544,0.80253,-0.0931984),(-0.0792145,0.807112,-0.497214),(-0.079651,0.759118,-0.720684),(-0.098215,0.759118,-0.373333),(-0.0792145,0.807112,-0.497214),(-0.0232391,0.80253,-0.80144),(-0.079651,0.759118,-0.720684),(0.0944915,1.14936,-1.08627),(-0.0232391,0.80253,-0.80144),(-0.00857241,0.838984,-0.701638),(-0.0232391,0.80253,-0.80144),(0.00601354,0.759118,-0.880381),(-0.079651,0.759118,-0.720684),(0.0944915,1.14936,-1.08627),(0.00601354,0.759118,-0.880381),(-0.0232391,0.80253,-0.80144),(0.0944915,1.14936,-1.08627),(0.0469623,0.759118,-0.908893),(0.00601354,0.759118,-0.880381),(-0.113414,0.838984,0.27759),(-0.106544,0.80253,-0.0931984),(-0.131733,0.794038,0.27759),(-0.079217,0.855105,-0.128407),(-0.106544,0.80253,-0.0931984),(-0.113414,0.838984,0.27759),(-0.079217,0.855105,-0.128407),(-0.0792145,0.807112,-0.497214),(-0.106544,0.80253,-0.0931984),(-0.0409195,0.855105,-0.497215),(-0.0792145,0.807112,-0.497214),(-0.079217,0.855105,-0.128407),(-0.0409195,0.855105,-0.497215),(-0.0232391,0.80253,-0.80144),(-0.0792145,0.807112,-0.497214),(-0.0409195,0.855105,-0.497215),(-0.00857241,0.838984,-0.701638),(-0.0232391,0.80253,-0.80144),(-0.076621,0.878917,0.27759),(-0.079217,0.855105,-0.128407),(-0.113414,0.838984,0.27759),(-0.037266,0.889352,-0.0931975),(-0.079217,0.855105,-0.128407),(-0.076621,0.878917,0.27759),(-0.037266,0.889352,-0.0931975),(-0.0409195,0.855105,-0.497215),(-0.079217,0.855105,-0.128407),(-0.037266,0.889352,-0.0931975),(-0.00262501,0.878917,-0.373331),(-0.0409195,0.855105,-0.497215),(-0.0026271,0.898799,-0.0279467),(-0.00262501,0.878917,-0.373331),(-0.037266,0.889352,-0.0931975),(-0.0352099,0.898799,0.27759),(-0.037266,0.889352,-0.0931975),(-0.076621,0.878917,0.27759),(-0.0026271,0.898799,-0.0279467),(-0.037266,0.889352,-0.0931975),(-0.0352099,0.898799,0.27759),(-0.0026271,0.898799,-0.0279467),(-0.0352099,0.898799,0.27759),(-0.0030355,0.903097,0.277591),(-0.00303999,0.794038,0.978427),(-0.0352143,0.759118,0.978427),(-0.00304013,0.759118,0.999996),(-0.00303999,0.794038,0.978427),(-0.0430376,0.80253,0.931031),(-0.0352143,0.759118,0.978427),(-0.0352143,0.759118,0.978427),(-0.0430376,0.80253,0.931031),(-0.076625,0.759118,0.878668),(-0.0430376,0.80253,0.931031),(-0.0914776,0.807112,0.759194),(-0.076625,0.759118,0.878668),(-0.076625,0.759118,0.878668),(-0.0914776,0.807112,0.759194),(-0.113416,0.759118,0.678308),(-0.0914776,0.807112,0.759194),(-0.123031,0.80253,0.495402),(-0.113416,0.759118,0.678308),(-0.123031,0.80253,0.495402),(-0.131734,0.759118,0.452799),(-0.113416,0.759118,0.678308),(-0.123031,0.80253,0.495402),(-0.131733,0.794038,0.27759),(-0.131734,0.759118,0.452799),(-0.131733,0.794038,0.27759),(-0.135694,0.759118,0.27759),(-0.131734,0.759118,0.452799),(-0.00303999,0.794038,0.978427),(-0.00303936,0.838984,0.878668),(-0.0430376,0.80253,0.931031),(-0.00303936,0.838984,0.878668),(-0.047258,0.855105,0.759194),(-0.0430376,0.80253,0.931031),(-0.0430376,0.80253,0.931031),(-0.047258,0.855105,0.759194),(-0.0914776,0.807112,0.759194),(-0.047258,0.855105,0.759194),(-0.091476,0.855105,0.518391),(-0.0914776,0.807112,0.759194),(-0.091476,0.855105,0.518391),(-0.123031,0.80253,0.495402),(-0.0914776,0.807112,0.759194),(-0.091476,0.855105,0.518391),(-0.113414,0.838984,0.27759),(-0.123031,0.80253,0.495402),(-0.113414,0.838984,0.27759),(-0.131733,0.794038,0.27759),(-0.123031,0.80253,0.495402),(-0.00303808,0.878917,0.678309),(-0.047258,0.855105,0.759194),(-0.00303936,0.838984,0.878668),(-0.00303808,0.878917,0.678309),(-0.0430349,0.889352,0.495403),(-0.047258,0.855105,0.759194),(-0.0430349,0.889352,0.495403),(-0.091476,0.855105,0.518391),(-0.047258,0.855105,0.759194),(-0.0430349,0.889352,0.495403),(-0.076621,0.878917,0.27759),(-0.091476,0.855105,0.518391),(-0.076621,0.878917,0.27759),(-0.113414,0.838984,0.27759),(-0.091476,0.855105,0.518391),(-0.00303808,0.878917,0.678309),(-0.00303664,0.898799,0.4528),(-0.0430349,0.889352,0.495403),(-0.00303664,0.898799,0.4528),(-0.0352099,0.898799,0.27759),(-0.0430349,0.889352,0.495403),(-0.0352099,0.898799,0.27759),(-0.076621,0.878917,0.27759),(-0.0430349,0.889352,0.495403),(-0.00303551,0.903097,0.277591),(-0.0352099,0.898799,0.27759),(-0.00303664,0.898799,0.4528),(-0.002624,0.85895,-0.526765),(-0.00857241,0.838984,-0.701638),(-0.0409195,0.855105,-0.497215),(-0.00262501,0.878917,-0.373331),(-0.002624,0.85895,-0.526765),(-0.0409195,0.855105,-0.497215),(-0.0928796,0.739516,-0.295207),(-0.0792145,0.738534,-0.497214),(-0.079217,0.717949,-0.128407),(-0.106544,0.740499,-0.0931984),(-0.0928796,0.739516,-0.295207),(-0.079217,0.717949,-0.128407),(-0.00857241,0.724864,-0.701638),(-0.002624,0.7163,-0.526765),(-0.0409195,0.717949,-0.497215),(-0.0409195,0.717949,-0.497215),(-0.002624,0.7163,-0.526765),(-0.00262501,0.707736,-0.373331),(0.101308,0.693107,-0.168737),(0.0891401,0.734624,-0.239595),(0.0982192,0.739931,-0.0978782),(0.101308,0.693107,-0.168737),(0.0967671,0.690461,-0.237269),(0.0891401,0.734624,-0.239595),(0.104374,0.648327,-0.275536),(0.0967671,0.690461,-0.237269),(0.101308,0.693107,-0.168737),(0.296012,0.706351,0.270908),(0.128173,0.756954,0.279345),(0.112384,0.756954,0.588829),(0.0604765,0.65237,-0.959427),(0.0540084,0.740499,-0.780742),(-0.00859463,0.724864,-0.701644),(0.035654,0.717949,-0.497215),(-0.00859463,0.724864,-0.701644),(0.0540084,0.740499,-0.780742),(0.073949,0.738534,-0.497215),(0.035654,0.717949,-0.497215),(0.0540084,0.740499,-0.780742),(0.0739513,0.717949,-0.128407),(0.035654,0.717949,-0.497215),(0.073949,0.738534,-0.497215),(0.0320005,0.70326,-0.0931975),(-0.00264051,0.707736,-0.373331),(0.035654,0.717949,-0.497215),(0.0320005,0.70326,-0.0931975),(-0.0026383,0.699209,-0.0279467),(-0.00264051,0.707736,-0.373331),(0.0739513,0.717949,-0.128407),(0.0320005,0.70326,-0.0931975),(0.035654,0.717949,-0.497215),(0.0705411,0.707736,0.27759),(0.0320005,0.70326,-0.0931975),(0.0739513,0.717949,-0.128407),(0.0705411,0.707736,0.27759),(0.0739513,0.717949,-0.128407),(0.107334,0.724864,0.27759),(0.0291299,0.699209,0.27759),(-0.0026383,0.699209,-0.0279467),(0.0320005,0.70326,-0.0931975),(0.0291299,0.699209,0.27759),(-0.00304449,0.697365,0.277591),(-0.0026383,0.699209,-0.0279467),(0.0705411,0.707736,0.27759),(0.0291299,0.699209,0.27759),(0.0320005,0.70326,-0.0931975),(0.0291299,0.699209,0.27759),(-0.00304335,0.699209,0.4528),(-0.00304447,0.697365,0.277591),(0.116951,0.740499,0.495402),(0.107334,0.724864,0.27759),(0.125653,0.744141,0.27759),(0.085396,0.717949,0.518391),(0.107334,0.724864,0.27759),(0.116951,0.740499,0.495402),(0.085396,0.717949,0.518391),(0.0705411,0.707736,0.27759),(0.107334,0.724864,0.27759),(-0.00304191,0.707736,0.678309),(0.041178,0.717949,0.759194),(-0.00304063,0.724864,0.878668),(0.085396,0.717949,0.518391),(0.0369549,0.70326,0.495403),(0.0705411,0.707736,0.27759),(0.0705411,0.707736,0.27759),(0.0369549,0.70326,0.495403),(0.0291299,0.699209,0.27759),(0.085396,0.717949,0.518391),(0.041178,0.717949,0.759194),(0.0369549,0.70326,0.495403),(0.0369549,0.70326,0.495403),(0.041178,0.717949,0.759194),(-0.00304191,0.707736,0.678309),(0.0369549,0.70326,0.495403),(-0.00304191,0.707736,0.678309),(-0.00304335,0.699209,0.4528),(0.0369549,0.70326,0.495403),(-0.00304335,0.699209,0.4528),(0.0291299,0.699209,0.27759),(0.041178,0.717949,0.759194),(0.0369577,0.740499,0.931031),(-0.00304063,0.724864,0.878668),(0.0369577,0.740499,0.931031),(0.070545,0.759118,0.878668),(0.0291344,0.759118,0.978427),(0.116951,0.740499,0.495402),(0.107336,0.759118,0.678308),(0.0853976,0.738534,0.759194),(0.085396,0.717949,0.518391),(0.116951,0.740499,0.495402),(0.0853976,0.738534,0.759194),(0.0853976,0.738534,0.759194),(0.107336,0.759118,0.678308),(0.070545,0.759118,0.878668),(0.0853976,0.738534,0.759194),(0.070545,0.759118,0.878668),(0.0369577,0.740499,0.931031),(0.041178,0.717949,0.759194),(0.0853976,0.738534,0.759194),(0.0369577,0.740499,0.931031),(0.085396,0.717949,0.518391),(0.0853976,0.738534,0.759194),(0.041178,0.717949,0.759194),(0.116951,0.740499,0.495402),(0.125654,0.759118,0.452799),(0.107336,0.759118,0.678308),(0.116951,0.740499,0.495402),(0.125653,0.744141,0.27759),(0.125654,0.759118,0.452799),(0.125654,0.759118,0.452799),(0.125653,0.744141,0.27759),(0.129614,0.759118,0.27759),(0.0876139,0.739516,-0.295207),(0.073949,0.738534,-0.497215),(0.101279,0.740499,-0.0931984),(0.101279,0.740499,-0.0931984),(0.073949,0.738534,-0.497215),(0.0929495,0.759118,-0.373333),(0.108817,0.759118,-0.0279473),(0.101279,0.740499,-0.0931984),(0.0929495,0.759118,-0.373333),(0.107334,0.724864,0.27759),(0.0739513,0.717949,-0.128407),(0.101279,0.740499,-0.0931984),(0.125653,0.744141,0.27759),(0.101279,0.740499,-0.0931984),(0.108817,0.759118,-0.0279473),(0.107334,0.724864,0.27759),(0.101279,0.740499,-0.0931984),(0.125653,0.744141,0.27759),(0.125653,0.744141,0.27759),(0.108817,0.759118,-0.0279473),(0.129614,0.759118,0.27759),(0.073949,0.738534,-0.497215),(0.0624843,0.759118,-0.6826),(0.0929495,0.759118,-0.373333),(0.073949,0.738534,-0.497215),(0.0540084,0.740499,-0.780742),(0.0624843,0.759118,-0.6826),(0.0624843,0.759118,-0.6826),(0.0540084,0.740499,-0.780742),(0.068145,0.759118,-0.86373),(0.0540084,0.740499,-0.780742),(0.0604765,0.65237,-0.959427),(0.068145,0.759118,-0.86373),(0.0604765,0.65237,-0.959427),(0.0469382,0.759118,-0.908899),(0.068145,0.759118,-0.86373),(-0.00303999,0.744141,0.978427),(0.0291344,0.759118,0.978427),(-0.00303985,0.759118,0.999996),(-0.00304063,0.724864,0.878668),(0.0369577,0.740499,0.931031),(-0.00303999,0.744141,0.978427),(0.0369577,0.740499,0.931031),(0.0291344,0.759118,0.978427),(-0.00303999,0.744141,0.978427),(0.129614,0.759118,0.27759),(0.108817,0.759118,-0.0279473),(0.125653,0.794038,0.27759),(0.125653,0.794038,0.27759),(0.108817,0.759118,-0.0279473),(0.101279,0.80253,-0.0931984),(0.108817,0.759118,-0.0279473),(0.0929495,0.759118,-0.373333),(0.101279,0.80253,-0.0931984),(0.101279,0.80253,-0.0931984),(0.0929495,0.759118,-0.373333),(0.073949,0.807112,-0.497215),(0.0929495,0.759118,-0.373333),(0.0624843,0.759118,-0.6826),(0.073949,0.807112,-0.497215),(0.0624843,0.759118,-0.6826),(0.0540084,0.80253,-0.780742),(0.073949,0.807112,-0.497215),(-0.00859463,0.838984,-0.701644),(0.0540084,0.80253,-0.780742),(0.094468,1.14936,-1.08629),(0.0624843,0.759118,-0.6826),(0.0681453,0.759118,-0.863733),(0.0540084,0.80253,-0.780742),(0.0540084,0.80253,-0.780742),(0.0681453,0.759118,-0.863733),(0.094468,1.14936,-1.08629),(0.0681453,0.759118,-0.863733),(0.0469382,0.759118,-0.908899),(0.094468,1.14936,-1.08629),(0.125653,0.794038,0.27759),(0.101279,0.80253,-0.0931984),(0.107334,0.838984,0.27759),(0.107334,0.838984,0.27759),(0.101279,0.80253,-0.0931984),(0.0739513,0.855105,-0.128407),(0.101279,0.80253,-0.0931984),(0.073949,0.807112,-0.497215),(0.0739513,0.855105,-0.128407),(0.0739513,0.855105,-0.128407),(0.073949,0.807112,-0.497215),(0.035654,0.855105,-0.497215),(0.073949,0.807112,-0.497215),(0.0540084,0.80253,-0.780742),(0.035654,0.855105,-0.497215),(0.0540084,0.80253,-0.780742),(-0.00859463,0.838984,-0.701644),(0.035654,0.855105,-0.497215),(0.107334,0.838984,0.27759),(0.0739513,0.855105,-0.128407),(0.070541,0.878917,0.27759),(0.070541,0.878917,0.27759),(0.0739513,0.855105,-0.128407),(0.0320005,0.889352,-0.0931975),(0.0739513,0.855105,-0.128407),(0.035654,0.855105,-0.497215),(0.0320005,0.889352,-0.0931975),(0.035654,0.855105,-0.497215),(-0.00264051,0.878917,-0.373331),(0.0320005,0.889352,-0.0931975),(0.0320005,0.889352,-0.0931975),(-0.00264051,0.878917,-0.373331),(-0.0026383,0.898799,-0.0279467),(0.070541,0.878917,0.27759),(0.0320005,0.889352,-0.0931975),(0.0291299,0.898799,0.27759),(0.0291299,0.898799,0.27759),(0.0320005,0.889352,-0.0931975),(-0.0026383,0.898799,-0.0279467),(-0.00304449,0.903097,0.277591),(0.0291299,0.898799,0.27759),(-0.0026383,0.898799,-0.0279467),(-0.00303986,0.759118,0.999996),(0.0291344,0.759118,0.978427),(-0.00303999,0.794038,0.978427),(0.0291344,0.759118,0.978427),(0.0369577,0.80253,0.931031),(-0.00303999,0.794038,0.978427),(0.070545,0.759118,0.878668),(0.0369577,0.80253,0.931031),(0.0291344,0.759118,0.978427),(0.070545,0.759118,0.878668),(0.0853976,0.807112,0.759194),(0.0369577,0.80253,0.931031),(0.107336,0.759118,0.678308),(0.0853976,0.807112,0.759194),(0.070545,0.759118,0.878668),(0.107336,0.759118,0.678308),(0.116951,0.80253,0.495402),(0.0853976,0.807112,0.759194),(0.107336,0.759118,0.678308),(0.125654,0.759118,0.452799),(0.116951,0.80253,0.495402),(0.125654,0.759118,0.452799),(0.125653,0.794038,0.27759),(0.116951,0.80253,0.495402),(0.125654,0.759118,0.452799),(0.129614,0.759118,0.27759),(0.125653,0.794038,0.27759),(0.0369577,0.80253,0.931031),(-0.00304063,0.838984,0.878668),(-0.00303999,0.794038,0.978427),(0.0369577,0.80253,0.931031),(0.041178,0.855105,0.759194),(-0.00304063,0.838984,0.878668),(0.0853976,0.807112,0.759194),(0.041178,0.855105,0.759194),(0.0369577,0.80253,0.931031),(0.0853976,0.807112,0.759194),(0.085396,0.855105,0.518391),(0.041178,0.855105,0.759194),(0.0853976,0.807112,0.759194),(0.116951,0.80253,0.495402),(0.085396,0.855105,0.518391),(0.116951,0.80253,0.495402),(0.107334,0.838984,0.27759),(0.085396,0.855105,0.518391),(0.116951,0.80253,0.495402),(0.125653,0.794038,0.27759),(0.107334,0.838984,0.27759),(-0.00304063,0.838984,0.878668),(0.041178,0.855105,0.759194),(-0.00304191,0.878917,0.678309),(0.041178,0.855105,0.759194),(0.0369549,0.889352,0.495403),(-0.00304191,0.878917,0.678309),(0.041178,0.855105,0.759194),(0.085396,0.855105,0.518391),(0.0369549,0.889352,0.495403),(0.085396,0.855105,0.518391),(0.070541,0.878917,0.27759),(0.0369549,0.889352,0.495403),(0.085396,0.855105,0.518391),(0.107334,0.838984,0.27759),(0.070541,0.878917,0.27759),(0.0369549,0.889352,0.495403),(-0.00304335,0.898799,0.4528),(-0.00304191,0.878917,0.678309),(0.0369549,0.889352,0.495403),(0.0291299,0.898799,0.27759),(-0.00304335,0.898799,0.4528),(0.0369549,0.889352,0.495403),(0.070541,0.878917,0.27759),(0.0291299,0.898799,0.27759),(-0.00304335,0.898799,0.4528),(0.0291299,0.898799,0.27759),(-0.00304448,0.903097,0.277591),(0.035654,0.855105,-0.497215),(-0.00859463,0.838984,-0.701644),(-0.00264151,0.85895,-0.526765),(0.035654,0.855105,-0.497215),(-0.00264151,0.85895,-0.526765),(-0.00264051,0.878917,-0.373331),(0.0739513,0.717949,-0.128407),(0.073949,0.738534,-0.497215),(0.0876139,0.739516,-0.295207),(0.0739513,0.717949,-0.128407),(0.0876139,0.739516,-0.295207),(0.101279,0.740499,-0.0931984),(0.035654,0.717949,-0.497215),(-0.00264151,0.7163,-0.526765),(-0.00859463,0.724864,-0.701644),(-0.00303241,0.934544,0.0504887),(-0.0026253,0.898799,-0.0279467),(-0.00303347,0.900948,0.122661),(-0.00303241,0.934544,0.0504887),(-0.00303347,0.900948,0.122661),(-0.0030355,0.903097,0.277591),(-0.0992783,0.707736,-0.360714),(-0.138991,0.7163,-0.508919),(-0.0943522,0.717949,-0.490288),(1.05985E-6,0.934617,0.0504887),(-0.00733395,0.898872,-0.0277818),(0.0,0.901021,0.122661),(1.05985E-6,0.934617,0.0504887),(0.0,0.901021,0.122661),(-2.03005E-6,0.90317,0.277591),(1.06986E-6,1.0793,3.47475E-7),(1.05985E-6,0.934617,0.0504887),(-2.03005E-6,0.90317,0.277591),(-0.126324,0.901562,-0.461715),(-0.130318,0.85895,-0.476619),(-0.107921,0.874364,-0.393021),(-0.130433,0.948969,-0.477049),(-0.126324,0.901562,-0.461715),(-0.107921,0.874364,-0.393021),(-0.163213,0.693025,-0.599374),(-0.169589,0.652374,-0.623171),(-0.15426,0.684348,-0.56596),(-0.138931,0.716322,-0.508751),(-0.163213,0.693025,-0.599374),(-0.15426,0.684348,-0.56596),(-0.167329,0.723428,-0.614733),(-0.163213,0.693025,-0.599374),(-0.138931,0.716322,-0.508751),(-0.125394,0.739931,-0.0685461),(-0.153303,0.734624,-0.207784),(-0.146717,0.693107,-0.136191),(-0.153303,0.734624,-0.207784),(-0.160068,0.690461,-0.203563),(-0.146717,0.693107,-0.136191),(-0.146717,0.693107,-0.136191),(-0.160068,0.690461,-0.203563),(-0.17732,0.648327,-0.238558),(-0.118464,0.756954,0.588829),(-0.134253,0.756954,0.279345),(-0.302092,0.706351,0.270908),(-0.190452,0.724864,-0.67732),(-0.230449,0.740499,-0.769925),(-0.19045,0.65237,-0.944196),(-0.230449,0.740499,-0.769925),(-0.190452,0.724864,-0.67732),(-0.168316,0.717949,-0.470469),(-0.230449,0.740499,-0.769925),(-0.168316,0.717949,-0.470469),(-0.205306,0.738534,-0.460557),(-0.205306,0.738534,-0.460557),(-0.168316,0.717949,-0.470469),(-0.109854,0.717949,-0.104316),(-0.168316,0.717949,-0.470469),(-0.0992633,0.707736,-0.360718),(-0.06022,0.70326,-0.0811637),(-0.0992633,0.707736,-0.360718),(-0.00987328,0.699209,-0.0271014),(-0.06022,0.70326,-0.0811637),(-0.168316,0.717949,-0.470469),(-0.0602199,0.70326,-0.0811637),(-0.109854,0.717949,-0.104316),(-0.109854,0.717949,-0.104316),(-0.06022,0.70326,-0.0811637),(-0.0766211,0.707736,0.27759),(-0.113414,0.724864,0.27759),(-0.109854,0.717949,-0.104316),(-0.0766211,0.707736,0.27759),(-0.06022,0.70326,-0.0811637),(-0.00987328,0.699209,-0.0271014),(-0.0352099,0.699209,0.27759),(-0.00987328,0.699209,-0.0271014),(-0.0030355,0.697365,0.277591),(-0.0352099,0.699209,0.27759),(-0.06022,0.70326,-0.0811637),(-0.0352099,0.699209,0.27759),(-0.0766211,0.707736,0.27759),(-0.00303552,0.697365,0.277591),(-0.00303664,0.699209,0.4528),(-0.0352099,0.699209,0.27759),(-0.131733,0.744141,0.27759),(-0.113414,0.724864,0.27759),(-0.123031,0.740499,0.495402),(-0.123031,0.740499,0.495402),(-0.113414,0.724864,0.27759),(-0.091476,0.717949,0.518391),(-0.113414,0.724864,0.27759),(-0.0766211,0.707736,0.27759),(-0.091476,0.717949,0.518391),(-0.00303936,0.724864,0.878668),(-0.047258,0.717949,0.759194),(-0.00303808,0.707736,0.678309),(-0.0766211,0.707736,0.27759),(-0.0430349,0.70326,0.495403),(-0.091476,0.717949,0.518391),(-0.0352099,0.699209,0.27759),(-0.0430349,0.70326,0.495403),(-0.0766211,0.707736,0.27759),(-0.0430349,0.70326,0.495403),(-0.047258,0.717949,0.759194),(-0.091476,0.717949,0.518391),(-0.00303808,0.707736,0.678309),(-0.047258,0.717949,0.759194),(-0.0430349,0.70326,0.495403),(-0.00303664,0.699209,0.4528),(-0.00303808,0.707736,0.678309),(-0.0430349,0.70326,0.495403),(-0.0352099,0.699209,0.27759),(-0.00303664,0.699209,0.4528),(-0.0430349,0.70326,0.495403),(-0.00303936,0.724864,0.878668),(-0.0430376,0.740499,0.931031),(-0.047258,0.717949,0.759194),(-0.0352143,0.759118,0.978427),(-0.076625,0.759118,0.878668),(-0.0430376,0.740499,0.931031),(-0.0914776,0.738534,0.759194),(-0.113416,0.759118,0.678308),(-0.123031,0.740499,0.495402),(-0.0914776,0.738534,0.759194),(-0.123031,0.740499,0.495402),(-0.091476,0.717949,0.518391),(-0.076625,0.759118,0.878668),(-0.113416,0.759118,0.678308),(-0.0914776,0.738534,0.759194),(-0.0430376,0.740499,0.931031),(-0.076625,0.759118,0.878668),(-0.0914776,0.738534,0.759194),(-0.0430376,0.740499,0.931031),(-0.0914776,0.738534,0.759194),(-0.047258,0.717949,0.759194),(-0.047258,0.717949,0.759194),(-0.0914776,0.738534,0.759194),(-0.091476,0.717949,0.518391),(-0.113416,0.759118,0.678308),(-0.131734,0.759118,0.452799),(-0.123031,0.740499,0.495402),(-0.131734,0.759118,0.452799),(-0.131733,0.744141,0.27759),(-0.123031,0.740499,0.495402),(-0.135694,0.759118,0.27759),(-0.131733,0.744141,0.27759),(-0.131734,0.759118,0.452799),(-0.127138,0.740499,-0.0632341),(-0.205306,0.738534,-0.460557),(-0.166223,0.739516,-0.261896),(-0.191597,0.759118,-0.335979),(-0.205306,0.738534,-0.460557),(-0.127138,0.740499,-0.0632341),(-0.191597,0.759118,-0.335979),(-0.127138,0.740499,-0.0632341),(-0.135697,0.759118,0.00213635),(-0.127138,0.740499,-0.0632341),(-0.109854,0.717949,-0.104316),(-0.113414,0.724864,0.27759),(-0.135697,0.759118,0.00213635),(-0.127138,0.740499,-0.0632341),(-0.131733,0.744141,0.27759),(-0.131733,0.744141,0.27759),(-0.127138,0.740499,-0.0632341),(-0.113414,0.724864,0.27759),(-0.135694,0.759118,0.27759),(-0.135697,0.759118,0.00213635),(-0.131733,0.744141,0.27759),(-0.191597,0.759118,-0.335979),(-0.264038,0.759118,-0.67732),(-0.205306,0.738534,-0.460557),(-0.264038,0.759118,-0.67732),(-0.230449,0.740499,-0.769925),(-0.205306,0.738534,-0.460557),(-0.222625,0.759118,-0.853743),(-0.230449,0.740499,-0.769925),(-0.264038,0.759118,-0.67732),(-0.222625,0.759118,-0.853743),(-0.19045,0.65237,-0.944196),(-0.230449,0.740499,-0.769925),(-0.222625,0.759118,-0.853743),(-0.190451,0.759118,-0.891886),(-0.19045,0.65237,-0.944196),(-0.00304013,0.759118,0.999996),(-0.0352143,0.759118,0.978427),(-0.00304,0.744141,0.978427),(-0.00304,0.744141,0.978427),(-0.0430376,0.740499,0.931031),(-0.00303936,0.724864,0.878668),(-0.00304,0.744141,0.978427),(-0.0352143,0.759118,0.978427),(-0.0430376,0.740499,0.931031),(-0.131733,0.794038,0.27759),(-0.135697,0.759118,0.00213635),(-0.135694,0.759118,0.27759),(-0.127138,0.80253,-0.0632341),(-0.135697,0.759118,0.00213635),(-0.131733,0.794038,0.27759),(-0.127138,0.80253,-0.0632341),(-0.191597,0.759118,-0.335979),(-0.135697,0.759118,0.00213635),(-0.205306,0.807112,-0.460557),(-0.191597,0.759118,-0.335979),(-0.127138,0.80253,-0.0632341),(-0.205306,0.807112,-0.460557),(-0.264038,0.759118,-0.67732),(-0.191597,0.759118,-0.335979),(-0.205306,0.807112,-0.460557),(-0.230449,0.80253,-0.769925),(-0.264038,0.759118,-0.67732),(-0.19045,1.14936,-1.07552),(-0.230449,0.80253,-0.769925),(-0.190452,0.838984,-0.67732),(-0.230449,0.80253,-0.769925),(-0.222625,0.759118,-0.853747),(-0.264038,0.759118,-0.67732),(-0.19045,1.14936,-1.07552),(-0.222625,0.759118,-0.853747),(-0.230449,0.80253,-0.769925),(-0.19045,1.14936,-1.07552),(-0.190451,0.759118,-0.891886),(-0.222625,0.759118,-0.853747),(-0.113414,0.838984,0.27759),(-0.127138,0.80253,-0.0632341),(-0.131733,0.794038,0.27759),(-0.109854,0.855105,-0.104316),(-0.127138,0.80253,-0.0632341),(-0.113414,0.838984,0.27759),(-0.109854,0.855105,-0.104316),(-0.205306,0.807112,-0.460557),(-0.127138,0.80253,-0.0632341),(-0.168316,0.855105,-0.470469),(-0.205306,0.807112,-0.460557),(-0.109854,0.855105,-0.104316),(-0.168316,0.855105,-0.470469),(-0.230449,0.80253,-0.769925),(-0.205306,0.807112,-0.460557),(-0.168316,0.855105,-0.470469),(-0.190452,0.838984,-0.67732),(-0.230449,0.80253,-0.769925),(-0.076621,0.878917,0.27759),(-0.109854,0.855105,-0.104316),(-0.113414,0.838984,0.27759),(-0.06022,0.889352,-0.0811637),(-0.109854,0.855105,-0.104316),(-0.076621,0.878917,0.27759),(-0.06022,0.889352,-0.0811637),(-0.168316,0.855105,-0.470469),(-0.109854,0.855105,-0.104316),(-0.06022,0.889352,-0.0811637),(-0.0992633,0.878917,-0.360718),(-0.168316,0.855105,-0.470469),(-0.00987328,0.898799,-0.0271014),(-0.0992633,0.878917,-0.360718),(-0.06022,0.889352,-0.0811637),(-0.0352099,0.898799,0.27759),(-0.06022,0.889352,-0.0811637),(-0.076621,0.878917,0.27759),(-0.00987328,0.898799,-0.0271014),(-0.06022,0.889352,-0.0811637),(-0.0352099,0.898799,0.27759),(-0.00987328,0.898799,-0.0271014),(-0.0352099,0.898799,0.27759),(-0.0030355,0.903097,0.277591),(-0.00303999,0.794038,0.978427),(-0.0352143,0.759118,0.978427),(-0.00304013,0.759118,0.999996),(-0.00303999,0.794038,0.978427),(-0.0430376,0.80253,0.931031),(-0.0352143,0.759118,0.978427),(-0.0352143,0.759118,0.978427),(-0.0430376,0.80253,0.931031),(-0.076625,0.759118,0.878668),(-0.0430376,0.80253,0.931031),(-0.0914776,0.807112,0.759194),(-0.076625,0.759118,0.878668),(-0.076625,0.759118,0.878668),(-0.0914776,0.807112,0.759194),(-0.113416,0.759118,0.678308),(-0.0914776,0.807112,0.759194),(-0.123031,0.80253,0.495402),(-0.113416,0.759118,0.678308),(-0.123031,0.80253,0.495402),(-0.131734,0.759118,0.452799),(-0.113416,0.759118,0.678308),(-0.123031,0.80253,0.495402),(-0.131733,0.794038,0.27759),(-0.131734,0.759118,0.452799),(-0.131733,0.794038,0.27759),(-0.135694,0.759118,0.27759),(-0.131734,0.759118,0.452799),(-0.00303999,0.794038,0.978427),(-0.00303936,0.838984,0.878668),(-0.0430376,0.80253,0.931031),(-0.00303936,0.838984,0.878668),(-0.047258,0.855105,0.759194),(-0.0430376,0.80253,0.931031),(-0.0430376,0.80253,0.931031),(-0.047258,0.855105,0.759194),(-0.0914776,0.807112,0.759194),(-0.047258,0.855105,0.759194),(-0.091476,0.855105,0.518391),(-0.0914776,0.807112,0.759194),(-0.091476,0.855105,0.518391),(-0.123031,0.80253,0.495402),(-0.0914776,0.807112,0.759194),(-0.091476,0.855105,0.518391),(-0.113414,0.838984,0.27759),(-0.123031,0.80253,0.495402),(-0.113414,0.838984,0.27759),(-0.131733,0.794038,0.27759),(-0.123031,0.80253,0.495402),(-0.00303808,0.878917,0.678309),(-0.047258,0.855105,0.759194),(-0.00303936,0.838984,0.878668),(-0.00303808,0.878917,0.678309),(-0.0430349,0.889352,0.495403),(-0.047258,0.855105,0.759194),(-0.0430349,0.889352,0.495403),(-0.091476,0.855105,0.518391),(-0.047258,0.855105,0.759194),(-0.0430349,0.889352,0.495403),(-0.076621,0.878917,0.27759),(-0.091476,0.855105,0.518391),(-0.076621,0.878917,0.27759),(-0.113414,0.838984,0.27759),(-0.091476,0.855105,0.518391),(-0.00303808,0.878917,0.678309),(-0.00303664,0.898799,0.4528),(-0.0430349,0.889352,0.495403),(-0.00303664,0.898799,0.4528),(-0.0352099,0.898799,0.27759),(-0.0430349,0.889352,0.495403),(-0.0352099,0.898799,0.27759),(-0.076621,0.878917,0.27759),(-0.0430349,0.889352,0.495403),(-0.00303551,0.903097,0.277591),(-0.0352099,0.898799,0.27759),(-0.00303664,0.898799,0.4528),(-0.138974,0.85895,-0.508924),(-0.190452,0.838984,-0.67732),(-0.168316,0.855105,-0.470469),(-0.0992633,0.878917,-0.360718),(-0.138974,0.85895,-0.508924),(-0.168316,0.855105,-0.470469),(-0.166223,0.739516,-0.261896),(-0.205306,0.738534,-0.460557),(-0.109854,0.717949,-0.104316),(-0.127138,0.740499,-0.0632341),(-0.166223,0.739516,-0.261896),(-0.109854,0.717949,-0.104316),(-0.190452,0.724864,-0.67732),(-0.138974,0.7163,-0.508924),(-0.168316,0.717949,-0.470469),(-0.168316,0.717949,-0.470469),(-0.138974,0.7163,-0.508924),(-0.0992633,0.707736,-0.360718),(0.0540811,0.693107,-0.189995),(0.0239884,0.734624,-0.255289),(0.0694372,0.739931,-0.120751),(0.0540811,0.693107,-0.189995),(0.0319575,0.690461,-0.255016),(0.0239884,0.734624,-0.255289),(0.029401,0.648327,-0.293948),(0.0319575,0.690461,-0.255016),(0.0540811,0.693107,-0.189995),(0.296012,0.706351,0.270908),(0.128173,0.756954,0.279345),(0.112384,0.756954,0.588829),(-0.190476,0.65237,-0.944196),(-0.150477,0.740499,-0.769925),(-0.190475,0.724864,-0.67732),(-0.0943522,0.717949,-0.490288),(-0.190475,0.724864,-0.67732),(-0.150477,0.740499,-0.769925),(-0.057362,0.738534,-0.500199),(-0.0943522,0.717949,-0.490288),(-0.150477,0.740499,-0.769925),(0.0380947,0.717949,-0.143959),(-0.0943522,0.717949,-0.490288),(-0.057362,0.738534,-0.500199),(0.00668626,0.70326,-0.0990912),(-0.0992783,0.707736,-0.360714),(-0.0943522,0.717949,-0.490288),(0.00668626,0.70326,-0.0990912),(-0.00988409,0.699209,-0.0270985),(-0.0992783,0.707736,-0.360714),(0.0380947,0.717949,-0.143959),(0.00668626,0.70326,-0.0990912),(-0.0943522,0.717949,-0.490288),(0.0705411,0.707736,0.27759),(0.00668626,0.70326,-0.0990912),(0.0380947,0.717949,-0.143959),(0.0705411,0.707736,0.27759),(0.0380947,0.717949,-0.143959),(0.107334,0.724864,0.27759),(0.0291299,0.699209,0.27759),(-0.00988409,0.699209,-0.0270985),(0.00668626,0.70326,-0.0990912),(0.0291299,0.699209,0.27759),(-0.00304449,0.697365,0.277591),(-0.00988409,0.699209,-0.0270985),(0.0705411,0.707736,0.27759),(0.0291299,0.699209,0.27759),(0.00668626,0.70326,-0.0990912),(0.0291299,0.699209,0.27759),(-0.00304335,0.699209,0.4528),(-0.00304447,0.697365,0.277591),(0.116951,0.740499,0.495402),(0.107334,0.724864,0.27759),(0.125653,0.744141,0.27759),(0.085396,0.717949,0.518391),(0.107334,0.724864,0.27759),(0.116951,0.740499,0.495402),(0.085396,0.717949,0.518391),(0.0705411,0.707736,0.27759),(0.107334,0.724864,0.27759),(-0.00304191,0.707736,0.678309),(0.041178,0.717949,0.759194),(-0.00304063,0.724864,0.878668),(0.085396,0.717949,0.518391),(0.0369549,0.70326,0.495403),(0.0705411,0.707736,0.27759),(0.0705411,0.707736,0.27759),(0.0369549,0.70326,0.495403),(0.0291299,0.699209,0.27759),(0.085396,0.717949,0.518391),(0.041178,0.717949,0.759194),(0.0369549,0.70326,0.495403),(0.0369549,0.70326,0.495403),(0.041178,0.717949,0.759194),(-0.00304191,0.707736,0.678309),(0.0369549,0.70326,0.495403),(-0.00304191,0.707736,0.678309),(-0.00304335,0.699209,0.4528),(0.0369549,0.70326,0.495403),(-0.00304335,0.699209,0.4528),(0.0291299,0.699209,0.27759),(0.041178,0.717949,0.759194),(0.0369577,0.740499,0.931031),(-0.00304063,0.724864,0.878668),(0.0369577,0.740499,0.931031),(0.070545,0.759118,0.878668),(0.0291344,0.759118,0.978427),(0.116951,0.740499,0.495402),(0.107336,0.759118,0.678308),(0.0853976,0.738534,0.759194),(0.085396,0.717949,0.518391),(0.116951,0.740499,0.495402),(0.0853976,0.738534,0.759194),(0.0853976,0.738534,0.759194),(0.107336,0.759118,0.678308),(0.070545,0.759118,0.878668),(0.0853976,0.738534,0.759194),(0.070545,0.759118,0.878668),(0.0369577,0.740499,0.931031),(0.041178,0.717949,0.759194),(0.0853976,0.738534,0.759194),(0.0369577,0.740499,0.931031),(0.085396,0.717949,0.518391),(0.0853976,0.738534,0.759194),(0.041178,0.717949,0.759194),(0.116951,0.740499,0.495402),(0.125654,0.759118,0.452799),(0.107336,0.759118,0.678308),(0.116951,0.740499,0.495402),(0.125653,0.744141,0.27759),(0.125654,0.759118,0.452799),(0.125654,0.759118,0.452799),(0.125653,0.744141,0.27759),(0.129614,0.759118,0.27759),(0.00812078,0.739516,-0.308611),(-0.057362,0.738534,-0.500199),(0.0736039,0.740499,-0.117023),(0.0736039,0.740499,-0.117023),(-0.057362,0.738534,-0.500199),(-0.00694594,0.759118,-0.385456),(0.0977733,0.759118,-0.0559459),(0.0736039,0.740499,-0.117023),(-0.00694594,0.759118,-0.385456),(0.107334,0.724864,0.27759),(0.0380947,0.717949,-0.143959),(0.0736039,0.740499,-0.117023),(0.125653,0.744141,0.27759),(0.0736039,0.740499,-0.117023),(0.0977733,0.759118,-0.0559459),(0.107334,0.724864,0.27759),(0.0736039,0.740499,-0.117023),(0.125653,0.744141,0.27759),(0.125653,0.744141,0.27759),(0.0977733,0.759118,-0.0559459),(0.129614,0.759118,0.27759),(-0.057362,0.738534,-0.500199),(-0.116889,0.759118,-0.677321),(-0.00694594,0.759118,-0.385456),(-0.057362,0.738534,-0.500199),(-0.150477,0.740499,-0.769925),(-0.116889,0.759118,-0.677321),(-0.116889,0.759118,-0.677321),(-0.150477,0.740499,-0.769925),(-0.158301,0.759118,-0.853744),(-0.150477,0.740499,-0.769925),(-0.190476,0.65237,-0.944196),(-0.158301,0.759118,-0.853744),(-0.190476,0.65237,-0.944196),(-0.190476,0.759118,-0.891886),(-0.158301,0.759118,-0.853744),(-0.00303999,0.744141,0.978427),(0.0291344,0.759118,0.978427),(-0.00303985,0.759118,0.999996),(-0.00304063,0.724864,0.878668),(0.0369577,0.740499,0.931031),(-0.00303999,0.744141,0.978427),(0.0369577,0.740499,0.931031),(0.0291344,0.759118,0.978427),(-0.00303999,0.744141,0.978427),(0.129614,0.759118,0.27759),(0.0977733,0.759118,-0.0559459),(0.125653,0.794038,0.27759),(0.125653,0.794038,0.27759),(0.0977733,0.759118,-0.0559459),(0.0736039,0.80253,-0.117023),(0.0977733,0.759118,-0.0559459),(-0.00694594,0.759118,-0.385456),(0.0736039,0.80253,-0.117023),(0.0736039,0.80253,-0.117023),(-0.00694594,0.759118,-0.385456),(-0.057362,0.807112,-0.500199),(-0.00694594,0.759118,-0.385456),(-0.116889,0.759118,-0.677321),(-0.057362,0.807112,-0.500199),(-0.116889,0.759118,-0.677321),(-0.150477,0.80253,-0.769925),(-0.057362,0.807112,-0.500199),(-0.190475,0.838984,-0.67732),(-0.150477,0.80253,-0.769925),(-0.190477,1.14936,-1.07553),(-0.116889,0.759118,-0.677321),(-0.158301,0.759118,-0.853747),(-0.150477,0.80253,-0.769925),(-0.150477,0.80253,-0.769925),(-0.158301,0.759118,-0.853747),(-0.190477,1.14936,-1.07553),(-0.158301,0.759118,-0.853747),(-0.190476,0.759118,-0.891886),(-0.190477,1.14936,-1.07553),(0.125653,0.794038,0.27759),(0.0736039,0.80253,-0.117023),(0.107334,0.838984,0.27759),(0.107334,0.838984,0.27759),(0.0736039,0.80253,-0.117023),(0.0380947,0.855105,-0.143959),(0.0736039,0.80253,-0.117023),(-0.057362,0.807112,-0.500199),(0.0380947,0.855105,-0.143959),(0.0380947,0.855105,-0.143959),(-0.057362,0.807112,-0.500199),(-0.0943522,0.855105,-0.490288),(-0.057362,0.807112,-0.500199),(-0.150477,0.80253,-0.769925),(-0.0943522,0.855105,-0.490288),(-0.150477,0.80253,-0.769925),(-0.190475,0.838984,-0.67732),(-0.0943522,0.855105,-0.490288),(0.107334,0.838984,0.27759),(0.0380947,0.855105,-0.143959),(0.070541,0.878917,0.27759),(0.070541,0.878917,0.27759),(0.0380947,0.855105,-0.143959),(0.00668626,0.889352,-0.0990912),(0.0380947,0.855105,-0.143959),(-0.0943522,0.855105,-0.490288),(0.00668626,0.889352,-0.0990912),(-0.0943522,0.855105,-0.490288),(-0.0992783,0.878917,-0.360714),(0.00668626,0.889352,-0.0990912),(0.00668626,0.889352,-0.0990912),(-0.0992783,0.878917,-0.360714),(-0.00988409,0.898799,-0.0270985),(0.070541,0.878917,0.27759),(0.00668626,0.889352,-0.0990912),(0.0291299,0.898799,0.27759),(0.0291299,0.898799,0.27759),(0.00668626,0.889352,-0.0990912),(-0.00988409,0.898799,-0.0270985),(-0.00304449,0.903097,0.277591),(0.0291299,0.898799,0.27759),(-0.00988409,0.898799,-0.0270985),(-0.00303986,0.759118,0.999996),(0.0291344,0.759118,0.978427),(-0.00303999,0.794038,0.978427),(0.0291344,0.759118,0.978427),(0.0369577,0.80253,0.931031),(-0.00303999,0.794038,0.978427),(0.070545,0.759118,0.878668),(0.0369577,0.80253,0.931031),(0.0291344,0.759118,0.978427),(0.070545,0.759118,0.878668),(0.0853976,0.807112,0.759194),(0.0369577,0.80253,0.931031),(0.107336,0.759118,0.678308),(0.0853976,0.807112,0.759194),(0.070545,0.759118,0.878668),(0.107336,0.759118,0.678308),(0.116951,0.80253,0.495402),(0.0853976,0.807112,0.759194),(0.107336,0.759118,0.678308),(0.125654,0.759118,0.452799),(0.116951,0.80253,0.495402),(0.125654,0.759118,0.452799),(0.125653,0.794038,0.27759),(0.116951,0.80253,0.495402),(0.125654,0.759118,0.452799),(0.129614,0.759118,0.27759),(0.125653,0.794038,0.27759),(0.0369577,0.80253,0.931031),(-0.00304063,0.838984,0.878668),(-0.00303999,0.794038,0.978427),(0.0369577,0.80253,0.931031),(0.041178,0.855105,0.759194),(-0.00304063,0.838984,0.878668),(0.0853976,0.807112,0.759194),(0.041178,0.855105,0.759194),(0.0369577,0.80253,0.931031),(0.0853976,0.807112,0.759194),(0.085396,0.855105,0.518391),(0.041178,0.855105,0.759194),(0.0853976,0.807112,0.759194),(0.116951,0.80253,0.495402),(0.085396,0.855105,0.518391),(0.116951,0.80253,0.495402),(0.107334,0.838984,0.27759),(0.085396,0.855105,0.518391),(0.116951,0.80253,0.495402),(0.125653,0.794038,0.27759),(0.107334,0.838984,0.27759),(-0.00304063,0.838984,0.878668),(0.041178,0.855105,0.759194),(-0.00304191,0.878917,0.678309),(0.041178,0.855105,0.759194),(0.0369549,0.889352,0.495403),(-0.00304191,0.878917,0.678309),(0.041178,0.855105,0.759194),(0.085396,0.855105,0.518391),(0.0369549,0.889352,0.495403),(0.085396,0.855105,0.518391),(0.070541,0.878917,0.27759),(0.0369549,0.889352,0.495403),(0.085396,0.855105,0.518391),(0.107334,0.838984,0.27759),(0.070541,0.878917,0.27759),(0.0369549,0.889352,0.495403),(-0.00304335,0.898799,0.4528),(-0.00304191,0.878917,0.678309),(0.0369549,0.889352,0.495403),(0.0291299,0.898799,0.27759),(-0.00304335,0.898799,0.4528),(0.0369549,0.889352,0.495403),(0.070541,0.878917,0.27759),(0.0291299,0.898799,0.27759),(-0.00304335,0.898799,0.4528),(0.0291299,0.898799,0.27759),(-0.00304448,0.903097,0.277591),(-0.0943522,0.855105,-0.490288),(-0.190475,0.838984,-0.67732),(-0.138991,0.85895,-0.508919),(-0.0943522,0.855105,-0.490288),(-0.138991,0.85895,-0.508919),(-0.0992783,0.878917,-0.360714),(0.0380947,0.717949,-0.143959),(-0.057362,0.738534,-0.500199),(0.00812078,0.739516,-0.308611),(0.0380947,0.717949,-0.143959),(0.00812078,0.739516,-0.308611),(0.0736039,0.740499,-0.117023),(-0.0943522,0.717949,-0.490288),(-0.138991,0.7163,-0.508919),(-0.190475,0.724864,-0.67732),(-0.00303241,0.934544,0.0504887),(-0.00987154,0.898799,-0.0271019),(-0.00303347,0.900948,0.122661),(-0.00303241,0.934544,0.0504887),(-0.00303347,0.900948,0.122661),(-0.0030355,0.903097,0.277591),(-0.0751188,0.707736,-0.363868),(-0.104904,0.7163,-0.513381),(-0.0618506,0.717949,-0.49202),(1.05985E-6,0.934617,0.0504887),(-0.00550002,0.898872,-0.0278231),(0.0,0.901021,0.122661),(1.05985E-6,0.934617,0.0504887),(0.0,0.901021,0.122661),(-2.03005E-6,0.90317,0.277591),(1.06986E-6,1.0793,3.47475E-7),(1.05985E-6,0.934617,0.0504887),(-2.03005E-6,0.90317,0.277591),(-0.0953993,0.901562,-0.465759),(-0.0983945,0.85895,-0.480795),(-0.0815976,0.874364,-0.39646),(-0.0984809,0.948969,-0.481228),(-0.0953993,0.901562,-0.465759),(-0.0815976,0.874364,-0.39646),(-0.123067,0.693025,-0.604632),(-0.127848,0.652374,-0.628639),(-0.116352,0.684348,-0.570923),(-0.104855,0.716322,-0.51321),(-0.123067,0.693025,-0.604632),(-0.116352,0.684348,-0.570923),(-0.126153,0.723428,-0.620127),(-0.123067,0.693025,-0.604632),(-0.104855,0.716322,-0.51321),(-0.119917,0.739931,-0.0758791),(-0.138579,0.734624,-0.215737),(-0.136681,0.693107,-0.144328),(-0.138579,0.734624,-0.215737),(-0.145559,0.690461,-0.21199),(-0.136681,0.693107,-0.144328),(-0.136681,0.693107,-0.144328),(-0.145559,0.690461,-0.21199),(-0.160399,0.648327,-0.247802),(-0.118464,0.756954,0.588829),(-0.134253,0.756954,0.279345),(-0.302092,0.706351,0.270908),(-0.14043,0.724864,-0.686497),(-0.221371,0.740499,-0.746696),(-0.273867,0.65237,-0.917619),(-0.221371,0.740499,-0.746696),(-0.14043,0.724864,-0.686497),(-0.136467,0.717949,-0.477156),(-0.221371,0.740499,-0.746696),(-0.136467,0.717949,-0.477156),(-0.173783,0.738534,-0.469721),(-0.173783,0.738534,-0.469721),(-0.136467,0.717949,-0.477156),(-0.102195,0.717949,-0.110339),(-0.136467,0.717949,-0.477156),(-0.0751037,0.707736,-0.363871),(-0.0544815,0.70326,-0.0841721),(-0.0751037,0.707736,-0.363871),(-0.00806173,0.699209,-0.0273128),(-0.0544815,0.70326,-0.0841721),(-0.136467,0.717949,-0.477156),(-0.0544814,0.70326,-0.0841721),(-0.102195,0.717949,-0.110339),(-0.102195,0.717949,-0.110339),(-0.0544815,0.70326,-0.0841721),(-0.0766211,0.707736,0.27759),(-0.113414,0.724864,0.27759),(-0.102195,0.717949,-0.110339),(-0.0766211,0.707736,0.27759),(-0.0544815,0.70326,-0.0841721),(-0.00806173,0.699209,-0.0273128),(-0.0352099,0.699209,0.27759),(-0.00806173,0.699209,-0.0273128),(-0.0030355,0.697365,0.277591),(-0.0352099,0.699209,0.27759),(-0.0544815,0.70326,-0.0841721),(-0.0352099,0.699209,0.27759),(-0.0766211,0.707736,0.27759),(-0.00303552,0.697365,0.277591),(-0.00303664,0.699209,0.4528),(-0.0352099,0.699209,0.27759),(-0.131733,0.744141,0.27759),(-0.113414,0.724864,0.27759),(-0.123031,0.740499,0.495402),(-0.123031,0.740499,0.495402),(-0.113414,0.724864,0.27759),(-0.091476,0.717949,0.518391),(-0.113414,0.724864,0.27759),(-0.0766211,0.707736,0.27759),(-0.091476,0.717949,0.518391),(-0.00303936,0.724864,0.878668),(-0.047258,0.717949,0.759194),(-0.00303808,0.707736,0.678309),(-0.0766211,0.707736,0.27759),(-0.0430349,0.70326,0.495403),(-0.091476,0.717949,0.518391),(-0.0352099,0.699209,0.27759),(-0.0430349,0.70326,0.495403),(-0.0766211,0.707736,0.27759),(-0.0430349,0.70326,0.495403),(-0.047258,0.717949,0.759194),(-0.091476,0.717949,0.518391),(-0.00303808,0.707736,0.678309),(-0.047258,0.717949,0.759194),(-0.0430349,0.70326,0.495403),(-0.00303664,0.699209,0.4528),(-0.00303808,0.707736,0.678309),(-0.0430349,0.70326,0.495403),(-0.0352099,0.699209,0.27759),(-0.00303664,0.699209,0.4528),(-0.0430349,0.70326,0.495403),(-0.00303936,0.724864,0.878668),(-0.0430376,0.740499,0.931031),(-0.047258,0.717949,0.759194),(-0.0352143,0.759118,0.978427),(-0.076625,0.759118,0.878668),(-0.0430376,0.740499,0.931031),(-0.0914776,0.738534,0.759194),(-0.113416,0.759118,0.678308),(-0.123031,0.740499,0.495402),(-0.0914776,0.738534,0.759194),(-0.123031,0.740499,0.495402),(-0.091476,0.717949,0.518391),(-0.076625,0.759118,0.878668),(-0.113416,0.759118,0.678308),(-0.0914776,0.738534,0.759194),(-0.0430376,0.740499,0.931031),(-0.076625,0.759118,0.878668),(-0.0914776,0.738534,0.759194),(-0.0430376,0.740499,0.931031),(-0.0914776,0.738534,0.759194),(-0.047258,0.717949,0.759194),(-0.047258,0.717949,0.759194),(-0.0914776,0.738534,0.759194),(-0.091476,0.717949,0.518391),(-0.113416,0.759118,0.678308),(-0.131734,0.759118,0.452799),(-0.123031,0.740499,0.495402),(-0.131734,0.759118,0.452799),(-0.131733,0.744141,0.27759),(-0.123031,0.740499,0.495402),(-0.135694,0.759118,0.27759),(-0.131733,0.744141,0.27759),(-0.131734,0.759118,0.452799),(-0.121989,0.740499,-0.0707252),(-0.173783,0.738534,-0.469721),(-0.147887,0.739516,-0.270224),(-0.168251,0.759118,-0.345318),(-0.173783,0.738534,-0.469721),(-0.121989,0.740499,-0.0707252),(-0.168251,0.759118,-0.345318),(-0.121989,0.740499,-0.0707252),(-0.134706,0.759118,-0.00646544),(-0.121989,0.740499,-0.0707252),(-0.102195,0.717949,-0.110339),(-0.113414,0.724864,0.27759),(-0.134706,0.759118,-0.00646544),(-0.121989,0.740499,-0.0707252),(-0.131733,0.744141,0.27759),(-0.131733,0.744141,0.27759),(-0.121989,0.740499,-0.0707252),(-0.113414,0.724864,0.27759),(-0.135694,0.759118,0.27759),(-0.134706,0.759118,-0.00646544),(-0.131733,0.744141,0.27759),(-0.168251,0.759118,-0.345318),(-0.204157,0.759118,-0.649704),(-0.173783,0.738534,-0.469721),(-0.204157,0.759118,-0.649704),(-0.221371,0.740499,-0.746696),(-0.173783,0.738534,-0.469721),(-0.256504,0.759118,-0.823197),(-0.221371,0.740499,-0.746696),(-0.204157,0.759118,-0.649704),(-0.256504,0.759118,-0.823197),(-0.273867,0.65237,-0.917619),(-0.221371,0.740499,-0.746696),(-0.256504,0.759118,-0.823197),(-0.247712,0.759118,-0.872317),(-0.273867,0.65237,-0.917619),(-0.00304013,0.759118,0.999996),(-0.0352143,0.759118,0.978427),(-0.00304,0.744141,0.978427),(-0.00304,0.744141,0.978427),(-0.0430376,0.740499,0.931031),(-0.00303936,0.724864,0.878668),(-0.00304,0.744141,0.978427),(-0.0352143,0.759118,0.978427),(-0.0430376,0.740499,0.931031),(-0.131733,0.794038,0.27759),(-0.134706,0.759118,-0.00646544),(-0.135694,0.759118,0.27759),(-0.121989,0.80253,-0.0707252),(-0.134706,0.759118,-0.00646544),(-0.131733,0.794038,0.27759),(-0.121989,0.80253,-0.0707252),(-0.168251,0.759118,-0.345318),(-0.134706,0.759118,-0.00646544),(-0.173783,0.807112,-0.469721),(-0.168251,0.759118,-0.345318),(-0.121989,0.80253,-0.0707252),(-0.173783,0.807112,-0.469721),(-0.204157,0.759118,-0.649704),(-0.168251,0.759118,-0.345318),(-0.173783,0.807112,-0.469721),(-0.221371,0.80253,-0.746696),(-0.204157,0.759118,-0.649704),(-0.339529,1.14936,-1.03135),(-0.221371,0.80253,-0.746696),(-0.14043,0.838984,-0.686497),(-0.221371,0.80253,-0.746696),(-0.256506,0.759118,-0.823201),(-0.204157,0.759118,-0.649704),(-0.339529,1.14936,-1.03135),(-0.256506,0.759118,-0.823201),(-0.221371,0.80253,-0.746696),(-0.339529,1.14936,-1.03135),(-0.247712,0.759118,-0.872317),(-0.256506,0.759118,-0.823201),(-0.113414,0.838984,0.27759),(-0.121989,0.80253,-0.0707252),(-0.131733,0.794038,0.27759),(-0.102195,0.855105,-0.110339),(-0.121989,0.80253,-0.0707252),(-0.113414,0.838984,0.27759),(-0.102195,0.855105,-0.110339),(-0.173783,0.807112,-0.469721),(-0.121989,0.80253,-0.0707252),(-0.136467,0.855105,-0.477156),(-0.173783,0.807112,-0.469721),(-0.102195,0.855105,-0.110339),(-0.136467,0.855105,-0.477156),(-0.221371,0.80253,-0.746696),(-0.173783,0.807112,-0.469721),(-0.136467,0.855105,-0.477156),(-0.14043,0.838984,-0.686497),(-0.221371,0.80253,-0.746696),(-0.076621,0.878917,0.27759),(-0.102195,0.855105,-0.110339),(-0.113414,0.838984,0.27759),(-0.0544815,0.889352,-0.0841721),(-0.102195,0.855105,-0.110339),(-0.076621,0.878917,0.27759),(-0.0544815,0.889352,-0.0841721),(-0.136467,0.855105,-0.477156),(-0.102195,0.855105,-0.110339),(-0.0544815,0.889352,-0.0841721),(-0.0751037,0.878917,-0.363871),(-0.136467,0.855105,-0.477156),(-0.00806173,0.898799,-0.0273128),(-0.0751037,0.878917,-0.363871),(-0.0544815,0.889352,-0.0841721),(-0.0352099,0.898799,0.27759),(-0.0544815,0.889352,-0.0841721),(-0.076621,0.878917,0.27759),(-0.00806173,0.898799,-0.0273128),(-0.0544815,0.889352,-0.0841721),(-0.0352099,0.898799,0.27759),(-0.00806173,0.898799,-0.0273128),(-0.0352099,0.898799,0.27759),(-0.0030355,0.903097,0.277591),(-0.00303999,0.794038,0.978427),(-0.0352143,0.759118,0.978427),(-0.00304013,0.759118,0.999996),(-0.00303999,0.794038,0.978427),(-0.0430376,0.80253,0.931031),(-0.0352143,0.759118,0.978427),(-0.0352143,0.759118,0.978427),(-0.0430376,0.80253,0.931031),(-0.076625,0.759118,0.878668),(-0.0430376,0.80253,0.931031),(-0.0914776,0.807112,0.759194),(-0.076625,0.759118,0.878668),(-0.076625,0.759118,0.878668),(-0.0914776,0.807112,0.759194),(-0.113416,0.759118,0.678308),(-0.0914776,0.807112,0.759194),(-0.123031,0.80253,0.495402),(-0.113416,0.759118,0.678308),(-0.123031,0.80253,0.495402),(-0.131734,0.759118,0.452799),(-0.113416,0.759118,0.678308),(-0.123031,0.80253,0.495402),(-0.131733,0.794038,0.27759),(-0.131734,0.759118,0.452799),(-0.131733,0.794038,0.27759),(-0.135694,0.759118,0.27759),(-0.131734,0.759118,0.452799),(-0.00303999,0.794038,0.978427),(-0.00303936,0.838984,0.878668),(-0.0430376,0.80253,0.931031),(-0.00303936,0.838984,0.878668),(-0.047258,0.855105,0.759194),(-0.0430376,0.80253,0.931031),(-0.0430376,0.80253,0.931031),(-0.047258,0.855105,0.759194),(-0.0914776,0.807112,0.759194),(-0.047258,0.855105,0.759194),(-0.091476,0.855105,0.518391),(-0.0914776,0.807112,0.759194),(-0.091476,0.855105,0.518391),(-0.123031,0.80253,0.495402),(-0.0914776,0.807112,0.759194),(-0.091476,0.855105,0.518391),(-0.113414,0.838984,0.27759),(-0.123031,0.80253,0.495402),(-0.113414,0.838984,0.27759),(-0.131733,0.794038,0.27759),(-0.123031,0.80253,0.495402),(-0.00303808,0.878917,0.678309),(-0.047258,0.855105,0.759194),(-0.00303936,0.838984,0.878668),(-0.00303808,0.878917,0.678309),(-0.0430349,0.889352,0.495403),(-0.047258,0.855105,0.759194),(-0.0430349,0.889352,0.495403),(-0.091476,0.855105,0.518391),(-0.047258,0.855105,0.759194),(-0.0430349,0.889352,0.495403),(-0.076621,0.878917,0.27759),(-0.091476,0.855105,0.518391),(-0.076621,0.878917,0.27759),(-0.113414,0.838984,0.27759),(-0.091476,0.855105,0.518391),(-0.00303808,0.878917,0.678309),(-0.00303664,0.898799,0.4528),(-0.0430349,0.889352,0.495403),(-0.00303664,0.898799,0.4528),(-0.0352099,0.898799,0.27759),(-0.0430349,0.889352,0.495403),(-0.0352099,0.898799,0.27759),(-0.076621,0.878917,0.27759),(-0.0430349,0.889352,0.495403),(-0.00303551,0.903097,0.277591),(-0.0352099,0.898799,0.27759),(-0.00303664,0.898799,0.4528),(-0.104886,0.85895,-0.513384),(-0.14043,0.838984,-0.686497),(-0.136467,0.855105,-0.477156),(-0.0751037,0.878917,-0.363871),(-0.104886,0.85895,-0.513384),(-0.136467,0.855105,-0.477156),(-0.147887,0.739516,-0.270224),(-0.173783,0.738534,-0.469721),(-0.102195,0.717949,-0.110339),(-0.121989,0.740499,-0.0707252),(-0.147887,0.739516,-0.270224),(-0.102195,0.717949,-0.110339),(-0.14043,0.724864,-0.686497),(-0.104886,0.7163,-0.513384),(-0.136467,0.717949,-0.477156),(-0.136467,0.717949,-0.477156),(-0.104886,0.7163,-0.513384),(-0.0751037,0.707736,-0.363871),(0.0658878,0.693107,-0.18468),(0.0402763,0.734624,-0.251366),(0.0766327,0.739931,-0.115033),(0.0658878,0.693107,-0.18468),(0.0481599,0.690461,-0.25058),(0.0402763,0.734624,-0.251366),(0.0481443,0.648327,-0.289345),(0.0481599,0.690461,-0.25058),(0.0658878,0.693107,-0.18468),(0.296012,0.706351,0.270908),(0.128173,0.756954,0.279345),(0.112384,0.756954,0.588829),(-0.273889,0.65237,-0.917606),(-0.152113,0.740499,-0.786683),(-0.14045,0.724864,-0.686485),(-0.0618506,0.717949,-0.49202),(-0.14045,0.724864,-0.686485),(-0.152113,0.740499,-0.786683),(-0.0245343,0.738534,-0.499453),(-0.0618506,0.717949,-0.49202),(-0.152113,0.740499,-0.786683),(0.0470589,0.717949,-0.140071),(-0.0618506,0.717949,-0.49202),(-0.0245343,0.738534,-0.499453),(0.0130148,0.70326,-0.0976177),(-0.0751188,0.707736,-0.363868),(-0.0618506,0.717949,-0.49202),(0.0130148,0.70326,-0.0976177),(-0.00807265,0.699209,-0.0273106),(-0.0751188,0.707736,-0.363868),(0.0470589,0.717949,-0.140071),(0.0130148,0.70326,-0.0976177),(-0.0618506,0.717949,-0.49202),(0.0705411,0.707736,0.27759),(0.0130148,0.70326,-0.0976177),(0.0470589,0.717949,-0.140071),(0.0705411,0.707736,0.27759),(0.0470589,0.717949,-0.140071),(0.107334,0.724864,0.27759),(0.0291299,0.699209,0.27759),(-0.00807265,0.699209,-0.0273106),(0.0130148,0.70326,-0.0976177),(0.0291299,0.699209,0.27759),(-0.00304449,0.697365,0.277591),(-0.00807265,0.699209,-0.0273106),(0.0705411,0.707736,0.27759),(0.0291299,0.699209,0.27759),(0.0130148,0.70326,-0.0976177),(0.0291299,0.699209,0.27759),(-0.00304335,0.699209,0.4528),(-0.00304447,0.697365,0.277591),(0.116951,0.740499,0.495402),(0.107334,0.724864,0.27759),(0.125653,0.744141,0.27759),(0.085396,0.717949,0.518391),(0.107334,0.724864,0.27759),(0.116951,0.740499,0.495402),(0.085396,0.717949,0.518391),(0.0705411,0.707736,0.27759),(0.107334,0.724864,0.27759),(-0.00304191,0.707736,0.678309),(0.041178,0.717949,0.759194),(-0.00304063,0.724864,0.878668),(0.085396,0.717949,0.518391),(0.0369549,0.70326,0.495403),(0.0705411,0.707736,0.27759),(0.0705411,0.707736,0.27759),(0.0369549,0.70326,0.495403),(0.0291299,0.699209,0.27759),(0.085396,0.717949,0.518391),(0.041178,0.717949,0.759194),(0.0369549,0.70326,0.495403),(0.0369549,0.70326,0.495403),(0.041178,0.717949,0.759194),(-0.00304191,0.707736,0.678309),(0.0369549,0.70326,0.495403),(-0.00304191,0.707736,0.678309),(-0.00304335,0.699209,0.4528),(0.0369549,0.70326,0.495403),(-0.00304335,0.699209,0.4528),(0.0291299,0.699209,0.27759),(0.041178,0.717949,0.759194),(0.0369577,0.740499,0.931031),(-0.00304063,0.724864,0.878668),(0.0369577,0.740499,0.931031),(0.070545,0.759118,0.878668),(0.0291344,0.759118,0.978427),(0.116951,0.740499,0.495402),(0.107336,0.759118,0.678308),(0.0853976,0.738534,0.759194),(0.085396,0.717949,0.518391),(0.116951,0.740499,0.495402),(0.0853976,0.738534,0.759194),(0.0853976,0.738534,0.759194),(0.107336,0.759118,0.678308),(0.070545,0.759118,0.878668),(0.0853976,0.738534,0.759194),(0.070545,0.759118,0.878668),(0.0369577,0.740499,0.931031),(0.041178,0.717949,0.759194),(0.0853976,0.738534,0.759194),(0.0369577,0.740499,0.931031),(0.085396,0.717949,0.518391),(0.0853976,0.738534,0.759194),(0.041178,0.717949,0.759194),(0.116951,0.740499,0.495402),(0.125654,0.759118,0.452799),(0.107336,0.759118,0.678308),(0.116951,0.740499,0.495402),(0.125653,0.744141,0.27759),(0.125654,0.759118,0.452799),(0.125654,0.759118,0.452799),(0.125653,0.744141,0.27759),(0.129614,0.759118,0.27759),(0.0279941,0.739516,-0.30526),(-0.0245343,0.738534,-0.499453),(0.0805227,0.740499,-0.111067),(0.0805227,0.740499,-0.111067),(-0.0245343,0.738534,-0.499453),(0.0180279,0.759118,-0.382425),(0.100534,0.759118,-0.0489462),(0.0805227,0.740499,-0.111067),(0.0180279,0.759118,-0.382425),(0.107334,0.724864,0.27759),(0.0470589,0.717949,-0.140071),(0.0805227,0.740499,-0.111067),(0.125653,0.744141,0.27759),(0.0805227,0.740499,-0.111067),(0.100534,0.759118,-0.0489462),(0.107334,0.724864,0.27759),(0.0805227,0.740499,-0.111067),(0.125653,0.744141,0.27759),(0.125653,0.744141,0.27759),(0.100534,0.759118,-0.0489462),(0.129614,0.759118,0.27759),(-0.0245343,0.738534,-0.499453),(-0.076723,0.759118,-0.723279),(0.0180279,0.759118,-0.382425),(-0.0245343,0.738534,-0.499453),(-0.152113,0.740499,-0.786683),(-0.076723,0.759118,-0.723279),(-0.076723,0.759118,-0.723279),(-0.152113,0.740499,-0.786683),(-0.200798,0.759118,-0.85536),(-0.152113,0.740499,-0.786683),(-0.273889,0.65237,-0.917606),(-0.200798,0.759118,-0.85536),(-0.273889,0.65237,-0.917606),(-0.247734,0.759118,-0.872304),(-0.200798,0.759118,-0.85536),(-0.00303999,0.744141,0.978427),(0.0291344,0.759118,0.978427),(-0.00303985,0.759118,0.999996),(-0.00304063,0.724864,0.878668),(0.0369577,0.740499,0.931031),(-0.00303999,0.744141,0.978427),(0.0369577,0.740499,0.931031),(0.0291344,0.759118,0.978427),(-0.00303999,0.744141,0.978427),(0.129614,0.759118,0.27759),(0.100534,0.759118,-0.0489462),(0.125653,0.794038,0.27759),(0.125653,0.794038,0.27759),(0.100534,0.759118,-0.0489462),(0.0805227,0.80253,-0.111067),(0.100534,0.759118,-0.0489462),(0.0180279,0.759118,-0.382425),(0.0805227,0.80253,-0.111067),(0.0805227,0.80253,-0.111067),(0.0180279,0.759118,-0.382425),(-0.0245343,0.807112,-0.499453),(0.0180279,0.759118,-0.382425),(-0.076723,0.759118,-0.723279),(-0.0245343,0.807112,-0.499453),(-0.076723,0.759118,-0.723279),(-0.152113,0.80253,-0.786683),(-0.0245343,0.807112,-0.499453),(-0.14045,0.838984,-0.686485),(-0.152113,0.80253,-0.786683),(-0.339557,1.14936,-1.03135),(-0.076723,0.759118,-0.723279),(-0.2008,0.759118,-0.855362),(-0.152113,0.80253,-0.786683),(-0.152113,0.80253,-0.786683),(-0.2008,0.759118,-0.855362),(-0.339557,1.14936,-1.03135),(-0.2008,0.759118,-0.855362),(-0.247734,0.759118,-0.872304),(-0.339557,1.14936,-1.03135),(0.125653,0.794038,0.27759),(0.0805227,0.80253,-0.111067),(0.107334,0.838984,0.27759),(0.107334,0.838984,0.27759),(0.0805227,0.80253,-0.111067),(0.0470589,0.855105,-0.140071),(0.0805227,0.80253,-0.111067),(-0.0245343,0.807112,-0.499453),(0.0470589,0.855105,-0.140071),(0.0470589,0.855105,-0.140071),(-0.0245343,0.807112,-0.499453),(-0.0618506,0.855105,-0.49202),(-0.0245343,0.807112,-0.499453),(-0.152113,0.80253,-0.786683),(-0.0618506,0.855105,-0.49202),(-0.152113,0.80253,-0.786683),(-0.14045,0.838984,-0.686485),(-0.0618506,0.855105,-0.49202),(0.107334,0.838984,0.27759),(0.0470589,0.855105,-0.140071),(0.070541,0.878917,0.27759),(0.070541,0.878917,0.27759),(0.0470589,0.855105,-0.140071),(0.0130148,0.889352,-0.0976177),(0.0470589,0.855105,-0.140071),(-0.0618506,0.855105,-0.49202),(0.0130148,0.889352,-0.0976177),(-0.0618506,0.855105,-0.49202),(-0.0751188,0.878917,-0.363868),(0.0130148,0.889352,-0.0976177),(0.0130148,0.889352,-0.0976177),(-0.0751188,0.878917,-0.363868),(-0.00807265,0.898799,-0.0273106),(0.070541,0.878917,0.27759),(0.0130148,0.889352,-0.0976177),(0.0291299,0.898799,0.27759),(0.0291299,0.898799,0.27759),(0.0130148,0.889352,-0.0976177),(-0.00807265,0.898799,-0.0273106),(-0.00304449,0.903097,0.277591),(0.0291299,0.898799,0.27759),(-0.00807265,0.898799,-0.0273106),(-0.00303986,0.759118,0.999996),(0.0291344,0.759118,0.978427),(-0.00303999,0.794038,0.978427),(0.0291344,0.759118,0.978427),(0.0369577,0.80253,0.931031),(-0.00303999,0.794038,0.978427),(0.070545,0.759118,0.878668),(0.0369577,0.80253,0.931031),(0.0291344,0.759118,0.978427),(0.070545,0.759118,0.878668),(0.0853976,0.807112,0.759194),(0.0369577,0.80253,0.931031),(0.107336,0.759118,0.678308),(0.0853976,0.807112,0.759194),(0.070545,0.759118,0.878668),(0.107336,0.759118,0.678308),(0.116951,0.80253,0.495402),(0.0853976,0.807112,0.759194),(0.107336,0.759118,0.678308),(0.125654,0.759118,0.452799),(0.116951,0.80253,0.495402),(0.125654,0.759118,0.452799),(0.125653,0.794038,0.27759),(0.116951,0.80253,0.495402),(0.125654,0.759118,0.452799),(0.129614,0.759118,0.27759),(0.125653,0.794038,0.27759),(0.0369577,0.80253,0.931031),(-0.00304063,0.838984,0.878668),(-0.00303999,0.794038,0.978427),(0.0369577,0.80253,0.931031),(0.041178,0.855105,0.759194),(-0.00304063,0.838984,0.878668),(0.0853976,0.807112,0.759194),(0.041178,0.855105,0.759194),(0.0369577,0.80253,0.931031),(0.0853976,0.807112,0.759194),(0.085396,0.855105,0.518391),(0.041178,0.855105,0.759194),(0.0853976,0.807112,0.759194),(0.116951,0.80253,0.495402),(0.085396,0.855105,0.518391),(0.116951,0.80253,0.495402),(0.107334,0.838984,0.27759),(0.085396,0.855105,0.518391),(0.116951,0.80253,0.495402),(0.125653,0.794038,0.27759),(0.107334,0.838984,0.27759),(-0.00304063,0.838984,0.878668),(0.041178,0.855105,0.759194),(-0.00304191,0.878917,0.678309),(0.041178,0.855105,0.759194),(0.0369549,0.889352,0.495403),(-0.00304191,0.878917,0.678309),(0.041178,0.855105,0.759194),(0.085396,0.855105,0.518391),(0.0369549,0.889352,0.495403),(0.085396,0.855105,0.518391),(0.070541,0.878917,0.27759),(0.0369549,0.889352,0.495403),(0.085396,0.855105,0.518391),(0.107334,0.838984,0.27759),(0.070541,0.878917,0.27759),(0.0369549,0.889352,0.495403),(-0.00304335,0.898799,0.4528),(-0.00304191,0.878917,0.678309),(0.0369549,0.889352,0.495403),(0.0291299,0.898799,0.27759),(-0.00304335,0.898799,0.4528),(0.0369549,0.889352,0.495403),(0.070541,0.878917,0.27759),(0.0291299,0.898799,0.27759),(-0.00304335,0.898799,0.4528),(0.0291299,0.898799,0.27759),(-0.00304448,0.903097,0.277591),(-0.0618506,0.855105,-0.49202),(-0.14045,0.838984,-0.686485),(-0.104904,0.85895,-0.513381),(-0.0618506,0.855105,-0.49202),(-0.104904,0.85895,-0.513381),(-0.0751188,0.878917,-0.363868),(0.0470589,0.717949,-0.140071),(-0.0245343,0.738534,-0.499453),(0.0279941,0.739516,-0.30526),(0.0470589,0.717949,-0.140071),(0.0279941,0.739516,-0.30526),(0.0805227,0.740499,-0.111067),(-0.0618506,0.717949,-0.49202),(-0.104904,0.7163,-0.513381),(-0.14045,0.724864,-0.686485),(-0.00303241,0.934544,0.0504887),(-0.00805998,0.898799,-0.0273131),(-0.00303347,0.900948,0.122661),(-0.00303241,0.934544,0.0504887),(-0.00303347,0.900948,0.122661),(-0.0030355,0.903097,0.277591),(-0.00264051,0.707736,-0.373331),(-0.0026415,0.7163,-0.526765),(0.035654,0.717949,-0.497215),(1.05985E-6,0.934617,0.0504887),(1.80003E-6,0.898872,-0.0279467),(0.0,0.901021,0.122661),(1.05985E-6,0.934617,0.0504887),(0.0,0.901021,0.122661),(-2.03005E-6,0.90317,0.277591),(1.06986E-6,1.0793,3.47475E-7),(1.05985E-6,0.934617,0.0504887),(-2.03005E-6,0.90317,0.277591),(-0.00262399,0.901562,-0.477891),(-0.00262401,0.85895,-0.493321),(-0.00262701,0.874364,-0.406775),(-0.002624,0.948969,-0.493766),(-0.00262399,0.901562,-0.477891),(-0.00262701,0.874364,-0.406775),(-0.002627,0.693025,-0.620407),(-0.00262649,0.652374,-0.645043),(-0.00262702,0.684348,-0.585814),(-0.00262751,0.716322,-0.526587),(-0.002627,0.693025,-0.620407),(-0.00262702,0.684348,-0.585814),(-0.002627,0.723428,-0.636308),(-0.002627,0.693025,-0.620407),(-0.00262751,0.716322,-0.526587),(-0.103485,0.739931,-0.0978782),(-0.0944054,0.734624,-0.239595),(-0.106573,0.693107,-0.168737),(-0.0944054,0.734624,-0.239595),(-0.102032,0.690461,-0.237269),(-0.106573,0.693107,-0.168737),(-0.106573,0.693107,-0.168737),(-0.102032,0.690461,-0.237269),(-0.109639,0.648327,-0.275536),(-0.118464,0.756954,0.588829),(-0.134253,0.756954,0.279345),(-0.302092,0.706351,0.270908),(-0.00783302,0.724864,-0.700279),(-0.0478305,0.740499,-0.792884),(-0.00783149,0.65237,-0.967155),(-0.0478305,0.740499,-0.792884),(-0.00783302,0.724864,-0.700279),(-0.0409195,0.717949,-0.497215),(-0.0478305,0.740499,-0.792884),(-0.0409195,0.717949,-0.497215),(-0.0792145,0.738534,-0.497214),(-0.0792145,0.738534,-0.497214),(-0.0409195,0.717949,-0.497215),(-0.079217,0.717949,-0.128407),(-0.0409195,0.717949,-0.497215),(-0.00262501,0.707736,-0.373331),(-0.037266,0.70326,-0.0931975),(-0.00262501,0.707736,-0.373331),(-0.0026271,0.699209,-0.0279467),(-0.037266,0.70326,-0.0931975),(-0.0409195,0.717949,-0.497215),(-0.0372659,0.70326,-0.0931975),(-0.079217,0.717949,-0.128407),(-0.079217,0.717949,-0.128407),(-0.037266,0.70326,-0.0931975),(-0.0766211,0.707736,0.27759),(-0.113414,0.724864,0.27759),(-0.079217,0.717949,-0.128407),(-0.0766211,0.707736,0.27759),(-0.037266,0.70326,-0.0931975),(-0.0026271,0.699209,-0.0279467),(-0.0352099,0.699209,0.27759),(-0.0026271,0.699209,-0.0279467),(-0.0030355,0.697365,0.277591),(-0.0352099,0.699209,0.27759),(-0.037266,0.70326,-0.0931975),(-0.0352099,0.699209,0.27759),(-0.0766211,0.707736,0.27759),(-0.00303552,0.697365,0.277591),(-0.00303664,0.699209,0.4528),(-0.0352099,0.699209,0.27759),(-0.131733,0.744141,0.27759),(-0.113414,0.724864,0.27759),(-0.123031,0.740499,0.495402),(-0.123031,0.740499,0.495402),(-0.113414,0.724864,0.27759),(-0.091476,0.717949,0.518391),(-0.113414,0.724864,0.27759),(-0.0766211,0.707736,0.27759),(-0.091476,0.717949,0.518391),(-0.00303936,0.724864,0.878668),(-0.047258,0.717949,0.759194),(-0.00303808,0.707736,0.678309),(-0.0766211,0.707736,0.27759),(-0.0430349,0.70326,0.495403),(-0.091476,0.717949,0.518391),(-0.0352099,0.699209,0.27759),(-0.0430349,0.70326,0.495403),(-0.0766211,0.707736,0.27759),(-0.0430349,0.70326,0.495403),(-0.047258,0.717949,0.759194),(-0.091476,0.717949,0.518391),(-0.00303808,0.707736,0.678309),(-0.047258,0.717949,0.759194),(-0.0430349,0.70326,0.495403),(-0.00303664,0.699209,0.4528),(-0.00303808,0.707736,0.678309),(-0.0430349,0.70326,0.495403),(-0.0352099,0.699209,0.27759),(-0.00303664,0.699209,0.4528),(-0.0430349,0.70326,0.495403),(-0.00303936,0.724864,0.878668),(-0.0430376,0.740499,0.931031),(-0.047258,0.717949,0.759194),(-0.0352143,0.759118,0.978427),(-0.076625,0.759118,0.878668),(-0.0430376,0.740499,0.931031),(-0.0914776,0.738534,0.759194),(-0.113416,0.759118,0.678308),(-0.123031,0.740499,0.495402),(-0.0914776,0.738534,0.759194),(-0.123031,0.740499,0.495402),(-0.091476,0.717949,0.518391),(-0.076625,0.759118,0.878668),(-0.113416,0.759118,0.678308),(-0.0914776,0.738534,0.759194),(-0.0430376,0.740499,0.931031),(-0.076625,0.759118,0.878668),(-0.0914776,0.738534,0.759194),(-0.0430376,0.740499,0.931031),(-0.0914776,0.738534,0.759194),(-0.047258,0.717949,0.759194),(-0.047258,0.717949,0.759194),(-0.0914776,0.738534,0.759194),(-0.091476,0.717949,0.518391),(-0.113416,0.759118,0.678308),(-0.131734,0.759118,0.452799),(-0.123031,0.740499,0.495402),(-0.131734,0.759118,0.452799),(-0.131733,0.744141,0.27759),(-0.123031,0.740499,0.495402),(-0.135694,0.759118,0.27759),(-0.131733,0.744141,0.27759),(-0.131734,0.759118,0.452799),(-0.106544,0.740499,-0.0931984),(-0.0792145,0.738534,-0.497214),(-0.0928796,0.739516,-0.295207),(-0.098215,0.759118,-0.373333),(-0.0792145,0.738534,-0.497214),(-0.106544,0.740499,-0.0931984),(-0.098215,0.759118,-0.373333),(-0.106544,0.740499,-0.0931984),(-0.131731,0.759118,-0.0322708),(-0.106544,0.740499,-0.0931984),(-0.079217,0.717949,-0.128407),(-0.113414,0.724864,0.27759),(-0.131731,0.759118,-0.0322708),(-0.106544,0.740499,-0.0931984),(-0.131733,0.744141,0.27759),(-0.131733,0.744141,0.27759),(-0.106544,0.740499,-0.0931984),(-0.113414,0.724864,0.27759),(-0.135694,0.759118,0.27759),(-0.131731,0.759118,-0.0322708),(-0.131733,0.744141,0.27759),(-0.098215,0.759118,-0.373333),(-0.081419,0.759118,-0.700279),(-0.0792145,0.738534,-0.497214),(-0.081419,0.759118,-0.700279),(-0.0478305,0.740499,-0.792884),(-0.0792145,0.738534,-0.497214),(-0.040006,0.759118,-0.876702),(-0.0478305,0.740499,-0.792884),(-0.081419,0.759118,-0.700279),(-0.040006,0.759118,-0.876702),(-0.00783149,0.65237,-0.967155),(-0.0478305,0.740499,-0.792884),(-0.040006,0.759118,-0.876702),(-0.007832,0.759118,-0.914845),(-0.00783149,0.65237,-0.967155),(-0.00304013,0.759118,0.999996),(-0.0352143,0.759118,0.978427),(-0.00304,0.744141,0.978427),(-0.00304,0.744141,0.978427),(-0.0430376,0.740499,0.931031),(-0.00303936,0.724864,0.878668),(-0.00304,0.744141,0.978427),(-0.0352143,0.759118,0.978427),(-0.0430376,0.740499,0.931031),(-0.131733,0.794038,0.27759),(-0.131731,0.759118,-0.0322708),(-0.135694,0.759118,0.27759),(-0.106544,0.80253,-0.0931984),(-0.131731,0.759118,-0.0322708),(-0.131733,0.794038,0.27759),(-0.106544,0.80253,-0.0931984),(-0.098215,0.759118,-0.373333),(-0.131731,0.759118,-0.0322708),(-0.0792145,0.807112,-0.497214),(-0.098215,0.759118,-0.373333),(-0.106544,0.80253,-0.0931984),(-0.0792145,0.807112,-0.497214),(-0.081419,0.759118,-0.700279),(-0.098215,0.759118,-0.373333),(-0.0792145,0.807112,-0.497214),(-0.0478305,0.80253,-0.792884),(-0.081419,0.759118,-0.700279),(-0.00783102,1.14936,-1.09848),(-0.0478305,0.80253,-0.792884),(-0.00783302,0.838984,-0.700279),(-0.0478305,0.80253,-0.792884),(-0.040006,0.759118,-0.876706),(-0.081419,0.759118,-0.700279),(-0.00783102,1.14936,-1.09848),(-0.040006,0.759118,-0.876706),(-0.0478305,0.80253,-0.792884),(-0.00783102,1.14936,-1.09848),(-0.007832,0.759118,-0.914845),(-0.040006,0.759118,-0.876706),(-0.113414,0.838984,0.27759),(-0.106544,0.80253,-0.0931984),(-0.131733,0.794038,0.27759),(-0.079217,0.855105,-0.128407),(-0.106544,0.80253,-0.0931984),(-0.113414,0.838984,0.27759),(-0.079217,0.855105,-0.128407),(-0.0792145,0.807112,-0.497214),(-0.106544,0.80253,-0.0931984),(-0.0409195,0.855105,-0.497215),(-0.0792145,0.807112,-0.497214),(-0.079217,0.855105,-0.128407),(-0.0409195,0.855105,-0.497215),(-0.0478305,0.80253,-0.792884),(-0.0792145,0.807112,-0.497214),(-0.0409195,0.855105,-0.497215),(-0.00783302,0.838984,-0.700279),(-0.0478305,0.80253,-0.792884),(-0.076621,0.878917,0.27759),(-0.079217,0.855105,-0.128407),(-0.113414,0.838984,0.27759),(-0.037266,0.889352,-0.0931975),(-0.079217,0.855105,-0.128407),(-0.076621,0.878917,0.27759),(-0.037266,0.889352,-0.0931975),(-0.0409195,0.855105,-0.497215),(-0.079217,0.855105,-0.128407),(-0.037266,0.889352,-0.0931975),(-0.00262501,0.878917,-0.373331),(-0.0409195,0.855105,-0.497215),(-0.0026271,0.898799,-0.0279467),(-0.00262501,0.878917,-0.373331),(-0.037266,0.889352,-0.0931975),(-0.0352099,0.898799,0.27759),(-0.037266,0.889352,-0.0931975),(-0.076621,0.878917,0.27759),(-0.0026271,0.898799,-0.0279467),(-0.037266,0.889352,-0.0931975),(-0.0352099,0.898799,0.27759),(-0.0026271,0.898799,-0.0279467),(-0.0352099,0.898799,0.27759),(-0.0030355,0.903097,0.277591),(-0.00303999,0.794038,0.978427),(-0.0352143,0.759118,0.978427),(-0.00304013,0.759118,0.999996),(-0.00303999,0.794038,0.978427),(-0.0430376,0.80253,0.931031),(-0.0352143,0.759118,0.978427),(-0.0352143,0.759118,0.978427),(-0.0430376,0.80253,0.931031),(-0.076625,0.759118,0.878668),(-0.0430376,0.80253,0.931031),(-0.0914776,0.807112,0.759194),(-0.076625,0.759118,0.878668),(-0.076625,0.759118,0.878668),(-0.0914776,0.807112,0.759194),(-0.113416,0.759118,0.678308),(-0.0914776,0.807112,0.759194),(-0.123031,0.80253,0.495402),(-0.113416,0.759118,0.678308),(-0.123031,0.80253,0.495402),(-0.131734,0.759118,0.452799),(-0.113416,0.759118,0.678308),(-0.123031,0.80253,0.495402),(-0.131733,0.794038,0.27759),(-0.131734,0.759118,0.452799),(-0.131733,0.794038,0.27759),(-0.135694,0.759118,0.27759),(-0.131734,0.759118,0.452799),(-0.00303999,0.794038,0.978427),(-0.00303936,0.838984,0.878668),(-0.0430376,0.80253,0.931031),(-0.00303936,0.838984,0.878668),(-0.047258,0.855105,0.759194),(-0.0430376,0.80253,0.931031),(-0.0430376,0.80253,0.931031),(-0.047258,0.855105,0.759194),(-0.0914776,0.807112,0.759194),(-0.047258,0.855105,0.759194),(-0.091476,0.855105,0.518391),(-0.0914776,0.807112,0.759194),(-0.091476,0.855105,0.518391),(-0.123031,0.80253,0.495402),(-0.0914776,0.807112,0.759194),(-0.091476,0.855105,0.518391),(-0.113414,0.838984,0.27759),(-0.123031,0.80253,0.495402),(-0.113414,0.838984,0.27759),(-0.131733,0.794038,0.27759),(-0.123031,0.80253,0.495402),(-0.00303808,0.878917,0.678309),(-0.047258,0.855105,0.759194),(-0.00303936,0.838984,0.878668),(-0.00303808,0.878917,0.678309),(-0.0430349,0.889352,0.495403),(-0.047258,0.855105,0.759194),(-0.0430349,0.889352,0.495403),(-0.091476,0.855105,0.518391),(-0.047258,0.855105,0.759194),(-0.0430349,0.889352,0.495403),(-0.076621,0.878917,0.27759),(-0.091476,0.855105,0.518391),(-0.076621,0.878917,0.27759),(-0.113414,0.838984,0.27759),(-0.091476,0.855105,0.518391),(-0.00303808,0.878917,0.678309),(-0.00303664,0.898799,0.4528),(-0.0430349,0.889352,0.495403),(-0.00303664,0.898799,0.4528),(-0.0352099,0.898799,0.27759),(-0.0430349,0.889352,0.495403),(-0.0352099,0.898799,0.27759),(-0.076621,0.878917,0.27759),(-0.0430349,0.889352,0.495403),(-0.00303551,0.903097,0.277591),(-0.0352099,0.898799,0.27759),(-0.00303664,0.898799,0.4528),(-0.002624,0.85895,-0.526765),(-0.00783302,0.838984,-0.700279),(-0.0409195,0.855105,-0.497215),(-0.00262501,0.878917,-0.373331),(-0.002624,0.85895,-0.526765),(-0.0409195,0.855105,-0.497215),(-0.0928796,0.739516,-0.295207),(-0.0792145,0.738534,-0.497214),(-0.079217,0.717949,-0.128407),(-0.106544,0.740499,-0.0931984),(-0.0928796,0.739516,-0.295207),(-0.079217,0.717949,-0.128407),(-0.00783302,0.724864,-0.700279),(-0.002624,0.7163,-0.526765),(-0.0409195,0.717949,-0.497215),(-0.0409195,0.717949,-0.497215),(-0.002624,0.7163,-0.526765),(-0.00262501,0.707736,-0.373331),(0.101308,0.693107,-0.168737),(0.0891401,0.734624,-0.239595),(0.0982192,0.739931,-0.0978782),(0.101308,0.693107,-0.168737),(0.0967671,0.690461,-0.237269),(0.0891401,0.734624,-0.239595),(0.104374,0.648327,-0.275536),(0.0967671,0.690461,-0.237269),(0.101308,0.693107,-0.168737),(0.296012,0.706351,0.270908),(0.128173,0.756954,0.279345),(0.112384,0.756954,0.588829),(-0.0078575,0.65237,-0.967155),(0.032142,0.740499,-0.792884),(-0.00785602,0.724864,-0.700279),(0.035654,0.717949,-0.497215),(-0.00785602,0.724864,-0.700279),(0.032142,0.740499,-0.792884),(0.073949,0.738534,-0.497215),(0.035654,0.717949,-0.497215),(0.032142,0.740499,-0.792884),(0.0739513,0.717949,-0.128407),(0.035654,0.717949,-0.497215),(0.073949,0.738534,-0.497215),(0.0320005,0.70326,-0.0931975),(-0.00264051,0.707736,-0.373331),(0.035654,0.717949,-0.497215),(0.0320005,0.70326,-0.0931975),(-0.0026383,0.699209,-0.0279467),(-0.00264051,0.707736,-0.373331),(0.0739513,0.717949,-0.128407),(0.0320005,0.70326,-0.0931975),(0.035654,0.717949,-0.497215),(0.0705411,0.707736,0.27759),(0.0320005,0.70326,-0.0931975),(0.0739513,0.717949,-0.128407),(0.0705411,0.707736,0.27759),(0.0739513,0.717949,-0.128407),(0.107334,0.724864,0.27759),(0.0291299,0.699209,0.27759),(-0.0026383,0.699209,-0.0279467),(0.0320005,0.70326,-0.0931975),(0.0291299,0.699209,0.27759),(-0.00304449,0.697365,0.277591),(-0.0026383,0.699209,-0.0279467),(0.0705411,0.707736,0.27759),(0.0291299,0.699209,0.27759),(0.0320005,0.70326,-0.0931975),(0.0291299,0.699209,0.27759),(-0.00304335,0.699209,0.4528),(-0.00304447,0.697365,0.277591),(0.116951,0.740499,0.495402),(0.107334,0.724864,0.27759),(0.125653,0.744141,0.27759),(0.085396,0.717949,0.518391),(0.107334,0.724864,0.27759),(0.116951,0.740499,0.495402),(0.085396,0.717949,0.518391),(0.0705411,0.707736,0.27759),(0.107334,0.724864,0.27759),(-0.00304191,0.707736,0.678309),(0.041178,0.717949,0.759194),(-0.00304063,0.724864,0.878668),(0.085396,0.717949,0.518391),(0.0369549,0.70326,0.495403),(0.0705411,0.707736,0.27759),(0.0705411,0.707736,0.27759),(0.0369549,0.70326,0.495403),(0.0291299,0.699209,0.27759),(0.085396,0.717949,0.518391),(0.041178,0.717949,0.759194),(0.0369549,0.70326,0.495403),(0.0369549,0.70326,0.495403),(0.041178,0.717949,0.759194),(-0.00304191,0.707736,0.678309),(0.0369549,0.70326,0.495403),(-0.00304191,0.707736,0.678309),(-0.00304335,0.699209,0.4528),(0.0369549,0.70326,0.495403),(-0.00304335,0.699209,0.4528),(0.0291299,0.699209,0.27759),(0.041178,0.717949,0.759194),(0.0369577,0.740499,0.931031),(-0.00304063,0.724864,0.878668),(0.0369577,0.740499,0.931031),(0.070545,0.759118,0.878668),(0.0291344,0.759118,0.978427),(0.116951,0.740499,0.495402),(0.107336,0.759118,0.678308),(0.0853976,0.738534,0.759194),(0.085396,0.717949,0.518391),(0.116951,0.740499,0.495402),(0.0853976,0.738534,0.759194),(0.0853976,0.738534,0.759194),(0.107336,0.759118,0.678308),(0.070545,0.759118,0.878668),(0.0853976,0.738534,0.759194),(0.070545,0.759118,0.878668),(0.0369577,0.740499,0.931031),(0.041178,0.717949,0.759194),(0.0853976,0.738534,0.759194),(0.0369577,0.740499,0.931031),(0.085396,0.717949,0.518391),(0.0853976,0.738534,0.759194),(0.041178,0.717949,0.759194),(0.116951,0.740499,0.495402),(0.125654,0.759118,0.452799),(0.107336,0.759118,0.678308),(0.116951,0.740499,0.495402),(0.125653,0.744141,0.27759),(0.125654,0.759118,0.452799),(0.125654,0.759118,0.452799),(0.125653,0.744141,0.27759),(0.129614,0.759118,0.27759),(0.0876139,0.739516,-0.295207),(0.073949,0.738534,-0.497215),(0.101279,0.740499,-0.0931984),(0.101279,0.740499,-0.0931984),(0.073949,0.738534,-0.497215),(0.0929495,0.759118,-0.373333),(0.108817,0.759118,-0.0279473),(0.101279,0.740499,-0.0931984),(0.0929495,0.759118,-0.373333),(0.107334,0.724864,0.27759),(0.0739513,0.717949,-0.128407),(0.101279,0.740499,-0.0931984),(0.125653,0.744141,0.27759),(0.101279,0.740499,-0.0931984),(0.108817,0.759118,-0.0279473),(0.107334,0.724864,0.27759),(0.101279,0.740499,-0.0931984),(0.125653,0.744141,0.27759),(0.125653,0.744141,0.27759),(0.108817,0.759118,-0.0279473),(0.129614,0.759118,0.27759),(0.073949,0.738534,-0.497215),(0.06573,0.759118,-0.70028),(0.0929495,0.759118,-0.373333),(0.073949,0.738534,-0.497215),(0.032142,0.740499,-0.792884),(0.06573,0.759118,-0.70028),(0.06573,0.759118,-0.70028),(0.032142,0.740499,-0.792884),(0.024318,0.759118,-0.876703),(0.032142,0.740499,-0.792884),(-0.0078575,0.65237,-0.967155),(0.024318,0.759118,-0.876703),(-0.0078575,0.65237,-0.967155),(-0.00785699,0.759118,-0.914845),(0.024318,0.759118,-0.876703),(-0.00303999,0.744141,0.978427),(0.0291344,0.759118,0.978427),(-0.00303985,0.759118,0.999996),(-0.00304063,0.724864,0.878668),(0.0369577,0.740499,0.931031),(-0.00303999,0.744141,0.978427),(0.0369577,0.740499,0.931031),(0.0291344,0.759118,0.978427),(-0.00303999,0.744141,0.978427),(0.129614,0.759118,0.27759),(0.108817,0.759118,-0.0279473),(0.125653,0.794038,0.27759),(0.125653,0.794038,0.27759),(0.108817,0.759118,-0.0279473),(0.101279,0.80253,-0.0931984),(0.108817,0.759118,-0.0279473),(0.0929495,0.759118,-0.373333),(0.101279,0.80253,-0.0931984),(0.101279,0.80253,-0.0931984),(0.0929495,0.759118,-0.373333),(0.073949,0.807112,-0.497215),(0.0929495,0.759118,-0.373333),(0.06573,0.759118,-0.70028),(0.073949,0.807112,-0.497215),(0.06573,0.759118,-0.70028),(0.032142,0.80253,-0.792884),(0.073949,0.807112,-0.497215),(-0.00785602,0.838984,-0.700279),(0.032142,0.80253,-0.792884),(-0.007858,1.14936,-1.09849),(0.06573,0.759118,-0.70028),(0.0243175,0.759118,-0.876706),(0.032142,0.80253,-0.792884),(0.032142,0.80253,-0.792884),(0.0243175,0.759118,-0.876706),(-0.007858,1.14936,-1.09849),(0.0243175,0.759118,-0.876706),(-0.00785699,0.759118,-0.914845),(-0.007858,1.14936,-1.09849),(0.125653,0.794038,0.27759),(0.101279,0.80253,-0.0931984),(0.107334,0.838984,0.27759),(0.107334,0.838984,0.27759),(0.101279,0.80253,-0.0931984),(0.0739513,0.855105,-0.128407),(0.101279,0.80253,-0.0931984),(0.073949,0.807112,-0.497215),(0.0739513,0.855105,-0.128407),(0.0739513,0.855105,-0.128407),(0.073949,0.807112,-0.497215),(0.035654,0.855105,-0.497215),(0.073949,0.807112,-0.497215),(0.032142,0.80253,-0.792884),(0.035654,0.855105,-0.497215),(0.032142,0.80253,-0.792884),(-0.00785602,0.838984,-0.700279),(0.035654,0.855105,-0.497215),(0.107334,0.838984,0.27759),(0.0739513,0.855105,-0.128407),(0.070541,0.878917,0.27759),(0.070541,0.878917,0.27759),(0.0739513,0.855105,-0.128407),(0.0320005,0.889352,-0.0931975),(0.0739513,0.855105,-0.128407),(0.035654,0.855105,-0.497215),(0.0320005,0.889352,-0.0931975),(0.035654,0.855105,-0.497215),(-0.00264051,0.878917,-0.373331),(0.0320005,0.889352,-0.0931975),(0.0320005,0.889352,-0.0931975),(-0.00264051,0.878917,-0.373331),(-0.0026383,0.898799,-0.0279467),(0.070541,0.878917,0.27759),(0.0320005,0.889352,-0.0931975),(0.0291299,0.898799,0.27759),(0.0291299,0.898799,0.27759),(0.0320005,0.889352,-0.0931975),(-0.0026383,0.898799,-0.0279467),(-0.00304449,0.903097,0.277591),(0.0291299,0.898799,0.27759),(-0.0026383,0.898799,-0.0279467),(-0.00303986,0.759118,0.999996),(0.0291344,0.759118,0.978427),(-0.00303999,0.794038,0.978427),(0.0291344,0.759118,0.978427),(0.0369577,0.80253,0.931031),(-0.00303999,0.794038,0.978427),(0.070545,0.759118,0.878668),(0.0369577,0.80253,0.931031),(0.0291344,0.759118,0.978427),(0.070545,0.759118,0.878668),(0.0853976,0.807112,0.759194),(0.0369577,0.80253,0.931031),(0.107336,0.759118,0.678308),(0.0853976,0.807112,0.759194),(0.070545,0.759118,0.878668),(0.107336,0.759118,0.678308),(0.116951,0.80253,0.495402),(0.0853976,0.807112,0.759194),(0.107336,0.759118,0.678308),(0.125654,0.759118,0.452799),(0.116951,0.80253,0.495402),(0.125654,0.759118,0.452799),(0.125653,0.794038,0.27759),(0.116951,0.80253,0.495402),(0.125654,0.759118,0.452799),(0.129614,0.759118,0.27759),(0.125653,0.794038,0.27759),(0.0369577,0.80253,0.931031),(-0.00304063,0.838984,0.878668),(-0.00303999,0.794038,0.978427),(0.0369577,0.80253,0.931031),(0.041178,0.855105,0.759194),(-0.00304063,0.838984,0.878668),(0.0853976,0.807112,0.759194),(0.041178,0.855105,0.759194),(0.0369577,0.80253,0.931031),(0.0853976,0.807112,0.759194),(0.085396,0.855105,0.518391),(0.041178,0.855105,0.759194),(0.0853976,0.807112,0.759194),(0.116951,0.80253,0.495402),(0.085396,0.855105,0.518391),(0.116951,0.80253,0.495402),(0.107334,0.838984,0.27759),(0.085396,0.855105,0.518391),(0.116951,0.80253,0.495402),(0.125653,0.794038,0.27759),(0.107334,0.838984,0.27759),(-0.00304063,0.838984,0.878668),(0.041178,0.855105,0.759194),(-0.00304191,0.878917,0.678309),(0.041178,0.855105,0.759194),(0.0369549,0.889352,0.495403),(-0.00304191,0.878917,0.678309),(0.041178,0.855105,0.759194),(0.085396,0.855105,0.518391),(0.0369549,0.889352,0.495403),(0.085396,0.855105,0.518391),(0.070541,0.878917,0.27759),(0.0369549,0.889352,0.495403),(0.085396,0.855105,0.518391),(0.107334,0.838984,0.27759),(0.070541,0.878917,0.27759),(0.0369549,0.889352,0.495403),(-0.00304335,0.898799,0.4528),(-0.00304191,0.878917,0.678309),(0.0369549,0.889352,0.495403),(0.0291299,0.898799,0.27759),(-0.00304335,0.898799,0.4528),(0.0369549,0.889352,0.495403),(0.070541,0.878917,0.27759),(0.0291299,0.898799,0.27759),(-0.00304335,0.898799,0.4528),(0.0291299,0.898799,0.27759),(-0.00304448,0.903097,0.277591),(0.035654,0.855105,-0.497215),(-0.00785602,0.838984,-0.700279),(-0.0026415,0.85895,-0.526765),(0.035654,0.855105,-0.497215),(-0.0026415,0.85895,-0.526765),(-0.00264051,0.878917,-0.373331),(0.0739513,0.717949,-0.128407),(0.073949,0.738534,-0.497215),(0.0876139,0.739516,-0.295207),(0.0739513,0.717949,-0.128407),(0.0876139,0.739516,-0.295207),(0.101279,0.740499,-0.0931984),(0.035654,0.717949,-0.497215),(-0.0026415,0.7163,-0.526765),(-0.00785602,0.724864,-0.700279),(-0.00303241,0.934544,0.0504887),(-0.0026253,0.898799,-0.0279467),(-0.00303347,0.900948,0.122661),(-0.00303241,0.934544,0.0504887),(-0.00303347,0.900948,0.122661),(-0.0030355,0.903097,0.277591)]),
      TimeSensor(DEF='Clock',cycleInterval=4,loop=True),
      ROUTE(fromField='fraction_changed',fromNode='Clock',toField='set_fraction',toNode='MeshInterpolator'),
      ROUTE(fromField='value_changed',fromNode='MeshInterpolator',toField='set_point',toNode='_1')])])
) # X3D model complete

####################################################################################################
# Self-test diagnostics
####################################################################################################

print('Self-test diagnostics for SharkLefty.py:')
if        metaDiagnostics(newModel): # built-in utility method in X3D class
    print(metaDiagnostics(newModel)) # display meta info, hint, warning, error, TODO values in this model
# print('check newModel.XML() serialization...')
newModelXML= newModel.XML() # test export method XML() for exceptions during export
newModel.XMLvalidate()
# print(newModelXML) # diagnostic

try:
#   print('check newModel.VRML() serialization...')
    newModelVRML=newModel.VRML() # test export method VRML() for exceptions during export
    # print(prependLineNumbers(newModelVRML)) # debug
    print("Python-to-VRML export of VRML output successful", flush=True)
except Exception as err: # usually BaseException
    # https://stackoverflow.com/questions/18176602/how-to-get-the-name-of-an-exception-that-was-caught-in-python
    print("*** Python-to-VRML export of VRML output failed:", type(err).__name__, err)
    if newModelVRML: # may have failed to generate
        print(prependLineNumbers(newModelVRML, err.lineno))

try:
#   print('check newModel.JSON() serialization...')
    newModelJSON=newModel.JSON() # test export method JSON() for exceptions during export
#   print(prependLineNumbers(newModelJSON)) # debug
    print("Python-to-JSON export of JSON output successful (under development)")
except Exception as err: # usually SyntaxError
    print("*** Python-to-JSON export of JSON output failed:", type(err).__name__, err)
    if newModelJSON: # may have failed to generate
        print(prependLineNumbers(newModelJSON,err.lineno))

print("python SharkLefty.py load and self-test diagnostics complete.")
